@@ -131,7 +131,7 @@ void sync(const SyncCTX &ctx)
         selectiveSyncFixup(db, selectiveSyncList);
     }
 
-    SyncOptions opt { QSharedPointer<Vfs>(createVfsFromPlugin(Vfs::Off).release()) };
+    SyncOptions opt { QSharedPointer<Vfs>(VfsPluginManager::instance().createVfsFromPlugin(Vfs::Off).release()) };
     opt.fillFromEnvironmentVariables();
     opt.verifyChunkSizes();
     auto engine = new SyncEngine(
@@ -162,24 +162,22 @@ void sync(const SyncCTX &ctx)
             abort(false);
         } else {
             if (dir == SyncFileItem::Down) {
-                std::cout << "All files in the sync folder '" << qPrintable(ctx.options.remoteFolder) << "' folder were deleted on the server.\n"
-                          << "These deletes will be synchronized to your local sync folder, making such files "
-                          << "unavailable unless you have a right to restore. \n"
-                          << "If you decide to keep the files, they will be re-synced with the server if you have rights to do so.\n"
-                          << "If you decide to delete the files, they will be unavailable to you, unless you are the owner."
-                          << std::endl;
+                qInfo() << "All files in the sync folder '" << ctx.options.remoteFolder << "' folder were deleted on the server.";
+                qInfo() << "These deletes will be synchronized to your local sync folder, making such files "
+                        << "unavailable unless you have a right to restore.";
+                qInfo() << "If you decide to keep the files, they will be re-synced with the server if you have rights to do so.";
+                qInfo() << "If you decide to delete the files, they will be unavailable to you, unless you are the owner.";
 
 
             } else {
-                std::cout << "All the files in your local sync folder '" << qPrintable(ctx.options.source_dir) << "' were deleted. These deletes will be "
-                          << "synchronized with your server, making such files unavailable unless restored.\n"
-                          << "Are you sure you want to sync those actions with the server?\n"
-                          << "If this was an accident and you decide to keep your files, they will be re-synced from the server."
-                          << std::endl;
+                qInfo() << "All the files in your local sync folder '" << ctx.options.source_dir << "' were deleted. These deletes will be "
+                        << "synchronized with your server, making such files unavailable unless restored.";
+                qInfo() << "Are you sure you want to sync those actions with the server?";
+                qInfo() << "If this was an accident and you decide to keep your files, they will be re-synced from the server.";
             }
             std::string s;
             while (true) {
-                std::cout << "Remove all files?[y,n]";
+                qInfo() << "Remove all files? [y,n]";
                 std::getline(std::cin, s);
                 if (s == "y") {
                     abort(false);
@@ -213,7 +211,8 @@ void sync(const SyncCTX &ctx)
     }
 
     if (!engine->excludedFiles().reloadExcludeFiles()) {
-        qFatal("Cannot load system exclude list or list supplied via --exclude");
+        qCritical() << "Cannot load system exclude list or list supplied via --exclude";
+        qApp->exit(EXIT_FAILURE);
     }
     engine->startSync();
 }
@@ -262,13 +261,15 @@ void setupCredentials(SyncCTX &ctx)
 
             port = pList.at(2).toUInt(&ok);
             if (!ok || port > std::numeric_limits<uint16_t>::max()) {
-                qFatal("Invalid port number");
+                qCritical() << "Invalid port number";
+                qApp->exit(EXIT_FAILURE);
             }
 
             QNetworkProxyFactory::setUseSystemConfiguration(false);
             QNetworkProxy::setApplicationProxy(QNetworkProxy(QNetworkProxy::HttpProxy, host, static_cast<uint16_t>(port)));
         } else {
-            qFatal("Could not read httpproxy. The proxy should have the format \"http://hostname:port\".");
+            qCritical() << "Could not read httpproxy. The proxy should have the format \"http://hostname:port\".";
+            qApp->exit(EXIT_FAILURE);
         }
     }
 
@@ -278,9 +279,8 @@ void setupCredentials(SyncCTX &ctx)
         // also fail
         QFile f(ctx.options.unsyncedfolders);
         if (!f.open(QFile::ReadOnly)) {
-            qFatal("Cannot read unsyncedfolders file '%s': %s",
-                qPrintable(ctx.options.unsyncedfolders),
-                qPrintable(f.errorString()));
+            qCritical() << "Cannot read unsyncedfolders file '" << ctx.options.unsyncedfolders << "': " << f.errorString();
+            qApp->exit(EXIT_FAILURE);
         }
         f.close();
     }
@@ -298,7 +298,8 @@ void setupCredentials(SyncCTX &ctx)
             for (const auto &e : errors) {
                 qCritical() << e.errorString();
             }
-            qFatal("If you trust the certificate and want to ignore the errors, use the --trust option.");
+            qCritical() << "If you trust the certificate and want to ignore the errors, use the --trust option.";
+            qApp->exit(EXIT_FAILURE);
         });
     }
 }
@@ -316,7 +317,7 @@ CmdOptions parseOptions(const QStringList &app_args)
         return option;
     };
 
-    auto silentOption = addOption({ { QStringLiteral("s"), QStringLiteral("silten") }, QStringLiteral("Don't be so verbose.") });
+    auto silentOption = addOption({ { QStringLiteral("s"), QStringLiteral("silent") }, QStringLiteral("Don't be so verbose.") });
     auto httpproxyOption = addOption({ { QStringLiteral("httpproxy") }, QStringLiteral("Specify a http proxy to use."), QStringLiteral("http://server:port") });
     auto trustOption = addOption({ { QStringLiteral("trust") }, QStringLiteral("Trust the SSL certification") });
     auto excludeOption = addOption({ { QStringLiteral("exclude") }, QStringLiteral("Path to an exclude list [file]"), QStringLiteral("file") });
@@ -331,6 +332,7 @@ CmdOptions parseOptions(const QStringList &app_args)
     auto maxRetriesOption = addOption({ { QStringLiteral("max-sync-retries") }, QStringLiteral("Retries maximum n times (default to 3)"), QStringLiteral("n") });
     auto uploadLimitOption = addOption({ { QStringLiteral("uplimit") }, QStringLiteral("Limit the upload speed of files to n KB/s"), QStringLiteral("n") });
     auto downloadLimitption = addOption({ { QStringLiteral("downlimit") }, QStringLiteral("Limit the download speed of files to n KB/s"), QStringLiteral("n") });
+    auto syncHiddenFilesOption = addOption({ { QStringLiteral("sync-hidden-files") }, QStringLiteral("Enables synchronization of hidden files") });
 
     auto logdebugOption = addOption({ { QStringLiteral("logdebug") }, QStringLiteral("More verbose logging") });
 
@@ -347,14 +349,14 @@ CmdOptions parseOptions(const QStringList &app_args)
     const QStringList args = parser.positionalArguments();
     if (args.size() < 2 || args.size() > 3) {
         parser.showHelp();
-        exit(1);
+        qApp->exit(EXIT_FAILURE);
     }
 
     options.source_dir = [arg = args[0]] {
         QFileInfo fi(arg);
         if (!fi.exists()) {
-            std::cerr << "Source dir '" << qPrintable(arg) << "' does not exist." << std::endl;
-            exit(1);
+            qCritical() << "Source dir '" << arg << "' does not exist.";
+            qApp->exit(EXIT_FAILURE);
         }
         QString sourceDir = fi.absoluteFilePath();
         if (!sourceDir.endsWith(QLatin1Char('/'))) {
@@ -384,6 +386,8 @@ CmdOptions parseOptions(const QStringList &app_args)
     }
     if (parser.isSet(serverOption)) {
         options.server_url = QUrl::fromUserInput(parser.value(serverOption));
+    } else {
+        options.server_url = options.target_url;
     }
     if (parser.isSet(userOption)) {
         options.user = parser.value(userOption);
@@ -406,6 +410,9 @@ CmdOptions parseOptions(const QStringList &app_args)
     if (parser.isSet(downloadLimitption)) {
         options.downlimit = parser.value(downloadLimitption).toInt() * 1000;
     }
+    if (parser.isSet(syncHiddenFilesOption)) {
+        options.ignoreHiddenFiles = false;
+    }
     if (parser.isSet(logdebugOption)) {
         Logger::instance()->setLogFile(QStringLiteral("-"));
         Logger::instance()->setLogDebug(true);
@@ -418,12 +425,12 @@ int main(int argc, char **argv)
     auto platform = OCC::Platform::create();
 
     QCoreApplication app(argc, argv);
-    app.setApplicationVersion(Theme::instance()->versionSwitchOutput());
 
     platform->migrate();
-
     platform->setApplication(&app);
 
+    app.setApplicationVersion(Theme::instance()->versionSwitchOutput());
+
     SyncCTX ctx { parseOptions(app.arguments()) };
 
     if (ctx.options.silent) {
@@ -432,22 +439,15 @@ int main(int argc, char **argv)
         qSetMessagePattern(Logger::loggerPattern());
     }
 
-    ctx.account = Account::create();
+    ctx.account = Account::create(QUuid::createUuid());
 
     if (!ctx.account) {
-        qFatal("Could not initialize account!");
+        qCritical() << "Could not initialize account!";
+        qApp->exit(EXIT_FAILURE);
     }
 
     setupCredentials(ctx);
 
-    if (ctx.options.server_url.isEmpty()) {
-        ctx.options.server_url = ctx.options.target_url;
-        // guess dav path
-        if (!ctx.options.target_url.path().contains(ctx.account->davPath())) {
-            ctx.options.target_url = OCC::Utility::concatUrlPath(ctx.options.target_url, ctx.account->davPath());
-        }
-    }
-
     // don't leak credentials more than needed
     ctx.options.server_url = ctx.options.server_url.adjusted(QUrl::RemoveUserInfo);
     ctx.options.target_url = ctx.options.target_url.adjusted(QUrl::RemoveUserInfo);
@@ -464,13 +464,17 @@ int main(int argc, char **argv)
 
     ctx.account->setUrl(baseUrl);
 
-    auto *checkServerJob = CheckServerJobFactory(ctx.account->accessManager()).startJob(ctx.account->url());
+    auto *checkServerJob = CheckServerJobFactory(ctx.account->accessManager()).startJob(ctx.account->url(), qApp);
 
     QObject::connect(checkServerJob, &CoreJob::finished, [ctx, checkServerJob] {
         if (checkServerJob->success()) {
             // Perform a call to get the capabilities.
             auto *capabilitiesJob = new JsonApiJob(ctx.account, QStringLiteral("ocs/v1.php/cloud/capabilities"), {}, {}, nullptr);
             QObject::connect(capabilitiesJob, &JsonApiJob::finishedSignal, qApp, [capabilitiesJob, ctx] {
+                if (capabilitiesJob->reply()->error() != QNetworkReply::NoError || capabilitiesJob->httpStatusCode() != 200) {
+                    qCritical() << "Error connecting to server";
+                    qApp->exit(EXIT_FAILURE);
+                }
                 auto caps = capabilitiesJob->data().value(QStringLiteral("ocs")).toObject().value(QStringLiteral("data")).toObject().value(QStringLiteral("capabilities")).toObject();
                 qDebug() << "Server capabilities" << caps;
                 ctx.account->setCapabilities(caps.toVariantMap());
@@ -482,19 +486,23 @@ int main(int argc, char **argv)
                     qWarning() << "Failed to detect server version";
                     break;
                 case Account::ServerSupportLevel::Unsupported:
-                    qFatal("Error unsupported server");
-                }
-
-                if (capabilitiesJob->reply()->error() != QNetworkReply::NoError) {
-                    qFatal("Error connecting to server");
+                    qCritical() << "Error unsupported server";
+                    qApp->exit(EXIT_FAILURE);
                 }
 
                 auto userJob = new JsonApiJob(ctx.account, QStringLiteral("ocs/v1.php/cloud/user"), {}, {}, nullptr);
-                QObject::connect(userJob, &JsonApiJob::finishedSignal, qApp, [userJob, ctx] {
+                QObject::connect(userJob, &JsonApiJob::finishedSignal, qApp, [userJob, ctx = ctx]() mutable {
                     const QJsonObject data = userJob->data().value(QStringLiteral("ocs")).toObject().value(QStringLiteral("data")).toObject();
                     ctx.account->setDavUser(data.value(QStringLiteral("id")).toString());
                     ctx.account->setDavDisplayName(data.value(QStringLiteral("display-name")).toString());
 
+                    if (ctx.options.server_url == ctx.options.target_url) {
+                        // guess dav path
+                        if (!ctx.options.target_url.path().contains(ctx.account->davPath())) {
+                            ctx.options.target_url = OCC::Utility::concatUrlPath(ctx.options.target_url, ctx.account->davPath());
+                        }
+                    }
+
                     // much lower age than the default since this utility is usually made to be run right after a change in the tests
                     SyncEngine::minimumFileAgeForUpload = std::chrono::milliseconds(0);
                     sync(ctx);
@@ -503,13 +511,12 @@ int main(int argc, char **argv)
             });
             capabilitiesJob->start();
         } else {
-            switch (checkServerJob->reply()->error()) {
-            case QNetworkReply::OperationCanceledError:
-                qFatal("Looking up %s timed out.", qPrintable(ctx.account->url().toString()));
-                break;
-            default:
-                qFatal("Failed to resolve %s Error: %s.", qPrintable(ctx.account->url().toString()), qPrintable(checkServerJob->reply()->errorString()));
+            if (checkServerJob->reply()->error() == QNetworkReply::OperationCanceledError) {
+                qCritical() << "Looking up " << ctx.account->url().toString() << " timed out.";
+            } else {
+                qCritical() << "Failed to resolve " << ctx.account->url().toString() << " Error: " << checkServerJob->reply()->errorString();
             }
+            qApp->exit(EXIT_FAILURE);
         }
     });
 
