@@ -24,7 +24,9 @@
 #include <QString>
 
 #include "common/asserts.h"
+#include "common/filesystembase.h"
 #include "common/utility.h"
+
 #include "ownsql.h"
 
 #include <sqlite3.h>
@@ -64,7 +66,7 @@ SqlDatabase::~SqlDatabase()
 }
 
 
-bool SqlDatabase::isOpen()
+bool SqlDatabase::isOpen() const
 {
     return _db != nullptr;
 }
@@ -77,7 +79,14 @@ bool SqlDatabase::openHelper(const QString &filename, int sqliteFlags)
 
     sqliteFlags |= SQLITE_OPEN_NOMUTEX;
 
-    SQLITE_DO(sqlite3_open_v2(filename.toUtf8().constData(), &_db, sqliteFlags, nullptr));
+#ifdef Q_OS_WIN
+    // allow file paths > 260
+    // https://www.sqlite.org/vfs.html#standard_windows_vfses
+    char *vfs = "win32-longpath";
+#else
+    char *vfs = nullptr;
+#endif
+    SQLITE_DO(sqlite3_open_v2(FileSystem::longWinPath(filename).toUtf8().constData(), &_db, sqliteFlags, vfs));
 
     if (_errId != SQLITE_OK) {
         qCWarning(lcSql) << "Error:" << _error << "for" << filename;
@@ -264,20 +273,18 @@ int SqlQuery::prepare(const QByteArray &sql, bool allow_failure)
         finish();
     }
     if (!_sql.isEmpty()) {
-        int rc = {};
         for (int n = 0; n < SQLITE_REPEAT_COUNT; ++n) {
             qCDebug(lcSql) << "SQL prepare" << _sql << "Try:" << n;
-            rc = sqlite3_prepare_v2(_db, _sql.constData(), -1, &_stmt, nullptr);
-            if (rc != SQLITE_OK) {
+            _errId = sqlite3_prepare_v2(_db, _sql.constData(), -1, &_stmt, nullptr);
+            if (_errId != SQLITE_OK) {
                 qCWarning(lcSql) << "SQL prepare failed" << _sql << QString::fromUtf8(sqlite3_errmsg(_db));
-                if ((rc == SQLITE_BUSY) || (rc == SQLITE_LOCKED)) {
+                if ((_errId == SQLITE_BUSY) || (_errId == SQLITE_LOCKED)) {
                     std::this_thread::sleep_for(SQLITE_SLEEP_TIME);
                     continue;
                 }
             }
             break;
         }
-        _errId = rc;
 
         if (_errId != SQLITE_OK) {
             _error = QString::fromUtf8(sqlite3_errmsg(_db));
@@ -286,6 +293,13 @@ int SqlQuery::prepare(const QByteArray &sql, bool allow_failure)
         } else {
             OC_ASSERT(_stmt);
             _sqldb->_queries.insert(this);
+
+            if (lcSql().isDebugEnabled()) {
+                _boundValues.resize(sqlite3_bind_parameter_count(_stmt));
+                for (int i = 1; i <= _boundValues.size(); ++i) {
+                    _boundValues[i - 1].name = QString::fromUtf8(sqlite3_bind_parameter_name(_stmt, i));
+                }
+            }
         }
     }
     return _errId;
@@ -316,23 +330,34 @@ bool SqlQuery::exec()
         qCWarning(lcSql) << "Can't exec query, statement unprepared.";
         return false;
     }
-
     // Don't do anything for selects, that is how we use the lib :-|
     if (!isSelect() && !isPragma()) {
-        int rc = 0;
         for (int n = 0; n < SQLITE_REPEAT_COUNT; ++n) {
-            qCDebug(lcSql) << "SQL exec" << _sql << "Try:" << n;
-            rc = sqlite3_step(_stmt);
-            if (rc != SQLITE_DONE || rc != SQLITE_ROW) {
+            if (lcSql().isDebugEnabled()) {
+                if (!_boundValues.isEmpty()) {
+                    // Try to create a pretty printed version of the sql query
+                    // the result can be inaccurate
+                    QString query = QString::fromUtf8(_sql);
+                    for (const auto &v : _boundValues) {
+                        query.replace(query.indexOf(v.name), v.name.size(), v.value);
+                    }
+                    char *actualQuery = sqlite3_expanded_sql(_stmt);
+                    qCDebug(lcSql) << "SQL exec: Estimated query:" << query << "Actual query:" << QString::fromUtf8(actualQuery) << "Try:" << n;
+                    sqlite3_free(actualQuery);
+                } else {
+                    qCDebug(lcSql) << "SQL exec:" << _sql << "Try:" << n;
+                }
+            }
+            _errId = sqlite3_step(_stmt);
+            if (_errId != SQLITE_DONE && _errId != SQLITE_ROW) {
                 qCWarning(lcSql) << "SQL exec failed" << _sql << QString::fromUtf8(sqlite3_errmsg(_db));
-                if (rc == SQLITE_LOCKED || rc == SQLITE_BUSY) {
+                if (_errId == SQLITE_LOCKED || _errId == SQLITE_BUSY) {
                     std::this_thread::sleep_for(SQLITE_SLEEP_TIME);
                     continue;
                 }
-                break;
             }
+            break;
         }
-        _errId = rc;
 
         if (_errId != SQLITE_DONE && _errId != SQLITE_ROW) {
             _error = QString::fromUtf8(sqlite3_errmsg(_db));
