@@ -22,9 +22,10 @@
 #include "plugin.h"
 #include "syncjournaldb.h"
 
+#include <QCoreApplication>
 #include <QDir>
-#include <QPluginLoader>
 #include <QLoggingCategory>
+#include <QPluginLoader>
 
 using namespace OCC;
 
@@ -153,46 +154,58 @@ static QString modeToPluginName(Vfs::Mode mode)
 
 Q_LOGGING_CATEGORY(lcPlugin, "plugins", QtInfoMsg)
 
-bool OCC::isVfsPluginAvailable(Vfs::Mode mode)
+OCC::VfsPluginManager *OCC::VfsPluginManager::_instance = nullptr;
+
+bool OCC::VfsPluginManager::isVfsPluginAvailable(Vfs::Mode mode) const
 {
-    // TODO: cache plugins available?
-    auto name = modeToPluginName(mode);
-    if (name.isEmpty())
-        return false;
-    auto pluginPath = pluginFileName(QStringLiteral("vfs"), name);
-    QPluginLoader loader(pluginPath);
+    {
+        auto result = _pluginCache.constFind(mode);
+        if (result != _pluginCache.cend()) {
+            return *result;
+        }
+    }
+    const bool out = [mode] {
+        const QString name = modeToPluginName(mode);
+        if (!OC_ENSURE_NOT(name.isEmpty())) {
+            return false;
+        }
+        auto pluginPath = pluginFileName(QStringLiteral("vfs"), name);
+        QPluginLoader loader(pluginPath);
 
-    auto basemeta = loader.metaData();
-    if (basemeta.isEmpty() || !basemeta.contains(QStringLiteral("IID"))) {
-        qCDebug(lcPlugin) << "Plugin doesn't exist" << loader.fileName();
-        return false;
-    }
-    if (basemeta[QStringLiteral("IID")].toString() != QLatin1String("org.owncloud.PluginFactory")) {
-        qCWarning(lcPlugin) << "Plugin has wrong IID" << loader.fileName() << basemeta[QStringLiteral("IID")];
-        return false;
-    }
+        auto basemeta = loader.metaData();
+        if (basemeta.isEmpty() || !basemeta.contains(QStringLiteral("IID"))) {
+            qCDebug(lcPlugin) << "Plugin doesn't exist:" << loader.fileName() << "LibraryPath:" << QCoreApplication::libraryPaths();
+            return false;
+        }
+        if (basemeta[QStringLiteral("IID")].toString() != QLatin1String("org.owncloud.PluginFactory")) {
+            qCWarning(lcPlugin) << "Plugin has wrong IID" << loader.fileName() << basemeta[QStringLiteral("IID")];
+            return false;
+        }
 
-    auto metadata = basemeta[QStringLiteral("MetaData")].toObject();
-    if (metadata[QStringLiteral("type")].toString() != QLatin1String("vfs")) {
-        qCWarning(lcPlugin) << "Plugin has wrong type" << loader.fileName() << metadata[QStringLiteral("type")];
-        return false;
-    }
-    if (metadata[QStringLiteral("version")].toString() != OCC::Version::version().toString()) {
-        qCWarning(lcPlugin) << "Plugin has wrong version" << loader.fileName() << metadata[QStringLiteral("version")];
-        return false;
-    }
+        auto metadata = basemeta[QStringLiteral("MetaData")].toObject();
+        if (metadata[QStringLiteral("type")].toString() != QLatin1String("vfs")) {
+            qCWarning(lcPlugin) << "Plugin has wrong type" << loader.fileName() << metadata[QStringLiteral("type")];
+            return false;
+        }
+        if (metadata[QStringLiteral("version")].toString() != OCC::Version::version().toString()) {
+            qCWarning(lcPlugin) << "Plugin has wrong version" << loader.fileName() << metadata[QStringLiteral("version")];
+            return false;
+        }
 
-    // Attempting to load the plugin is essential as it could have dependencies that
-    // can't be resolved and thus not be available after all.
-    if (!loader.load()) {
-        qCWarning(lcPlugin) << "Plugin failed to load:" << loader.errorString();
-        return false;
-    }
+        // Attempting to load the plugin is essential as it could have dependencies that
+        // can't be resolved and thus not be available after all.
+        if (!loader.load()) {
+            qCWarning(lcPlugin) << "Plugin failed to load:" << loader.errorString();
+            return false;
+        }
 
-    return true;
+        return true;
+    }();
+    _pluginCache[mode] = out;
+    return out;
 }
 
-Vfs::Mode OCC::bestAvailableVfsMode()
+Vfs::Mode OCC::VfsPluginManager::bestAvailableVfsMode() const
 {
     if (isVfsPluginAvailable(Vfs::WindowsCfApi)) {
         return Vfs::WindowsCfApi;
@@ -204,7 +217,7 @@ Vfs::Mode OCC::bestAvailableVfsMode()
     Q_UNREACHABLE();
 }
 
-std::unique_ptr<Vfs> OCC::createVfsFromPlugin(Vfs::Mode mode)
+std::unique_ptr<Vfs> OCC::VfsPluginManager::createVfsFromPlugin(Vfs::Mode mode) const
 {
     auto name = modeToPluginName(mode);
     if (name.isEmpty())
@@ -238,3 +251,11 @@ std::unique_ptr<Vfs> OCC::createVfsFromPlugin(Vfs::Mode mode)
     qCInfo(lcPlugin) << "Created VFS instance from plugin" << pluginPath;
     return vfs;
 }
+
+const VfsPluginManager &VfsPluginManager::instance()
+{
+    if (!_instance) {
+        _instance = new VfsPluginManager();
+    }
+    return *_instance;
+}
