@@ -13,20 +13,21 @@
  */
 #pragma once
 
+#include "assert.h"
+#include "csync/csync.h"
+#include "ocsynclib.h"
+#include "pinstate.h"
+#include "result.h"
+#include "syncfilestatus.h"
+
 #include <QObject>
 #include <QScopedPointer>
 #include <QSharedPointer>
+#include <QUrl>
+#include <QVersionNumber>
 
 #include <memory>
 
-#include "assert.h"
-#include "ocsynclib.h"
-#include "result.h"
-#include "syncfilestatus.h"
-#include "pinstate.h"
-
-typedef struct csync_file_stat_s csync_file_stat_t;
-
 namespace OCC {
 
 class Account;
@@ -38,6 +39,12 @@ class SyncFileItem;
 /** Collection of parameters for initializing a Vfs instance. */
 struct OCSYNC_EXPORT VfsSetupParams
 {
+    explicit VfsSetupParams(const AccountPtr &account, const QUrl &baseUrl, bool groupInSidebar)
+        : account(account)
+        , _baseUrl(baseUrl)
+        , _groupInSidebar(groupInSidebar)
+    {
+    }
     /** The full path to the folder on the local filesystem
      *
      * Always ends with /.
@@ -62,12 +69,27 @@ struct OCSYNC_EXPORT VfsSetupParams
     /// Strings potentially passed on to the platform
     QString providerDisplayName;
     QString providerName;
-    QString providerVersion;
+    QVersionNumber providerVersion;
 
     /** when registering with the system we might use
      *  a different presentaton to identify the accounts
      */
     bool multipleAccountsRegistered = false;
+
+    const QUrl &baseUrl() const
+    {
+        return _baseUrl;
+    }
+
+    bool groupInSidebar() const
+    {
+        return _groupInSidebar;
+    }
+
+
+private:
+    QUrl _baseUrl;
+    bool _groupInSidebar = false;
 };
 
 /** Interface describing how to deal with virtual/placeholder files.
@@ -91,15 +113,13 @@ public:
      *
      * Currently plugins and modes are one-to-one but that's not required.
      */
-    enum Mode
-    {
+    enum Mode {
         Off,
         WithSuffix,
-        WindowsCfApi,
+        WindowsCfApi
     };
     Q_ENUM(Mode)
     enum class ConvertToPlaceholderResult {
-        Error,
         Ok,
         Locked
     };
@@ -133,7 +153,7 @@ public:
     virtual QString underlyingFileName(const QString &fileName) const;
 
     /// Access to the parameters the instance was start()ed with.
-    const VfsSetupParams &params() const { return _setupParams; }
+    const VfsSetupParams &params() const { return *_setupParams.get(); }
 
     /** Initializes interaction with the VFS provider.
      *
@@ -155,31 +175,18 @@ public:
      */
     virtual bool socketApiPinStateActionsShown() const = 0;
 
-    /** Return true when download of a file's data is currently ongoing.
-     *
-     * See also the beginHydrating() and doneHydrating() signals.
-     */
-    virtual bool isHydrating() const = 0;
-
     /// Create a new dehydrated placeholder. Called from PropagateDownload.
-    virtual OC_REQUIRED_RESULT Result<void, QString> createPlaceholder(const SyncFileItem &item) = 0;
-
-    /** Convert a hydrated placeholder to a dehydrated one. Called from PropagateDownlaod.
-     *
-     * This is different from delete+create because preserving some file metadata
-     * (like pin states) may be essential for some vfs plugins.
-     */
-    virtual OC_REQUIRED_RESULT Result<void, QString> dehydratePlaceholder(const SyncFileItem &item) = 0;
+    [[nodiscard]] virtual Result<void, QString> createPlaceholder(const SyncFileItem &item) = 0;
 
     /** Discovery hook: even unchanged files may need UPDATE_METADATA.
      *
      * For instance cfapi vfs wants local hydrated non-placeholder files to
      * become hydrated placeholder files.
      */
-    virtual OC_REQUIRED_RESULT bool needsMetadataUpdate(const SyncFileItem &item) = 0;
+    [[nodiscard]] virtual bool needsMetadataUpdate(const SyncFileItem &item) = 0;
 
     /// Determine whether the file at the given absolute path is a dehydrated placeholder.
-    virtual OC_REQUIRED_RESULT bool isDehydratedPlaceholder(const QString &filePath) = 0;
+    [[nodiscard]] virtual bool isDehydratedPlaceholder(const QString &filePath) = 0;
 
     /** Similar to isDehydratedPlaceholder() but used from sync discovery.
      *
@@ -188,7 +195,7 @@ public:
      *
      * Returning true means that type was fully determined.
      */
-    virtual OC_REQUIRED_RESULT bool statTypeVirtualFile(csync_file_stat_t *stat, void *stat_data) = 0;
+    [[nodiscard]] virtual bool statTypeVirtualFile(csync_file_stat_t *stat, void *stat_data) = 0;
 
     /** Sets the pin state for the item at a path.
      *
@@ -197,20 +204,20 @@ public:
      * Usually this would forward to setting the pin state flag in the db table,
      * but some vfs plugins will store the pin state in file attributes instead.
      *
-     * folderPath is relative to the sync folder. Can be "" for root folder.
+     * relFilePath is relative to the sync folder. Can be "" for root folder.
      */
-    virtual OC_REQUIRED_RESULT bool setPinState(const QString &folderPath, PinState state) = 0;
+    [[nodiscard]] virtual bool setPinState(const QString &relFilePath, PinState state) = 0;
 
     /** Returns the pin state of an item at a path.
      *
      * Usually backed by the db's effectivePinState() function but some vfs
      * plugins will override it to retrieve the state from elsewhere.
      *
-     * folderPath is relative to the sync folder. Can be "" for root folder.
+     * relFilePath is relative to the sync folder. Can be "" for root folder.
      *
      * Returns none on retrieval error.
      */
-    virtual OC_REQUIRED_RESULT Optional<PinState> pinState(const QString &folderPath) = 0;
+    [[nodiscard]] virtual Optional<PinState> pinState(const QString &relFilePath) = 0;
 
     /** Returns availability status of an item at a path.
      *
@@ -219,7 +226,7 @@ public:
      *
      * folderPath is relative to the sync folder. Can be "" for root folder.
      */
-    virtual OC_REQUIRED_RESULT AvailabilityResult availability(const QString &folderPath) = 0;
+    [[nodiscard]] virtual AvailabilityResult availability(const QString &folderPath) = 0;
 
 public slots:
     /** Update in-sync state based on SyncFileStatusTracker signal.
@@ -231,10 +238,6 @@ public slots:
     virtual void fileStatusChanged(const QString &systemFileName, SyncFileStatus fileStatus) = 0;
 
 signals:
-    /// Emitted when a user-initiated hydration starts
-    void beginHydrating();
-    /// Emitted when the hydration ends
-    void doneHydrating();
     /// start complete
     void started();
 
@@ -247,7 +250,7 @@ protected:
      * If the remote metadata changes, the local placeholder's metadata should possibly
      * change as well.
      */
-    virtual OC_REQUIRED_RESULT Result<ConvertToPlaceholderResult, QString> updateMetadata(const SyncFileItem &item, const QString &filePath, const QString &replacesFile) = 0;
+    [[nodiscard]] virtual Result<ConvertToPlaceholderResult, QString> updateMetadata(const SyncFileItem &item, const QString &filePath, const QString &replacesFile) = 0;
 
     /** Setup the plugin for the folder.
      *
@@ -265,57 +268,34 @@ protected:
     Optional<PinState> pinStateInDb(const QString &folderPath);
     AvailabilityResult availabilityInDb(const QString &folderPath);
 
+private:
     // the parameters passed to start()
-    VfsSetupParams _setupParams;
+    std::unique_ptr<VfsSetupParams> _setupParams;
 
     friend class OwncloudPropagator;
 };
 
-/// Implementation of Vfs for Vfs::Off mode - does nothing
-class OCSYNC_EXPORT VfsOff : public Vfs
+class OCSYNC_EXPORT VfsPluginManager
 {
-    Q_OBJECT
-
 public:
-    VfsOff(QObject* parent = nullptr);
-    ~VfsOff() override;
+    /// Check whether the plugin for the mode is available.
+    bool isVfsPluginAvailable(Vfs::Mode mode) const;
 
-    Mode mode() const override { return Vfs::Off; }
+    /// Return the best available VFS mode.
+    Vfs::Mode bestAvailableVfsMode() const;
 
-    QString fileSuffix() const override { return QString(); }
+    /// Create a VFS instance for the mode, returns nullptr on failure.
+    std::unique_ptr<Vfs> createVfsFromPlugin(Vfs::Mode mode) const;
 
-    void stop() override {}
-    void unregisterFolder() override {}
-
-    bool socketApiPinStateActionsShown() const override { return false; }
-    bool isHydrating() const override { return false; }
-
-    Result<void, QString> createPlaceholder(const SyncFileItem &) override { return {}; }
-    Result<void, QString> dehydratePlaceholder(const SyncFileItem &) override { return {}; }
-
-    bool needsMetadataUpdate(const SyncFileItem &) override { return false; }
-    bool isDehydratedPlaceholder(const QString &) override { return false; }
-    bool statTypeVirtualFile(csync_file_stat_t *, void *) override { return false; }
-
-    bool setPinState(const QString &, PinState) override { return true; }
-    Optional<PinState> pinState(const QString &) override { return PinState::AlwaysLocal; }
-    AvailabilityResult availability(const QString &) override { return VfsItemAvailability::AlwaysLocal; }
-
-public slots:
-    void fileStatusChanged(const QString &, SyncFileStatus) override {}
+    static const VfsPluginManager &instance();
 
 protected:
-    Result<ConvertToPlaceholderResult, QString> updateMetadata(const SyncFileItem &, const QString &, const QString &) override { return { ConvertToPlaceholderResult::Ok }; }
-    void startImpl(const VfsSetupParams &) override { Q_EMIT started(); }
+    VfsPluginManager() = default;
+
+private:
+    static VfsPluginManager *_instance;
+
+    mutable QMap<Vfs::Mode, bool> _pluginCache;
 };
 
-/// Check whether the plugin for the mode is available.
-OCSYNC_EXPORT bool isVfsPluginAvailable(Vfs::Mode mode);
-
-/// Return the best available VFS mode.
-OCSYNC_EXPORT Vfs::Mode bestAvailableVfsMode();
-
-/// Create a VFS instance for the mode, returns nullptr on failure.
-OCSYNC_EXPORT std::unique_ptr<Vfs> createVfsFromPlugin(Vfs::Mode mode);
-
 } // namespace OCC
