@@ -18,25 +18,44 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
+#include "config.h"
 #include "config_csync.h"
 
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE
 #endif
-
-#include "c_lib.h"
-#include "c_private.h"
-
 #include "csync_exclude.h"
 
+#include "common/filesystembase.h"
 #include "common/utility.h"
-#include "../version.h"
+#include "common/version.h"
 
 #include <QString>
 #include <QFileInfo>
 #include <QFile>
 #include <QDir>
 
+namespace {
+
+// See http://support.microsoft.com/kb/74496 and
+// https://msdn.microsoft.com/en-us/library/windows/desktop/aa365247(v=vs.85).aspx
+// Additionally, we ignore '$Recycle.Bin', see https://github.com/owncloud/client/issues/2955
+const QLatin1String win_device_names[] = {
+    QLatin1String("CON"), QLatin1String("PRN"), QLatin1String("AUX"), QLatin1String("NUL"),
+    QLatin1String("COM1"), QLatin1String("COM2"), QLatin1String("COM3"),
+    QLatin1String("COM4"), QLatin1String("COM5"), QLatin1String("COM6"),
+    QLatin1String("COM7"), QLatin1String("COM8"), QLatin1String("COM9"),
+    QLatin1String("LPT1"), QLatin1String("LPT2"), QLatin1String("LPT3"),
+    QLatin1String("LPT4"), QLatin1String("LPT5"), QLatin1String("LPT6"),
+    QLatin1String("LPT7"), QLatin1String("LPT8"), QLatin1String("LPT9"),
+    QLatin1String("CLOCK$")
+};
+
+const QLatin1String win_system_files[] = {
+    QLatin1String("$Recycle.Bin"),
+    QLatin1String("System Volume Information")
+};
+}
 
 /** Expands C-like escape sequences (in place)
  */
@@ -76,16 +95,6 @@ OCSYNC_EXPORT void csync_exclude_expand_escapes(QByteArray &input)
     input.resize(o);
 }
 
-// See http://support.microsoft.com/kb/74496 and
-// https://msdn.microsoft.com/en-us/library/windows/desktop/aa365247(v=vs.85).aspx
-// Additionally, we ignore '$Recycle.Bin', see https://github.com/owncloud/client/issues/2955
-static const char *win_reserved_words_3[] = { "CON", "PRN", "AUX", "NUL" };
-static const char *win_reserved_words_4[] = {
-    "COM1", "COM2", "COM3", "COM4", "COM5", "COM6", "COM7", "COM8", "COM9",
-    "LPT1", "LPT2", "LPT3", "LPT4", "LPT5", "LPT6", "LPT7", "LPT8", "LPT9"
-};
-static const char *win_reserved_words_n[] = { "CLOCK$", "$Recycle.Bin" };
-
 /**
  * @brief Checks if filename is considered reserved by Windows
  * @param file_name filename
@@ -105,38 +114,42 @@ OCSYNC_EXPORT bool csync_is_windows_reserved_word(const QStringRef &filename)
         }
     }
 
-    if (len_filename == 3 || (len_filename > 3 && filename.at(3) == QLatin1Char('.'))) {
-        for (const char *word : win_reserved_words_3) {
-            if (filename.left(3).compare(QLatin1String(word), Qt::CaseInsensitive) == 0) {
-                return true;
-            }
+    // win_reserved_words is ordered so we can stop once we encounter a longer word
+    Q_ASSERT(std::is_sorted(std::begin(win_device_names), std::end(win_device_names), [](const QLatin1String &a, const QLatin1String &b) {
+        return a.size() < b.size();
+    }));
+    for (const auto &word : win_device_names) {
+        if (static_cast<size_t>(word.size()) > len_filename) {
+            break;
         }
+        // until windows 11, not only the device names where illegal file names
+        // also COM9.png was illegal
+        if ((static_cast<size_t>(word.size()) == len_filename || filename.at(word.size()) == QLatin1Char('.')) && filename.startsWith(word, Qt::CaseInsensitive)) {
+            return true;
+        }
     }
-
-    if (len_filename == 4 || (len_filename > 4 && filename.at(4) == QLatin1Char('.'))) {
-        for (const char *word : win_reserved_words_4) {
-            if (filename.left(4).compare(QLatin1String(word), Qt::CaseInsensitive) == 0) {
-                return true;
-            }
+    // win_reserved_words is ordered so we can stop once we encounter a longer word
+    Q_ASSERT(std::is_sorted(std::begin(win_system_files), std::end(win_system_files), [](const QLatin1String &a, const QLatin1String &b) {
+        return a.size() < b.size();
+    }));
+    for (const auto &word : win_system_files) {
+        if (static_cast<size_t>(word.size()) > len_filename) {
+            break;
         }
-    }
-
-    for (const char *word : win_reserved_words_n) {
-        if (filename.compare(QLatin1String(word), Qt::CaseInsensitive) == 0) {
+        if (static_cast<size_t>(word.size()) == len_filename && filename.compare(word, Qt::CaseInsensitive) == 0) {
             return true;
         }
     }
-
     return false;
 }
 
-static CSYNC_EXCLUDE_TYPE _csync_excluded_common(const QString &path, bool excludeConflictFiles)
+static CSYNC_EXCLUDE_TYPE _csync_excluded_common(const QStringRef &path, bool excludeConflictFiles)
 {
     /* split up the path */
-    QStringRef bname(&path);
+    QStringRef bname(path);
     int lastSlash = path.lastIndexOf(QLatin1Char('/'));
     if (lastSlash >= 0) {
-        bname = path.midRef(lastSlash + 1);
+        bname = path.mid(lastSlash + 1);
     }
 
     qsizetype blen = bname.size();
@@ -154,6 +167,10 @@ static CSYNC_EXCLUDE_TYPE _csync_excluded_common(const QString &path, bool exclu
         }
     }
 
+    if (bname.endsWith(QLatin1String(APPLICATION_DOTVIRTUALFILE_SUFFIX), Qt::CaseInsensitive)) { // ".owncloud" placeholder
+        return CSYNC_FILE_EXCLUDE_RESERVED;
+    }
+
     // check the strlen and ignore the file if its name is longer than 254 chars.
     // whenever changing this also check createDownloadTmpFileName
     if (blen > 254) {
@@ -179,34 +196,27 @@ static CSYNC_EXCLUDE_TYPE _csync_excluded_common(const QString &path, bool exclu
 
     // Filter out characters not allowed in a filename on windows
     // see https://docs.microsoft.com/en-us/windows/desktop/fileio/naming-a-file
-    for (auto p : path) {
-        ushort c = p.unicode();
+    for (auto c : path) {
         if (c < 32) {
             return CSYNC_FILE_EXCLUDE_INVALID_CHAR;
         }
-        switch (c) {
-        case '\\':
-        case ':':
-        case '?':
-        case '*':
-        case '"':
-        case '>':
-        case '<':
-        case '|':
+        if (std::find_if(
+                OCC::FileSystem::IllegalFilenameCharsWindows.begin(), OCC::FileSystem::IllegalFilenameCharsWindows.end(), [c](const auto illegal) {
+                    return c == illegal;
+                })
+            != OCC::FileSystem::IllegalFilenameCharsWindows.end()) {
             return CSYNC_FILE_EXCLUDE_INVALID_CHAR;
-        default:
-            break;
         }
     }
 #endif
 
-    /* We create a Desktop.ini on Windows for the sidebar icon, make sure we don't sync it. */
-    if (blen == 11 && path == bname) {
-        if (bname.compare(QLatin1String("Desktop.ini"), Qt::CaseInsensitive) == 0) {
-            return CSYNC_FILE_SILENTLY_EXCLUDED;
-        }
+    /* Do not sync desktop.ini files anywhere in the tree. */
+    const auto desktopIniFile = QStringLiteral("desktop.ini");
+    if (blen == static_cast<qsizetype>(desktopIniFile.length()) && bname.compare(desktopIniFile, Qt::CaseInsensitive) == 0) {
+        return CSYNC_FILE_SILENTLY_EXCLUDED;
     }
 
+
     if (excludeConflictFiles && OCC::Utility::isConflictFile(path)) {
         return CSYNC_FILE_EXCLUDE_CONFLICT;
     }
@@ -217,7 +227,7 @@ static CSYNC_EXCLUDE_TYPE _csync_excluded_common(const QString &path, bool exclu
 using namespace OCC;
 
 ExcludedFiles::ExcludedFiles()
-    : _clientVersion(MIRALL_VERSION_MAJOR, MIRALL_VERSION_MINOR, MIRALL_VERSION_PATCH)
+    : _clientVersion(OCC::Version::version())
 {
     // Windows used to use PathMatchSpec which allows *foo to match abc/deffoo.
     _wildcardsMatchSlash = Utility::isWindows();
@@ -256,7 +266,7 @@ void ExcludedFiles::setWildcardsMatchSlash(bool onoff)
     prepare();
 }
 
-void ExcludedFiles::setClientVersion(ExcludedFiles::Version version)
+void ExcludedFiles::setClientVersion(const QVersionNumber &version)
 {
     _clientVersion = version;
 }
@@ -265,7 +275,7 @@ bool ExcludedFiles::reloadExcludeFiles()
 {
     _allExcludes.clear();
     bool success = true;
-    foreach (const QString &file, _excludeFiles) {
+    for (const auto &file : qAsConst(_excludeFiles)) {
         QFile f(file);
         if (!f.open(QIODevice::ReadOnly)) {
             success = false;
@@ -300,7 +310,7 @@ bool ExcludedFiles::versionDirectiveKeepNextLine(const QByteArray &directive) co
     if (argVersions.size() != 3)
         return true;
 
-    auto argVersion = std::make_tuple(argVersions[0].toInt(), argVersions[1].toInt(), argVersions[2].toInt());
+    const auto argVersion = QVersionNumber(argVersions[0].toInt(), argVersions[1].toInt(), argVersions[2].toInt());
     if (op == "<=")
         return _clientVersion <= argVersion;
     if (op == "<")
@@ -319,41 +329,60 @@ bool ExcludedFiles::isExcluded(
     const QString &basePath,
     bool excludeHidden) const
 {
-    if (!filePath.startsWith(basePath, Utility::fsCasePreserving() ? Qt::CaseInsensitive : Qt::CaseSensitive)) {
+    if (!FileSystem::isChildPathOf(filePath, basePath)) {
         // Mark paths we're not responsible for as excluded...
         return true;
     }
 
+    const QFileInfo fileInfo(filePath);
+    if (!fileInfo.exists()) {
+        return isExcluded(fileInfo.path(), basePath, excludeHidden);
+    }
+
     if (excludeHidden) {
-        QString path = filePath;
+        QFileInfo fi = fileInfo;
         // Check all path subcomponents, but to *not* check the base path:
         // We do want to be able to sync with a hidden folder as the target.
-        while (path.size() > basePath.size()) {
-            QFileInfo fi(path);
-            if (fi.isHidden() || fi.fileName().startsWith(QLatin1Char('.'))) {
+        while (fi.filePath().size() > basePath.size()) {
+            if (fi.isHidden()) {
                 return true;
             }
 
             // Get the parent path
-            path = fi.absolutePath();
+            fi = { fi.path() };
         }
     }
-
-    QFileInfo fi(filePath);
     ItemType type = ItemTypeFile;
-    if (fi.isDir()) {
+    if (fileInfo.isDir()) {
         type = ItemTypeDirectory;
     }
+    return isExcludedRemote(filePath, basePath, excludeHidden, type);
+}
 
-    QString relativePath = filePath.mid(basePath.size());
+bool ExcludedFiles::isExcludedRemote(const QString &filePath, const QString &basePath, bool excludeHidden, ItemType type) const
+{
+    if (!FileSystem::isChildPathOf(filePath, basePath)) {
+        // Mark paths we're not responsible for as excluded...
+        return true;
+    }
+
+    auto relativePath = filePath.midRef(basePath.size());
     if (relativePath.endsWith(QLatin1Char('/'))) {
         relativePath.chop(1);
     }
 
+    if (excludeHidden) {
+        // Check their parent directories for . hidden files
+        if (relativePath.startsWith(QLatin1Char('.')) || relativePath.contains(QLatin1String("/."))) {
+            return true;
+        }
+    }
+
+
     return fullPatternMatch(relativePath, type) != CSYNC_NOT_EXCLUDED;
 }
 
-CSYNC_EXCLUDE_TYPE ExcludedFiles::traversalPatternMatch(const QString &path, ItemType filetype) const
+CSYNC_EXCLUDE_TYPE ExcludedFiles::traversalPatternMatch(const QStringRef &path, ItemType filetype) const
 {
     auto match = _csync_excluded_common(path, _excludeConflictFiles);
     if (match != CSYNC_NOT_EXCLUDED)
@@ -363,10 +392,10 @@ CSYNC_EXCLUDE_TYPE ExcludedFiles::traversalPatternMatch(const QString &path, Ite
 
     // Check the bname part of the path to see whether the full
     // regex should be run.
-    QStringRef bnameStr(&path);
+    QStringRef bnameStr(path);
     int lastSlash = path.lastIndexOf(QLatin1Char('/'));
     if (lastSlash >= 0) {
-        bnameStr = path.midRef(lastSlash + 1);
+        bnameStr = path.mid(lastSlash + 1);
     }
 
     QRegularExpressionMatch m;
@@ -384,7 +413,7 @@ CSYNC_EXCLUDE_TYPE ExcludedFiles::traversalPatternMatch(const QString &path, Ite
     }
 
     // third capture: full path matching is triggered
-    QString pathStr = path;
+    QStringRef pathStr = path;
 
     if (filetype == ItemTypeDirectory) {
         m = _fullTraversalRegexDir.match(pathStr);
@@ -401,7 +430,7 @@ CSYNC_EXCLUDE_TYPE ExcludedFiles::traversalPatternMatch(const QString &path, Ite
     return CSYNC_NOT_EXCLUDED;
 }
 
-CSYNC_EXCLUDE_TYPE ExcludedFiles::fullPatternMatch(const QString &p, ItemType filetype) const
+CSYNC_EXCLUDE_TYPE ExcludedFiles::fullPatternMatch(const QStringRef &p, ItemType filetype) const
 {
     auto match = _csync_excluded_common(p, _excludeConflictFiles);
     if (match != CSYNC_NOT_EXCLUDED)
@@ -486,7 +515,7 @@ QString ExcludedFiles::convertToRegexpSyntax(QString exclude, bool wildcardsMatc
             // Translate [! to [^
             QString bracketExpr = exclude.mid(i, j - i + 1);
             if (bracketExpr.startsWith(QLatin1String("[!")))
-                bracketExpr[1] = '^';
+                bracketExpr[1] = QLatin1Char('^');
             regex.append(bracketExpr);
             i = j;
             break;
@@ -599,7 +628,7 @@ void ExcludedFiles::prepare()
         pattern.append(appendMe);
     };
 
-    for (auto exclude : _allExcludes) {
+    for (auto exclude : qAsConst(_allExcludes)) {
         if (exclude[0] == QLatin1Char('\n'))
             continue; // empty line
         if (exclude[0] == QLatin1Char('\r'))
