@@ -18,20 +18,24 @@
 
 #include "jobqueue.h"
 
+#include "connectionvalidator.h"
+#include "creds/abstractcredentials.h"
+#include "updateurldialog.h"
 #include <QByteArray>
 #include <QElapsedTimer>
 #include <QPointer>
-#include "connectionvalidator.h"
-#include "creds/abstractcredentials.h"
 #include <memory>
 
-class QSettings;
+class QDialog;
 class QMessageBox;
+class QSettings;
 
 namespace OCC {
 
 class AccountState;
 class Account;
+class QuotaInfo;
+class TlsErrorDialog;
 
 /**
  * @brief Extra info about an ownCloud server account.
@@ -40,7 +44,7 @@ class Account;
 class AccountState : public QObject, public QSharedData
 {
     Q_OBJECT
-    Q_PROPERTY(AccountPtr account MEMBER _account)
+    Q_PROPERTY(AccountPtr account MEMBER _account READ account)
 
 public:
     enum State {
@@ -71,27 +75,30 @@ public:
         ConfigurationError,
 
         /// We are currently asking the user for credentials
-        AskingCredentials
+        AskingCredentials,
+
+        Connecting
     };
+    Q_ENUM(State)
 
     /// The actual current connectivity status.
     typedef ConnectionValidator::Status ConnectionStatus;
 
-    /// Use the account as parent
-    explicit AccountState(AccountPtr account);
     ~AccountState() override;
 
     /** Creates an account state from settings and an Account object.
      *
      * Use from AccountManager with a prepared QSettings object only.
      */
-    static AccountState *loadFromSettings(AccountPtr account, QSettings &settings);
+    static AccountStatePtr loadFromSettings(AccountPtr account, const QSettings &settings);
+
+    static AccountStatePtr fromNewAccount(AccountPtr account);
 
     /** Writes account state information to settings.
      *
      * It does not write the Account data.
      */
-    void writeToSettings(QSettings &settings);
+    void writeToSettings(QSettings &settings) const;
 
     AccountPtr account() const;
 
@@ -99,7 +106,6 @@ public:
     QStringList connectionErrors() const;
 
     State state() const;
-    static QString stateString(State state);
 
     bool isSignedOut() const;
 
@@ -122,6 +128,11 @@ public:
 
     bool isConnected() const;
 
+    // weather the account was created after spaces where implemented
+    bool supportsSpaces() const;
+
+    QuotaInfo *quotaInfo();
+
     /** Returns a new settings object for this account, already in the right groups. */
     std::unique_ptr<QSettings> settings();
 
@@ -132,7 +143,7 @@ public:
      *  was not so long ago.
      */
     void tagLastSuccessfullETagRequest(const QDateTime &tp);
-    void updateUrlDialog(const QUrl &url);
+    UpdateUrlDialog *updateUrlDialog(const QUrl &newUrl);
 
 public slots:
     /// Triggers a ping to the server to update state and
@@ -141,6 +152,9 @@ public slots:
     void checkConnectivity(bool verifyServerState = false);
 
 private:
+    /// Use the account as parent
+    explicit AccountState(AccountPtr account);
+
     void setState(State state);
 
 signals:
@@ -163,7 +177,9 @@ private:
     bool _waitingForNewCredentials;
     QDateTime _timeOfLastETagCheck;
     QPointer<ConnectionValidator> _connectionValidator;
-    QPointer<QMessageBox> _updateUrlDialog;
+    QPointer<UpdateUrlDialog> _updateUrlDialog;
+    QPointer<TlsErrorDialog> _tlsDialog;
+    bool _supportsSpaces = true;
 
     /**
      * Starts counting when the server starts being back up after 503 or
@@ -175,7 +191,11 @@ private:
     /**
      * Milliseconds for which to delay reconnection after 503/maintenance.
      */
-    int _maintenanceToConnectedDelay;
+    std::chrono::milliseconds _maintenanceToConnectedDelay;
+
+    QuotaInfo *_quotaInfo = nullptr;
+
+    friend class SpaceMigration;
 };
 }
 
