@@ -23,13 +23,14 @@
 
 #include "3rdparty/qtsingleapplication/qtsingleapplication.h"
 
-#include "syncresult.h"
+#include "clientproxy.h"
+#include "connectionvalidator.h"
+#include "folderman.h"
 #include "logbrowser.h"
 #include "owncloudgui.h"
-#include "connectionvalidator.h"
+#include "platform.h"
 #include "progressdispatcher.h"
-#include "clientproxy.h"
-#include "folderman.h"
+#include "syncresult.h"
 
 class QMessageBox;
 class QSystemTrayIcon;
@@ -55,20 +56,20 @@ class Application : public SharedTools::QtSingleApplication
 {
     Q_OBJECT
 public:
-    explicit Application(int &argc, char **argv);
+    explicit Application(int &argc, char **argv, Platform *platform);
     ~Application() override;
 
     bool debugMode();
 
-    void showSettingsDialog();
+    Q_INVOKABLE void showSettingsDialog();
 
     ownCloudGui *gui() const;
 
     QString displayLanguage() const;
 
+    AccountStatePtr addNewAccount(AccountPtr newAccount);
+
 public slots:
-    // TODO: this should not be public
-    void slotownCloudWizardDone(int);
     void slotCrash();
     void slotCrashEnforce();
     void slotCrashFatal();
@@ -97,9 +98,9 @@ protected slots:
     void slotCheckConnection();
     void slotUseMonoIconsChanged(bool);
     void slotCleanup();
-    void slotAccountStateAdded(AccountState *accountState);
-    void slotAccountStateRemoved(AccountState *accountState);
-    void slotSystemOnlineConfigurationChanged(QNetworkConfiguration);
+    void slotAccountStateAdded(AccountStatePtr accountState) const;
+    void slotAccountStateRemoved() const;
+    void slotSystemOnlineConfigurationChanged(const QNetworkConfiguration &);
 
 private:
     /**
@@ -117,7 +118,6 @@ private:
     bool _quitInstance = false;
     QString _logFile;
     QString _logDir;
-    std::chrono::hours _logExpire;
     bool _logFlush;
     bool _logDebug;
     bool _userTriggeredConnect;
@@ -130,9 +130,6 @@ private:
     QNetworkConfigurationManager _networkConfigurationManager;
     QTimer _checkConnectionTimer;
 
-#if defined(WITH_CRASHREPORTER)
-    QScopedPointer<CrashReporter::Handler> _crashHandler;
-#endif
     QScopedPointer<FolderMan> _folderManager;
 };
 
