@@ -143,9 +143,10 @@ private:
     QString _targetPath;
     bool _deployed = false;
 
-    uint32_t _priority;
+    uint32_t _priority = 0;
 
     friend class FolderMan;
+    friend class SpaceMigration;
 };
 
 /**
@@ -212,7 +213,7 @@ public:
      */
     QString remotePathTrailingSlash() const;
 
-    void setNavigationPaneClsid(const QUuid &clsid) { _definition.navigationPaneClsid = clsid; }
+    void setNavigationPaneClsid(const QUuid &clsid);
     QUuid navigationPaneClsid() const { return _definition.navigationPaneClsid; }
 
     /**
@@ -426,7 +427,7 @@ public slots:
        * changes. Needs to check whether this change should trigger a new
        * sync run to be scheduled.
        */
-    void slotWatchedPathChanged(const QString &path, ChangeReason reason);
+    void slotWatchedPathsChanged(const QSet<QString> &paths, ChangeReason reason);
 
     /**
      * Mark a virtual file as being requested for download, and start a sync.
@@ -579,7 +580,7 @@ private:
     /**
      * Watches this folder's local directory for changes.
      *
-     * Created by registerFolderWatcher(), triggers slotWatchedPathChanged()
+     * Created by registerFolderWatcher(), triggers slotWatchedPathsChanged()
      */
     QScopedPointer<FolderWatcher> _folderWatcher;
 
@@ -592,6 +593,8 @@ private:
      * The vfs mode instance (created by plugin) to use. Never null.
      */
     QSharedPointer<Vfs> _vfs;
+
+    friend class SpaceMigration;
 };
 }
 
