@@ -25,6 +25,10 @@
 #include "navigationpanehelper.h"
 #include "syncfileitem.h"
 
+#include "folderwizard/folderwizard.h"
+
+#include "newwizard/enums.h"
+
 class TestFolderMigration;
 
 namespace OCC {
@@ -66,7 +70,7 @@ private:
  *   (_timeScheduler and slotScheduleFolderByTime())
  *
  * - A folder watcher receives a notification about a file change
- *   (_folderWatchers and Folder::slotWatchedPathChanged())
+ *   (_folderWatchers and Folder::slotWatchedPathsChanged())
  *
  * - The folder etag on the server has changed
  *   (_etagPollTimer)
@@ -82,11 +86,21 @@ class FolderMan : public QObject
 {
     Q_OBJECT
 public:
+    static QString suggestSyncFolder(const QUrl &server, const QString &displayName);
+    [[nodiscard]] static bool prepareFolder(const QString &folder);
+
+    static QString checkPathValidityRecursive(const QString &path);
+
     ~FolderMan() override;
+
+    /**
+     * Helper to access the FolderMan instance
+     * Warning: may be null in unit tests
+     */
+    // TODO: use acces throug ocApp and remove that instance pointer
     static FolderMan *instance();
 
     int setupFolders();
-    int setupFoldersMigration();
 
     /** Find folder setting keys that need to be ignored or deleted for being too new.
      *
@@ -107,11 +121,18 @@ public:
      */
     static void backwardMigrationSettingsKeys(QStringList *deleteKeys, QStringList *ignoreKeys);
 
-    const QMap<QString, Folder *> &map() const;
+    const QVector<Folder *> &folders() const;
 
     /** Adds a folder for an account, ensures the journal is gone and saves it in the settings.
       */
-    Folder *addFolder(AccountState *accountState, const FolderDefinition &folderDefinition);
+    Folder *addFolder(const AccountStatePtr &accountState, const FolderDefinition &folderDefinition);
+
+    /**
+     * Adds a folder for an account. Used to be part of the wizard code base. Constructs the folder definition from the parameters.
+     * In case Wizard::SyncMode::SelectiveSync is used, nullptr is returned.
+     */
+    Folder *addFolderFromWizard(const AccountStatePtr &accountStatePtr, const QString &localFolder, const QString &remotePath, const QUrl &webDavUrl, const QString &displayName, bool useVfs);
+    Folder *addFolderFromFolderWizardResult(const AccountStatePtr &accountStatePtr, const FolderWizard::Result &config);
 
     /** Removes a folder */
     void removeFolder(Folder *);
@@ -131,14 +152,8 @@ public:
       */
     QStringList findFileInLocalFolders(const QString &relPath, const AccountPtr acc);
 
-    /** Returns the folder by alias or NULL if no folder with the alias exists. */
-    Folder *folder(const QString &);
-
-    /**
-     * Migrate accounts from owncloud < 2.0
-     * Creates a folder for a specific configuration, identified by alias.
-     */
-    Folder *setupFolderFromOldConfigFile(const QString &, AccountState *account);
+    /** Returns the folder by id or NULL if no folder with the id exists. */
+    [[deprecated("directly reference the folder")]] Folder *folder(const QByteArray &id);
 
     /**
      * Ensures that a given directory does not contain a sync journal file.
@@ -157,12 +172,7 @@ public:
      * Compute status summarizing multiple folders
      * @return tuple containing folders, status, unresolvedConflicts and lastSyncDone
      */
-    static TrayOverallStatusResult trayOverallStatus(const QList<Folder *> &folders);
-
-    // Escaping of the alias which is used in QSettings AND the file
-    // system, thus need to be escaped.
-    static QString escapeAlias(const QString &);
-    static QString unescapeAlias(const QString &);
+    static TrayOverallStatusResult trayOverallStatus(const QVector<Folder *> &folders);
 
     SocketApi *socketApi();
 #ifdef Q_OS_WIN
@@ -186,7 +196,7 @@ public:
      * subfolder of ~ would be a good candidate. When that happens \a basePath
      * is returned.
      */
-    QString findGoodPathForNewSyncFolder(const QString &basePath) const;
+    QString findGoodPathForNewSyncFolder(const QString &basePath, const QString &newFolder) const;
 
     /**
      * While ignoring hidden files can theoretically be switched per folder,
@@ -229,11 +239,11 @@ public:
      */
     void setSyncEnabled(bool);
 
-    /** Queues a folder for syncing. */
-    void scheduleFolder(Folder *);
-
-    /** Puts a folder in the very front of the queue. */
-    void scheduleFolderNext(Folder *);
+    /**
+     * Queues a folder for syncing.
+     * When force is true it will be performed asap.
+     */
+    void scheduleFolder(Folder *folder, bool force = false);
 
     /** Queues all folders for syncing. */
     void scheduleAllFolders();
@@ -242,7 +252,7 @@ public:
     void setDirtyNetworkLimits();
 
     /** Whether or not vfs is supported in the location. */
-    bool checkVfsAvailability(const QString &path, Vfs::Mode mode = bestAvailableVfsMode()) const;
+    bool checkVfsAvailability(const QString &path, Vfs::Mode mode = VfsPluginManager::instance().bestAvailableVfsMode()) const;
 
     /** If the folder configuration is no longer supported this will return an error string */
     Result<void, QString> unsupportedConfiguration(const QString &path) const;
@@ -262,7 +272,7 @@ signals:
     /**
      * Emitted whenever the list of configured folders changes.
      */
-    void folderListChanged(const QMap<QString, Folder *> &);
+    void folderListChanged();
     void folderRemoved(Folder *folder);
 
 public slots:
@@ -286,8 +296,8 @@ public slots:
      */
     void slotSyncOnceFileUnlocks(const QString &path, FileSystem::LockMode mode);
 
-    // slot to schedule an ETag job (from Folder only)
-    void slotScheduleETagJob(const QString &alias, RequestEtagJob *job);
+    // slot to schedule an ETag job
+    void slotScheduleETagJob(RequestEtagJob *job);
 
 private slots:
     void slotFolderSyncPaused(Folder *, bool paused);
@@ -302,22 +312,10 @@ private slots:
     void slotStartScheduledFolderSync();
     void slotEtagPollTimerTimeout();
 
-    void slotRemoveFoldersForAccount(AccountState *accountState);
-
-    // Wraps the Folder::syncStateChange() signal into the
-    // FolderMan::folderSyncStateChange(Folder*) signal.
-    void slotForwardFolderSyncStateChange();
+    void slotRemoveFoldersForAccount(const AccountStatePtr &accountState);
 
     void slotServerVersionChanged(Account *account);
 
-    /**
-     * A file whose locks were being monitored has become unlocked.
-     *
-     * This schedules the folder for synchronization that contains
-     * the file with the given path.
-     */
-    void slotWatchedFileUnlocked(const QString &path);
-
     /**
      * Schedules folders whose time to sync has come.
      *
@@ -331,7 +329,7 @@ private:
      *  does not set an account on the new folder.
       */
     Folder *addFolderInternal(FolderDefinition folderDefinition,
-        AccountState *accountState, std::unique_ptr<Vfs> vfs);
+        const AccountStatePtr &accountState, std::unique_ptr<Vfs> vfs);
 
     /* unloads a folder object, does not delete it */
     void unloadFolder(Folder *);
@@ -352,9 +350,9 @@ private:
     void setupFoldersHelper(QSettings &settings, AccountStatePtr account, const QStringList &ignoreKeys, bool backwardsCompatible, bool foldersWithPlaceholders);
 
     QSet<Folder *> _disabledFolders;
-    QMap<QString, Folder *> _folderMap;
+    QVector<Folder *> _folders;
     QString _folderConfigPath;
-    Folder *_currentSyncFolder;
+    QPointer<Folder> _currentSyncFolder;
     QPointer<Folder> _lastSyncFolder;
     bool _syncEnabled;
 
