@@ -13,32 +13,71 @@
  */
 
 #include "folderstatusmodel.h"
-#include "folderman.h"
+#include "account.h"
 #include "accountstate.h"
 #include "common/asserts.h"
-#include <theme.h>
-#include <account.h>
+#include "folderman.h"
 #include "folderstatusdelegate.h"
+#include "gui/quotainfo.h"
+#include "theme.h"
 
+#include "resources/resources.h"
+
+#include "libsync/graphapi/space.h"
+#include "libsync/graphapi/spacesmanager.h"
+
+#include <QDir>
 #include <QFileIconProvider>
 #include <QVarLengthArray>
+
 #include <set>
 
+using namespace std::chrono_literals;
+
 Q_DECLARE_METATYPE(QPersistentModelIndex)
 
 namespace OCC {
 
 Q_LOGGING_CATEGORY(lcFolderStatus, "gui.folder.model", QtInfoMsg)
 
-static const char propertyParentIndexC[] = "oc_parentIndex";
-static const char propertyPermissionMap[] = "oc_permissionMap";
+namespace {
+    // minimum delay between progress updates
+    constexpr auto progressUpdateTimeOutC = 1s;
 
-static QString removeTrailingSlash(const QString &s)
-{
-    if (s.endsWith('/')) {
-        return s.left(s.size() - 1);
+    const char propertyParentIndexC[] = "oc_parentIndex";
+    const char propertyPermissionMap[] = "oc_permissionMap";
+
+    int64_t getQuota(const AccountStatePtr &accountState, const QUrl &davUrl, FolderStatusModel::Columns type)
+    {
+        if (accountState->supportsSpaces()) {
+            if (auto spacesManager = accountState->account()->spacesManager()) {
+                const auto *space = spacesManager->spaceByUrl(davUrl);
+                if (space) {
+                    const auto quota = space->drive().getQuota();
+                    if (quota.isValid()) {
+                        switch (type) {
+                        case FolderStatusModel::Columns::QuotaTotal:
+                            return quota.getTotal();
+                        case FolderStatusModel::Columns::QuotaUsed:
+                            return quota.getUsed();
+                        default:
+                            Q_UNREACHABLE();
+                        }
+                    }
+                }
+            }
+        } else {
+            switch (type) {
+            case FolderStatusModel::Columns::QuotaTotal:
+                return accountState->quotaInfo()->lastQuotaTotalBytes();
+            case FolderStatusModel::Columns::QuotaUsed:
+                return accountState->quotaInfo()->lastQuotaUsedBytes();
+            default:
+                Q_UNREACHABLE();
+            }
+        }
+        return {};
     }
-    return s;
 }
 
 FolderStatusModel::FolderStatusModel(QObject *parent)
@@ -46,57 +85,60 @@ FolderStatusModel::FolderStatusModel(QObject *parent)
     , _accountState(nullptr)
     , _dirty(false)
 {
+    connect(this, &FolderStatusModel::rowsInserted, this, &FolderStatusModel::dirtyChanged);
 }
 
 FolderStatusModel::~FolderStatusModel()
 {
 }
 
-static bool sortByFolderHeader(const FolderStatusModel::SubFolderInfo &lhs, const FolderStatusModel::SubFolderInfo &rhs)
-{
-    return QString::compare(lhs._folder->shortGuiRemotePathOrAppName(),
-               rhs._folder->shortGuiRemotePathOrAppName(),
-               Qt::CaseInsensitive)
-        < 0;
-}
-
-void FolderStatusModel::setAccountState(const AccountState *accountState)
+void FolderStatusModel::setAccountState(const AccountStatePtr &accountState)
 {
     beginResetModel();
     _dirty = false;
     _folders.clear();
-    _accountState = accountState;
+    if (_accountState != accountState) {
+        Q_ASSERT(!_accountState);
+        _accountState = accountState;
 
-    connect(FolderMan::instance(), &FolderMan::folderSyncStateChange,
-        this, &FolderStatusModel::slotFolderSyncStateChange, Qt::UniqueConnection);
-    connect(FolderMan::instance(), &FolderMan::scheduleQueueChanged,
-        this, &FolderStatusModel::slotFolderScheduleQueueChanged, Qt::UniqueConnection);
+        connect(FolderMan::instance(), &FolderMan::folderSyncStateChange, this, &FolderStatusModel::slotFolderSyncStateChange);
+        connect(FolderMan::instance(), &FolderMan::scheduleQueueChanged, this, &FolderStatusModel::slotFolderScheduleQueueChanged);
 
-    for (const auto &f : FolderMan::instance()->map()) {
+        if (accountState->supportsSpaces()) {
+            connect(accountState->account()->spacesManager(), &GraphApi::SpacesManager::updated, this, [this] {
+                beginResetModel();
+                endResetModel();
+            });
+        }
+    }
+    for (const auto &f : FolderMan::instance()->folders()) {
         if (!accountState)
             break;
         if (f->accountState() != accountState)
             continue;
         SubFolderInfo info;
-        info._name = f->alias();
-        info._path = "/";
+        // the name is not actually used with the root element
+        info._name = QLatin1Char('/');
+        info._path = QLatin1Char('/');
         info._folder = f;
         info._checked = Qt::PartiallyChecked;
         _folders << info;
 
-        connect(f, &Folder::progressInfo, this, &FolderStatusModel::slotSetProgress, Qt::UniqueConnection);
+        connect(ProgressDispatcher::instance(), &ProgressDispatcher::progressInfo, this, [f, this](Folder *folder, const ProgressInfo &progress) {
+            if (folder == f) {
+                slotSetProgress(progress, f);
+            }
+        });
+
         connect(f, &Folder::newBigFolderDiscovered, this, &FolderStatusModel::slotNewBigFolder, Qt::UniqueConnection);
     }
 
-    // Sort by header text
-    std::sort(_folders.begin(), _folders.end(), sortByFolderHeader);
-
     // Set the root _pathIdx after the sorting
     for (int i = 0; i < _folders.size(); ++i) {
         _folders[i]._pathIdx << i;
     }
 
-    endResetModel();
+    Q_EMIT endResetModel();
     emit dirtyChanged();
 }
 
@@ -104,7 +146,7 @@ void FolderStatusModel::setAccountState(const AccountState *accountState)
 Qt::ItemFlags FolderStatusModel::flags(const QModelIndex &index) const
 {
     if (!_accountState) {
-        return nullptr;
+        return Qt::NoItemFlags;
     }
 
     // Always enable the item. If it isn't enabled, it cannot be in the selection model, so all
@@ -113,14 +155,6 @@ Qt::ItemFlags FolderStatusModel::flags(const QModelIndex &index) const
     const auto flags = Qt::ItemIsEnabled;
 
     switch (classify(index)) {
-    case AddButton: {
-        Qt::ItemFlags ret;
-        ret = Qt::ItemNeverHasChildren;
-        if (!_accountState->isConnected()) {
-            return ret;
-        }
-        return flags | ret;
-    }
     case FetchLabel:
         return flags | Qt::ItemNeverHasChildren;
     case RootFolder:
@@ -139,26 +173,39 @@ QVariant FolderStatusModel::data(const QModelIndex &index, int role) const
     if (role == Qt::EditRole)
         return QVariant();
 
-    switch (classify(index)) {
-    case AddButton: {
-        if (role == FolderStatusDelegate::AddButton) {
-            return QVariant(true);
-        } else if (role == Qt::ToolTipRole) {
-            if (!_accountState->isConnected()) {
-                return tr("You need to be connected to add a folder");
-            }
-            return tr("Click this button to add a folder to synchronize.");
-        }
-        return QVariant();
+    const Columns column = static_cast<Columns>(index.column());
+    const auto itemType = classify(index);
+    switch (column) {
+    case Columns::ItemType:
+        return itemType;
+    case Columns::IsUsingSpaces:
+        return _accountState->supportsSpaces();
+    default:
+        break;
     }
+
+    switch (itemType) {
     case SubFolder: {
         const auto &x = static_cast<SubFolderInfo *>(index.internalPointer())->_subs.at(index.row());
+
         switch (role) {
         case Qt::DisplayRole:
-            //: Example text: "File.txt (23KB)"
-            return x._size < 0 ? x._name : tr("%1 (%2)").arg(x._name, Utility::octetsToString(x._size));
+            switch (column) {
+            case Columns::FolderPathRole: {
+                auto f = x._folder;
+                if (!f)
+                    return QVariant();
+                return QVariant(f->path() + x._path);
+            }
+            case Columns::IsReady:
+                return x._folder->isReady();
+            case Columns::HeaderRole:
+                return x._size < 0 ? x._name : tr("%1 (%2)", "filename (size)").arg(x._name, Utility::octetsToString(x._size));
+            default:
+                return {};
+            }
         case Qt::ToolTipRole:
-            return QString(QLatin1String("<qt>") + Utility::escape(x._size < 0 ? x._name : tr("%1 (%2)").arg(x._name, Utility::octetsToString(x._size))) + QLatin1String("</qt>"));
+            return QString(QLatin1String("<qt>") + Utility::escape(x._size < 0 ? x._name : tr("%1 (%2)", "filename (size)").arg(x._name, Utility::octetsToString(x._size))) + QLatin1String("</qt>"));
         case Qt::CheckStateRole:
             return x._checked;
         case Qt::DecorationRole:
@@ -168,12 +215,6 @@ QVariant FolderStatusModel::data(const QModelIndex &index, int role) const
                 return QColor(Qt::red);
             }
             break;
-        case FolderStatusDelegate::FolderPathRole: {
-            auto f = x._folder;
-            if (!f)
-                return QVariant();
-            return QVariant(f->path() + x._path);
-        }
         }
     }
         return QVariant();
@@ -183,7 +224,7 @@ QVariant FolderStatusModel::data(const QModelIndex &index, int role) const
         case Qt::DisplayRole:
             if (x->_hasError) {
                 return QVariant(tr("Error while loading the list of folders from the server.")
-                    + QString("\n") + x->_lastErrorString);
+                    + QStringLiteral("\n") + x->_lastErrorString);
             } else {
                 return tr("Fetching folder list from server...");
             }
@@ -204,78 +245,117 @@ QVariant FolderStatusModel::data(const QModelIndex &index, int role) const
     const SubFolderInfo::Progress &progress = folderInfo._progress;
     const bool accountConnected = _accountState->isConnected();
 
-    switch (role) {
-    case FolderStatusDelegate::FolderPathRole:
-        return f->shortGuiLocalPath();
-    case FolderStatusDelegate::FolderSecondPathRole:
-        return f->remotePath();
-    case FolderStatusDelegate::FolderConflictMsg:
-        return (f->syncResult().hasUnresolvedConflicts())
-            ? QStringList(tr("There are unresolved conflicts. Click for details."))
-            : QStringList();
-    case FolderStatusDelegate::FolderErrorMsg: {
-        auto errors = f->syncResult().errorStrings();
-        const auto legacyError = FolderMan::instance()->unsupportedConfiguration(f->path());
-        if (!legacyError) {
-            // the error message might contain new lines, the delegate only expect multiple single line values
-            errors.append(legacyError.error().split(QLatin1Char('\n')));
+    const auto getSpace = [&]() -> GraphApi::Space * {
+        if (_accountState->supportsSpaces()) {
+            return _accountState->account()->spacesManager()->spaceByUrl(f->webDavUrl());
         }
-        return errors;
-    }
-    case FolderStatusDelegate::FolderInfoMsg:
-        return f->isReady() && f->virtualFilesEnabled() && f->vfs().mode() != Vfs::Mode::WindowsCfApi
-            ? QStringList(tr("Virtual file support is enabled."))
-            : QStringList();
-    case FolderStatusDelegate::SyncRunning:
-        return f->syncResult().status() == SyncResult::SyncRunning;
-    case FolderStatusDelegate::HeaderRole:
-        return f->shortGuiRemotePathOrAppName();
-    case FolderStatusDelegate::FolderAliasRole:
-        return f->alias();
-    case FolderStatusDelegate::FolderSyncPaused:
-        return f->syncPaused();
-    case FolderStatusDelegate::FolderAccountConnected:
-        return accountConnected;
+        return nullptr;
+    };
+
+    switch (role) {
+    case Qt::DisplayRole:
+        switch (column) {
+        case Columns::FolderPathRole:
+            return f->shortGuiLocalPath();
+        case Columns::FolderSecondPathRole:
+            return f->remotePath();
+        case Columns::FolderConflictMsg:
+            return (f->syncResult().hasUnresolvedConflicts())
+                ? QStringList(tr("There are unresolved conflicts. Click for details."))
+                : QStringList();
+        case Columns::FolderErrorMsg: {
+            auto errors = f->syncResult().errorStrings();
+            const auto legacyError = FolderMan::instance()->unsupportedConfiguration(f->path());
+            if (!legacyError) {
+                // the error message might contain new lines, the delegate only expect multiple single line values
+                errors.append(legacyError.error().split(QLatin1Char('\n')));
+            }
+            return errors;
+        }
+        case Columns::FolderInfoMsg:
+            return f->isReady() && f->virtualFilesEnabled() && f->vfs().mode() != Vfs::Mode::WindowsCfApi ? QStringList(tr("Virtual file support is enabled."))
+                                                                                                          : QStringList();
+        case Columns::SyncRunning:
+            return f->syncResult().status() == SyncResult::SyncRunning;
+        case Columns::HeaderRole: {
+            if (auto *space = getSpace()) {
+                return space->displayName();
+            }
+            return f->displayName();
+        }
+        case Columns::FolderImage:
+            if (auto *space = getSpace()) {
+                return space->image();
+            }
+            return Resources::getCoreIcon(QStringLiteral("folder-sync"));
+        case Columns::FolderSyncPaused:
+            return f->syncPaused();
+        case Columns::FolderAccountConnected:
+            return accountConnected;
+        case Columns::FolderStatusIconRole: {
+            auto status = f->syncResult();
+            if (!accountConnected) {
+                status.setStatus(SyncResult::Status::Offline);
+            } else if (f->syncPaused()) {
+                status.setStatus(SyncResult::Status::Paused);
+            }
+            return Theme::instance()->syncStateIconName(status);
+        }
+        case Columns::SyncProgressItemString:
+            return progress._progressString;
+        case Columns::WarningCount:
+            return progress._warningCount;
+        case Columns::SyncProgressOverallPercent:
+            return progress._overallPercent;
+        case Columns::SyncProgressOverallString:
+            return progress._overallSyncString;
+        case Columns::FolderSyncText: {
+            if (auto *space = getSpace()) {
+                if (!space->drive().getDescription().isEmpty()) {
+                    return space->drive().getDescription();
+                }
+            }
+            return tr("Local folder: %1").arg(f->shortGuiLocalPath());
+        }
+        case Columns::IsReady:
+            return f->isReady();
+        case Columns::IsDeployed:
+            return f->isDeployed();
+        case Columns::Priority:
+            return f->priority();
+        case Columns::QuotaTotal:
+            return QVariant::fromValue(getQuota(_accountState, f->webDavUrl(), Columns::QuotaTotal));
+        case Columns::QuotaUsed:
+            return QVariant::fromValue(getQuota(_accountState, f->webDavUrl(), Columns::QuotaUsed));
+        case Columns::IsUsingSpaces: // handled before
+            [[fallthrough]];
+        case Columns::ItemType: // handled before
+            [[fallthrough]];
+        case Columns::ColumnCount:
+            Q_UNREACHABLE();
+            break;
+        }
+        break;
     case Qt::ToolTipRole: {
-        QString toolTip;
         if (!progress.isNull()) {
             return progress._progressString;
         }
-        if (accountConnected)
-            toolTip = Theme::instance()->statusHeaderText(f->syncResult().status());
-        else
-            toolTip = tr("Signed out");
-        toolTip += "\n";
-        toolTip += folderInfo._folder->path();
-        return toolTip;
-    }
-    case FolderStatusDelegate::FolderStatusIconRole:
         if (accountConnected) {
-            auto theme = Theme::instance();
-            if (f->syncPaused()) {
-                return theme->folderDisabledIcon();
-            } else {
-                return theme->syncStateIcon(f->syncResult().status());
-            }
+            return tr("%1\n%2").arg(Theme::instance()->statusHeaderText(f->syncResult().status()), QDir::toNativeSeparators(folderInfo._folder->path()));
         } else {
-            return Theme::instance()->folderOfflineIcon();
+            return tr("Signed out\n%1").arg(QDir::toNativeSeparators(folderInfo._folder->path()));
         }
-    case FolderStatusDelegate::SyncProgressItemString:
-        return progress._progressString;
-    case FolderStatusDelegate::WarningCount:
-        return progress._warningCount;
-    case FolderStatusDelegate::SyncProgressOverallPercent:
-        return progress._overallPercent;
-    case FolderStatusDelegate::SyncProgressOverallString:
-        return progress._overallSyncString;
-    case FolderStatusDelegate::FolderSyncText:
-        return tr("Local folder: %1").arg(f->shortGuiLocalPath());
-    case FolderStatusDelegate::IsReady:
-        return f->isReady();
+    }
     }
     return QVariant();
 }
 
+Folder *FolderStatusModel::folder(const QModelIndex &index) const
+{
+    Q_ASSERT(checkIndex(index, QAbstractItemModel::CheckIndexOption::IndexIsValid));
+    return _folders.at(index.row())._folder;
+}
+
 bool FolderStatusModel::setData(const QModelIndex &index, const QVariant &value, int role)
 {
     if (role == Qt::CheckStateRole) {
@@ -336,7 +416,7 @@ bool FolderStatusModel::setData(const QModelIndex &index, const QVariant &value,
         }
         _dirty = true;
         emit dirtyChanged();
-        emit dataChanged(index, index, QVector<int>() << role);
+        emit dataChanged(index, index, { role });
         return true;
     }
     return QAbstractItemModel::setData(index, value, role);
@@ -345,17 +425,13 @@ bool FolderStatusModel::setData(const QModelIndex &index, const QVariant &value,
 
 int FolderStatusModel::columnCount(const QModelIndex &) const
 {
-    return 1;
+    return static_cast<int>(Columns::ColumnCount);
 }
 
 int FolderStatusModel::rowCount(const QModelIndex &parent) const
 {
     if (!parent.isValid()) {
-        if (Theme::instance()->singleSyncFolder() && _folders.count() != 0) {
-            // "Add folder" button not visible in the singleSyncFolder configuration.
-            return _folders.count();
-        }
-        return _folders.count() + 1; // +1 for the "add folder" button
+        return _folders.count();
     }
     auto info = infoForIndex(parent);
     if (!info)
@@ -374,10 +450,8 @@ FolderStatusModel::ItemType FolderStatusModel::classify(const QModelIndex &index
             return SubFolder;
         }
     }
-    if (index.row() < _folders.count()) {
-        return RootFolder;
-    }
-    return AddButton;
+
+    return RootFolder;
 }
 
 FolderStatusModel::SubFolderInfo *FolderStatusModel::infoForIndex(const QModelIndex &index) const
@@ -407,7 +481,7 @@ QModelIndex FolderStatusModel::indexForPath(Folder *f, const QString &path) cons
         return QModelIndex();
     }
 
-    int slashPos = path.lastIndexOf('/');
+    int slashPos = path.lastIndexOf(QLatin1Char('/'));
     if (slashPos == -1) {
         // first level folder
         for (int i = 0; i < _folders.size(); ++i) {
@@ -456,7 +530,6 @@ QModelIndex FolderStatusModel::index(int row, int column, const QModelIndex &par
         return createIndex(row, column /*, nullptr*/);
     }
     switch (classify(parent)) {
-    case AddButton:
     case FetchLabel:
         return QModelIndex();
     case RootFolder:
@@ -484,9 +557,9 @@ QModelIndex FolderStatusModel::parent(const QModelIndex &child) const
     }
     switch (classify(child)) {
     case RootFolder:
-    case AddButton:
         return QModelIndex();
     case SubFolder:
+        [[fallthrough]];
     case FetchLabel:
         break;
     }
@@ -553,6 +626,16 @@ bool FolderStatusModel::canFetchMore(const QModelIndex &parent) const
 
 void FolderStatusModel::fetchMore(const QModelIndex &parent)
 {
+    {
+        const auto isReady = parent.siblingAtColumn(static_cast<int>(Columns::IsReady)).data();
+
+        Q_ASSERT(isReady.isValid());
+
+        if (!isReady.toBool()) {
+            return;
+        }
+    }
+
     auto info = infoForIndex(parent);
 
     if (!info || info->_fetched || info->_fetchingJob)
@@ -562,17 +645,15 @@ void FolderStatusModel::fetchMore(const QModelIndex &parent)
     if (info->_path != QLatin1String("/")) {
         path += info->_path;
     }
-    LsColJob *job = new LsColJob(_accountState->account(), path, this);
+    PropfindJob *job = new PropfindJob(_accountState->account(), info->_folder->webDavUrl(), path, PropfindJob::Depth::One, this);
     info->_fetchingJob = job;
-    job->setProperties(QList<QByteArray>() << "resourcetype"
-                                           << "http://owncloud.org/ns:size"
-                                           << "http://owncloud.org/ns:permissions");
-    job->setTimeout(60 * 1000);
-    connect(job, &LsColJob::directoryListingSubfolders,
+    job->setProperties({ QByteArrayLiteral("resourcetype"), QByteArrayLiteral("http://owncloud.org/ns:size"), QByteArrayLiteral("http://owncloud.org/ns:permissions") });
+    job->setTimeout(60s);
+    connect(job, &PropfindJob::directoryListingSubfolders,
         this, &FolderStatusModel::slotUpdateDirectories);
-    connect(job, &LsColJob::finishedWithError,
+    connect(job, &PropfindJob::finishedWithError,
         this, &FolderStatusModel::slotLscolFinishedWithError);
-    connect(job, &LsColJob::directoryListingIterated,
+    connect(job, &PropfindJob::directoryListingIterated,
         this, &FolderStatusModel::slotGatherPermissions);
 
     job->start();
@@ -582,7 +663,7 @@ void FolderStatusModel::fetchMore(const QModelIndex &parent)
 
     // Show 'fetching data...' hint after a while.
     _fetchingItems[persistentIndex].start();
-    QTimer::singleShot(1000, this, &FolderStatusModel::slotShowFetchProgress);
+    QTimer::singleShot(1s, this, &FolderStatusModel::slotShowFetchProgress);
 }
 
 void FolderStatusModel::resetAndFetch(const QModelIndex &parent)
@@ -594,7 +675,7 @@ void FolderStatusModel::resetAndFetch(const QModelIndex &parent)
 
 void FolderStatusModel::slotGatherPermissions(const QString &href, const QMap<QString, QString> &map)
 {
-    auto it = map.find("permissions");
+    auto it = map.find(QStringLiteral("permissions"));
     if (it == map.end())
         return;
 
@@ -608,7 +689,7 @@ void FolderStatusModel::slotGatherPermissions(const QString &href, const QMap<QS
 
 void FolderStatusModel::slotUpdateDirectories(const QStringList &list)
 {
-    auto job = qobject_cast<LsColJob *>(sender());
+    auto job = qobject_cast<PropfindJob *>(sender());
     OC_ASSERT(job);
     QModelIndex idx = qvariant_cast<QPersistentModelIndex>(job->property(propertyParentIndexC));
     auto parentInfo = infoForIndex(idx);
@@ -634,8 +715,8 @@ void FolderStatusModel::slotUpdateDirectories(const QStringList &list)
 
     QUrl url = parentInfo->_folder->remoteUrl();
     QString pathToRemove = url.path();
-    if (!pathToRemove.endsWith('/'))
-        pathToRemove += '/';
+    if (!pathToRemove.endsWith(QLatin1Char('/')))
+        pathToRemove += QLatin1Char('/');
 
     QStringList selectiveSyncBlackList;
     bool ok1 = true;
@@ -678,9 +759,9 @@ void FolderStatusModel::slotUpdateDirectories(const QStringList &list)
         newInfo._pathIdx = parentInfo->_pathIdx;
         newInfo._pathIdx << newSubs.size();
         newInfo._size = job->sizes().value(path);
-        newInfo._isExternal = permissionMap.value(removeTrailingSlash(path)).toString().contains("M");
+        newInfo._isExternal = permissionMap.value(Utility::stripTrailingSlash(path)).toString().contains(QLatin1String("M"));
         newInfo._path = relativePath;
-        newInfo._name = removeTrailingSlash(relativePath).split('/').last();
+        newInfo._name = Utility::stripTrailingSlash(relativePath).split(QLatin1Char('/')).last();
 
         if (relativePath.isEmpty())
             continue;
@@ -719,13 +800,13 @@ void FolderStatusModel::slotUpdateDirectories(const QStringList &list)
     }
 
     if (!newSubs.isEmpty()) {
-        beginInsertRows(idx, 0, newSubs.size() - 1);
+        Q_EMIT beginInsertRows(idx, 0, newSubs.size() - 1);
         parentInfo->_subs = std::move(newSubs);
-        endInsertRows();
+        Q_EMIT endInsertRows();
     }
 
     for (auto it = undecidedIndexes.begin(); it != undecidedIndexes.end(); ++it) {
-        suggestExpand(index(*it, 0, idx));
+        Q_EMIT suggestExpand(index(*it, 0, idx));
     }
     /* Try to remove the the undecided lists the items that are not on the server. */
     auto it = std::remove_if(selectiveSyncUndecidedList.begin(), selectiveSyncUndecidedList.end(),
@@ -740,7 +821,7 @@ void FolderStatusModel::slotUpdateDirectories(const QStringList &list)
 
 void FolderStatusModel::slotLscolFinishedWithError(QNetworkReply *r)
 {
-    auto job = qobject_cast<LsColJob *>(sender());
+    auto job = qobject_cast<PropfindJob *>(sender());
     OC_ASSERT(job);
     QModelIndex idx = qvariant_cast<QPersistentModelIndex>(job->property(propertyParentIndexC));
     if (!idx.isValid()) {
@@ -860,61 +941,55 @@ void FolderStatusModel::slotApplySelectiveSync()
     resetFolders();
 }
 
-void FolderStatusModel::slotSetProgress(const ProgressInfo &progress)
+void FolderStatusModel::slotSetProgress(const ProgressInfo &progress, Folder *f)
 {
-    auto par = qobject_cast<QWidget *>(QObject::parent());
-    if (!par->isVisible()) {
+    if (!qobject_cast<QWidget *>(QObject::parent())->isVisible()) {
         return; // for https://github.com/owncloud/client/issues/2648#issuecomment-71377909
     }
 
-    Folder *f = qobject_cast<Folder *>(sender());
-    if (!f) {
-        return;
-    }
-
-    int folderIndex = -1;
-    for (int i = 0; i < _folders.count(); ++i) {
-        if (_folders.at(i)._folder == f) {
-            folderIndex = i;
-            break;
-        }
-    }
+    const int folderIndex = indexOf(f);
     if (folderIndex < 0) {
         return;
     }
+    auto &folder = _folders[folderIndex];
 
-    auto *pi = &_folders[folderIndex]._progress;
+    auto *pi = &folder._progress;
 
-    QVector<int> roles;
-    roles << FolderStatusDelegate::SyncProgressItemString
-          << FolderStatusDelegate::WarningCount
-          << Qt::ToolTipRole;
-
-    if (progress.status() == ProgressInfo::Discovery) {
-        if (!progress._currentDiscoveredRemoteFolder.isEmpty()) {
-            pi->_overallSyncString = tr("Checking for changes in remote '%1'").arg(progress._currentDiscoveredRemoteFolder);
-            emit dataChanged(index(folderIndex), index(folderIndex), roles);
-            return;
-        } else if (!progress._currentDiscoveredLocalFolder.isEmpty()) {
-            pi->_overallSyncString = tr("Checking for changes in local '%1'").arg(progress._currentDiscoveredLocalFolder);
-            emit dataChanged(index(folderIndex), index(folderIndex), roles);
-            return;
-        }
-    }
-
-    if (progress.status() == ProgressInfo::Reconcile) {
-        pi->_overallSyncString = tr("Reconciling changes");
-        emit dataChanged(index(folderIndex), index(folderIndex), roles);
-        return;
-    }
-
-    // Status is Starting, Propagation or Done
-
-    if (!progress._lastCompletedItem.isEmpty()
+    if (progress.status() == ProgressInfo::Done && !progress._lastCompletedItem.isEmpty()
         && Progress::isWarningKind(progress._lastCompletedItem._status)) {
         pi->_warningCount++;
     }
+    // depending on the use of virtual files or small files this slot might be called very often.
+    // throttle the model updates to prevent an needlessly high cpu usage used on ui updates.
+    if (folder._lastProgressUpdateStatus != progress.status() || (std::chrono::steady_clock::now() - folder._lastProgressUpdated > progressUpdateTimeOutC)) {
+        folder._lastProgressUpdateStatus = progress.status();
 
+        switch (progress.status()) {
+        case ProgressInfo::None:
+            Q_UNREACHABLE();
+            break;
+        case ProgressInfo::Discovery:
+            if (!progress._currentDiscoveredRemoteFolder.isEmpty()) {
+                pi->_overallSyncString = tr("Checking for changes in remote '%1'").arg(progress._currentDiscoveredRemoteFolder);
+            } else if (!progress._currentDiscoveredLocalFolder.isEmpty()) {
+                pi->_overallSyncString = tr("Checking for changes in local '%1'").arg(progress._currentDiscoveredLocalFolder);
+            }
+            break;
+        case ProgressInfo::Reconcile:
+            pi->_overallSyncString = tr("Reconciling changes");
+            break;
+        case ProgressInfo::Propagation:
+            Q_FALLTHROUGH();
+        case ProgressInfo::Done:
+            computeProgress(progress, pi);
+        }
+        emit dataChanged(index(folderIndex), index(folderIndex));
+        folder._lastProgressUpdated = std::chrono::steady_clock::now();
+    }
+}
+
+void FolderStatusModel::computeProgress(const ProgressInfo &progress, SubFolderInfo::Progress *pi)
+{
     // find the single item to display:  This is going to be the bigger item, or the last completed
     // item if no items are in progress.
     SyncFileItem curItem = progress._lastCompletedItem;
@@ -922,7 +997,7 @@ void FolderStatusModel::slotSetProgress(const ProgressInfo &progress)
     qint64 biggerItemSize = 0;
     quint64 estimatedUpBw = 0;
     quint64 estimatedDownBw = 0;
-    QString allFilenames;
+    QStringList allFilenames;
     for (const auto &citm : progress._currentItems) {
         if (curItemProgress == -1 || (ProgressInfo::isSizeDependent(citm._item)
                                          && biggerItemSize < citm._item._size)) {
@@ -936,25 +1011,17 @@ void FolderStatusModel::slotSetProgress(const ProgressInfo &progress)
             estimatedUpBw += progress.fileProgress(citm._item).estimatedBandwidth;
         }
         auto fileName = QFileInfo(citm._item._file).fileName();
-        if (allFilenames.length() > 0) {
-            //: Build a list of file names
-            allFilenames.append(tr(", '%1'").arg(fileName));
-        } else {
-            //: Argument is a file name
-            allFilenames.append(tr("'%1'").arg(fileName));
-        }
+        allFilenames.append(tr("'%1'").arg(fileName));
     }
     if (curItemProgress == -1) {
         curItemProgress = curItem._size;
     }
 
-    QString itemFileName = curItem._file;
-    QString kindString = Progress::asActionString(curItem);
+    const QString itemFileName = curItem._file;
+    const QString kindString = Progress::asActionString(curItem);
 
     QString fileProgressString;
     if (ProgressInfo::isSizeDependent(curItem)) {
-        QString s1 = Utility::octetsToString(curItemProgress);
-        QString s2 = Utility::octetsToString(curItem._size);
         //quint64 estimatedBw = progress.fileProgress(curItem).estimatedBandwidth;
         if (estimatedUpBw || estimatedDownBw) {
             /*
@@ -965,7 +1032,7 @@ void FolderStatusModel::slotSetProgress(const ProgressInfo &progress)
                     Utility::octetsToString(estimatedBw) );
             */
             //: Example text: "Syncing 'foo.txt', 'bar.txt'"
-            fileProgressString = tr("Syncing %1").arg(allFilenames);
+            fileProgressString = tr("Syncing %1").arg(allFilenames.join(QStringLiteral(", ")));
             if (estimatedDownBw > 0) {
                 fileProgressString.append(tr(", "));
 // ifdefs: https://github.com/owncloud/client/issues/3095#issuecomment-128409294
@@ -989,7 +1056,7 @@ void FolderStatusModel::slotSetProgress(const ProgressInfo &progress)
             }
         } else {
             //: Example text: "uploading foobar.png (2MB of 2MB)"
-            fileProgressString = tr("%1 %2 (%3 of %4)").arg(kindString, itemFileName, s1, s2);
+            fileProgressString = tr("%1 %2 (%3 of %4)").arg(kindString, itemFileName, Utility::octetsToString(curItemProgress), Utility::octetsToString(curItem._size));
         }
     } else if (!kindString.isEmpty()) {
         //: Example text: "uploading foobar.png"
@@ -1005,8 +1072,8 @@ void FolderStatusModel::slotSetProgress(const ProgressInfo &progress)
     qint64 totalFileCount = qMax(currentFile, progress.totalFiles());
     QString overallSyncString;
     if (totalSize > 0) {
-        QString s1 = Utility::octetsToString(completedSize);
-        QString s2 = Utility::octetsToString(totalSize);
+        const QString s1 = Utility::octetsToString(completedSize);
+        const QString s2 = Utility::octetsToString(totalSize);
 
         if (progress.trustEta()) {
             //: Example text: "5 minutes left, 12 MB of 345 MB, file 6 of 7"
@@ -1036,7 +1103,17 @@ void FolderStatusModel::slotSetProgress(const ProgressInfo &progress)
         overallPercent = qRound(double(completedSize + completedFile) / double(totalSize + totalFileCount) * 100.0);
     }
     pi->_overallPercent = qBound(0, overallPercent, 100);
-    emit dataChanged(index(folderIndex), index(folderIndex), roles);
+}
+
+int FolderStatusModel::indexOf(Folder *f) const
+{
+    const auto it = std::find_if(_folders.cbegin(), _folders.cend(), [f](auto &it) {
+        return it._folder == f;
+    });
+    if (it == _folders.cend()) {
+        return -1;
+    }
+    return std::distance(_folders.cbegin(), it);
 }
 
 void FolderStatusModel::slotFolderSyncStateChange(Folder *f)
@@ -1045,13 +1122,7 @@ void FolderStatusModel::slotFolderSyncStateChange(Folder *f)
         return;
     }
 
-    int folderIndex = -1;
-    for (int i = 0; i < _folders.count(); ++i) {
-        if (_folders.at(i)._folder == f) {
-            folderIndex = i;
-            break;
-        }
-    }
+    const int folderIndex = indexOf(f);
     if (folderIndex < 0) {
         return;
     }
@@ -1065,21 +1136,17 @@ void FolderStatusModel::slotFolderSyncStateChange(Folder *f)
     } else if (state == SyncResult::NotYetStarted) {
         FolderMan *folderMan = FolderMan::instance();
         int pos = folderMan->scheduleQueue().indexOf(f);
-        for (auto other : folderMan->map()) {
+        for (auto other : folderMan->folders()) {
             if (other != f && other->isSyncRunning())
                 pos += 1;
         }
-        QString message;
-        if (pos <= 0) {
-            message = tr("Waiting...");
-        } else {
-            message = tr("Waiting for %n other folder(s)...", "", pos);
+        if (pos > 0) {
+            pi._overallSyncString = tr("Waiting for %n other folder(s)...", "", pos);
         }
         pi = SubFolderInfo::Progress();
-        pi._overallSyncString = message;
     } else if (state == SyncResult::SyncPrepare) {
         pi = SubFolderInfo::Progress();
-        pi._overallSyncString = tr("Preparing to sync...");
+        pi._overallSyncString = Theme::instance()->statusHeaderText(SyncResult::SyncPrepare);
     } else if (state == SyncResult::Problem || state == SyncResult::Success) {
         // Reset the progress info after a sync.
         pi = SubFolderInfo::Progress();
@@ -1100,7 +1167,7 @@ void FolderStatusModel::slotFolderSyncStateChange(Folder *f)
 void FolderStatusModel::slotFolderScheduleQueueChanged()
 {
     // Update messages on waiting folders.
-    for (auto *f : FolderMan::instance()->map()) {
+    for (auto *f : FolderMan::instance()->folders()) {
         slotFolderSyncStateChange(f);
     }
 }
@@ -1189,13 +1256,7 @@ void FolderStatusModel::slotNewBigFolder()
     auto f = qobject_cast<Folder *>(sender());
     OC_ASSERT(f);
 
-    int folderIndex = -1;
-    for (int i = 0; i < _folders.count(); ++i) {
-        if (_folders.at(i)._folder == f) {
-            folderIndex = i;
-            break;
-        }
-    }
+    const int folderIndex = indexOf(f);
     if (folderIndex < 0) {
         return;
     }
@@ -1234,7 +1295,7 @@ bool FolderStatusModel::SubFolderInfo::hasLabel() const
     return _hasError || _fetchingLabel;
 }
 
-void FolderStatusModel::SubFolderInfo::resetSubs(FolderStatusModel *model, QModelIndex index)
+void FolderStatusModel::SubFolderInfo::resetSubs(FolderStatusModel *model, const QModelIndex &index)
 {
     _fetched = false;
     if (_fetchingJob) {
