@@ -50,6 +50,7 @@ public:
         FolderSyncPaused,
         FolderStatusIconRole,
         FolderAccountConnected,
+        FolderImage,
 
         SyncProgressOverallPercent,
         SyncProgressOverallString,
@@ -65,6 +66,9 @@ public:
         Priority, // uint32_t
         IsDeployed, // bool
 
+        QuotaUsed,
+        QuotaTotal,
+
         ColumnCount
     };
     Q_ENUMS(Columns);
@@ -155,7 +159,6 @@ public:
     enum ItemType {
         RootFolder,
         SubFolder,
-        AddButton,
         FetchLabel
     };
     Q_ENUM(ItemType);
