@@ -15,20 +15,23 @@
 #include "generalsettings.h"
 #include "ui_generalsettings.h"
 
-#include "theme.h"
-#include "version.h"
-#include "configfile.h"
-#include "application.h"
-#include "configfile.h"
-#include "owncloudsetupwizard.h"
 #include "accountmanager.h"
+#include "application.h"
+#include "common/version.h"
+#include "configfile.h"
+#include "theme.h"
 
+#include "gui/settingsdialog.h"
+
+#ifdef WITH_AUTO_UPDATER
 #include "updater/updater.h"
 #include "updater/ocupdater.h"
 #ifdef Q_OS_MAC
 // FIXME We should unify those, but Sparkle does everything behind the scene transparently
 #include "updater/sparkleupdater.h"
 #endif
+#endif
+
 #include "ignorelisteditor.h"
 
 #include "config.h"
@@ -49,15 +52,13 @@ GeneralSettings::GeneralSettings(QWidget *parent)
 {
     _ui->setupUi(this);
 
-    connect(_ui->desktopNotificationsCheckBox, &QAbstractButton::toggled,
-        this, &GeneralSettings::slotToggleOptionalDesktopNotifications);
+    connect(_ui->desktopNotificationsCheckBox, &QAbstractButton::toggled, this, &GeneralSettings::slotToggleOptionalDesktopNotifications);
 #ifdef Q_OS_WIN
     connect(_ui->showInExplorerNavigationPaneCheckBox, &QAbstractButton::toggled, this, &GeneralSettings::slotShowInExplorerNavigationPane);
 #endif
 
     reloadConfig();
     loadMiscSettings();
-    slotUpdateInfo();
 
     // misc
     connect(_ui->monoIconsCheckBox, &QAbstractButton::toggled, this, &GeneralSettings::saveMiscSettings);
@@ -79,9 +80,7 @@ GeneralSettings::GeneralSettings(QWidget *parent)
     /* the ignoreHiddenFiles flag is a folder specific setting, but for now, it is
      * handled globally. Save it to every folder that is defined.
      */
-    connect(_ui->syncHiddenFilesCheckBox, &QCheckBox::toggled, this,[](bool checked){
-        FolderMan::instance()->setIgnoreHiddenFiles(!checked);
-    });
+    connect(_ui->syncHiddenFilesCheckBox, &QCheckBox::toggled, this, [](bool checked) { FolderMan::instance()->setIgnoreHiddenFiles(!checked); });
 
 #ifndef WITH_CRASHREPORTER
     _ui->crashreporterCheckBox->setVisible(false);
@@ -112,6 +111,7 @@ GeneralSettings::GeneralSettings(QWidget *parent)
     connect(AccountManager::instance(), &AccountManager::accountAdded, this, &GeneralSettings::loadMiscSettings);
 
     // Only our standard brandings currently support beta channel
+#ifdef WITH_AUTO_UPDATER
     if (Theme::instance()->appName() != QLatin1String("testpilotcloud")) {
 #ifdef Q_OS_MAC
         // Because we don't have any statusString from the SparkleUpdater anyway we can hide the whole thing
@@ -124,12 +124,45 @@ GeneralSettings::GeneralSettings(QWidget *parent)
         }
 #endif
     }
+    // we want to attach the known english identifiers which are also used within the configuration file as user data inside the data model
+    // that way, when we intend to reset to the original selection when the dialog, we can look up the config file's stored value in the data model
+    _ui->updateChannel->addItem(tr("stable"), QStringLiteral("stable"));
+    _ui->updateChannel->addItem(tr("beta"), QStringLiteral("beta"));
+
+    if (!ConfigFile().skipUpdateCheck() && Updater::instance()) {
+        // Channel selection
+        _ui->updateChannel->setCurrentIndex(_ui->updateChannel->findData(ConfigFile().updateChannel()));
+        connect(_ui->updateChannel, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged), this, &GeneralSettings::slotUpdateChannelChanged);
+
+        // Note: the sparkle-updater is not an OCUpdater
+        if (auto *ocupdater = qobject_cast<OCUpdater *>(Updater::instance())) {
+            auto updateInfo = [ocupdater, this] {
+                _ui->updateStateLabel->setText(ocupdater->statusString());
+                _ui->restartButton->setVisible(ocupdater->downloadState() == OCUpdater::DownloadComplete);
+            };
+            connect(ocupdater, &OCUpdater::downloadStateChanged, this, updateInfo);
+            connect(_ui->restartButton, &QAbstractButton::clicked, ocupdater, &OCUpdater::slotStartInstaller);
+            connect(_ui->restartButton, &QAbstractButton::clicked, qApp, &QApplication::quit);
+            updateInfo();
+        }
+#ifdef HAVE_SPARKLE
+        if (SparkleUpdater *sparkleUpdater = qobject_cast<SparkleUpdater *>(Updater::instance())) {
+            _ui->updateStateLabel->setText(sparkleUpdater->statusString());
+            _ui->restartButton->setVisible(false);
+        }
+#endif
+    } else {
+        _ui->updaterWidget->hide();
+    }
+#else
+    _ui->updaterWidget->hide();
+#endif
     connect(_ui->about_pushButton, &QPushButton::clicked, this, &GeneralSettings::showAbout);
 
     if (!Theme::instance()->aboutShowCopyright()) {
         _ui->copyrightLabel->hide();
     }
-    if (Theme::instance()->forceVirtualFilesOption()) {
+    if (Theme::instance()->forceVirtualFilesOption() && VfsPluginManager::instance().bestAvailableVfsMode() == Vfs::WindowsCfApi) {
         _ui->groupBox_non_vfs->hide();
     }
 }
@@ -166,41 +199,16 @@ void GeneralSettings::showEvent(QShowEvent *)
     reloadConfig();
 }
 
-void GeneralSettings::slotUpdateInfo()
-{
-    if (ConfigFile().skipUpdateCheck() || !Updater::instance()) {
-        // updater disabled on compile
-        _ui->updaterWidget->setVisible(false);
-        return;
-    }
-
-    // Note: the sparkle-updater is not an OCUpdater
-    OCUpdater *ocupdater = qobject_cast<OCUpdater *>(Updater::instance());
-    if (ocupdater) {
-        connect(ocupdater, &OCUpdater::downloadStateChanged, this, &GeneralSettings::slotUpdateInfo, Qt::UniqueConnection);
-        connect(_ui->restartButton, &QAbstractButton::clicked, ocupdater, &OCUpdater::slotStartInstaller, Qt::UniqueConnection);
-        connect(_ui->restartButton, &QAbstractButton::clicked, qApp, &QApplication::quit, Qt::UniqueConnection);
-
-        _ui->updateStateLabel->setText(ocupdater->statusString());
-        _ui->restartButton->setVisible(ocupdater->downloadState() == OCUpdater::DownloadComplete);
-
-    }
-#if defined(Q_OS_MAC) && defined(HAVE_SPARKLE)
-    else if (SparkleUpdater *sparkleUpdater = qobject_cast<SparkleUpdater *>(Updater::instance())) {
-        _ui->updateStateLabel->setText(sparkleUpdater->statusString());
-        _ui->restartButton->setVisible(false);
-    }
-#endif
-
-    // Channel selection
-    _ui->updateChannel->setCurrentIndex(ConfigFile().updateChannel() == "beta" ? 1 : 0);
-    connect(_ui->updateChannel, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
-        this, &GeneralSettings::slotUpdateChannelChanged, Qt::UniqueConnection);
-}
-
 void GeneralSettings::slotUpdateChannelChanged(int index)
 {
-    QString channel = index == 0 ? QStringLiteral("stable") : QStringLiteral("beta");
+#ifdef WITH_AUTO_UPDATER
+    QString channel;
+    if (index < 0) {
+        // invalid index reset to stable
+        channel = QStringLiteral("stable");
+    } else {
+        channel = _ui->updateChannel->itemData(index).toString();
+    }
     if (channel == ConfigFile().updateChannel())
         return;
 
@@ -222,7 +230,7 @@ void GeneralSettings::slotUpdateChannelChanged(int index)
         this);
     auto acceptButton = msgBox->addButton(tr("Change update channel"), QMessageBox::AcceptRole);
     msgBox->addButton(tr("Cancel"), QMessageBox::RejectRole);
-    connect(msgBox, &QMessageBox::finished, msgBox, [this, channel, msgBox, acceptButton] {
+    connect(msgBox, &QMessageBox::finished, msgBox, [this, channel, msgBox, acceptButton, index] {
         msgBox->deleteLater();
         if (msgBox->clickedButton() == acceptButton) {
             ConfigFile().setUpdateChannel(channel);
@@ -237,10 +245,14 @@ void GeneralSettings::slotUpdateChannelChanged(int index)
             }
 #endif
         } else {
-            _ui->updateChannel->setCurrentText(ConfigFile().updateChannel());
+            const auto oldChannel = _ui->updateChannel->findData(ConfigFile().updateChannel());
+            Q_ASSERT(oldChannel >= 0);
+            Q_ASSERT(oldChannel <= 1);
+            _ui->updateChannel->setCurrentIndex(oldChannel);
         }
     });
     msgBox->open();
+#endif
 }
 
 void GeneralSettings::saveMiscSettings()
@@ -287,12 +299,10 @@ void GeneralSettings::slotShowInExplorerNavigationPane(bool checked)
 void GeneralSettings::slotIgnoreFilesEditor()
 {
     if (_ignoreEditor.isNull()) {
-        _ignoreEditor = new IgnoreListEditor(this);
+        _ignoreEditor = new IgnoreListEditor(ocApp()->gui()->settingsDialog());
         _ignoreEditor->setAttribute(Qt::WA_DeleteOnClose, true);
-        _ignoreEditor->open();
-    } else {
-        ownCloudGui::raiseDialog(_ignoreEditor);
     }
+    ownCloudGui::raiseDialog(_ignoreEditor);
 }
 
 void GeneralSettings::reloadConfig()
