@@ -106,12 +106,12 @@ QSize SelectiveSyncWidget::sizeHint() const
 
 void SelectiveSyncWidget::refreshFolders()
 {
-    LsColJob *job = new LsColJob(_account, _folderPath, this);
-    job->setProperties(QList<QByteArray>() << "resourcetype"
-                                           << "http://owncloud.org/ns:size");
-    connect(job, &LsColJob::directoryListingSubfolders,
+    // TODO: legacy
+    PropfindJob *job = new PropfindJob(_account, _account->davUrl(), _folderPath, PropfindJob::Depth::One, this);
+    job->setProperties({ QByteArrayLiteral("resourcetype"), QByteArrayLiteral("http://owncloud.org/ns:size") });
+    connect(job, &PropfindJob::directoryListingSubfolders,
         this, &SelectiveSyncWidget::slotUpdateDirectories);
-    connect(job, &LsColJob::finishedWithError,
+    connect(job, &PropfindJob::finishedWithError,
         this, &SelectiveSyncWidget::slotLscolFinishedWithError);
     job->start();
     _folderTree->clear();
@@ -143,7 +143,7 @@ void SelectiveSyncWidget::recursiveInsert(QTreeWidgetItem *parent, QStringList p
     QFileIconProvider prov;
     QIcon folderIcon = prov.icon(QFileIconProvider::Folder);
     if (pathTrail.size() == 0) {
-        if (path.endsWith('/')) {
+        if (path.endsWith(QLatin1Char('/'))) {
             path.chop(1);
         }
         parent->setToolTip(0, path);
@@ -156,7 +156,7 @@ void SelectiveSyncWidget::recursiveInsert(QTreeWidgetItem *parent, QStringList p
                 || parent->checkState(0) == Qt::PartiallyChecked) {
                 item->setCheckState(0, Qt::Checked);
                 for (const auto &str : qAsConst(_oldBlackList)) {
-                    if (str == path || str == QLatin1String("/")) {
+                    if (str == path || str == QLatin1Char('/')) {
                         item->setCheckState(0, Qt::Unchecked);
                         break;
                     } else if (str.startsWith(path)) {
@@ -183,7 +183,7 @@ void SelectiveSyncWidget::recursiveInsert(QTreeWidgetItem *parent, QStringList p
 
 void SelectiveSyncWidget::slotUpdateDirectories(QStringList list)
 {
-    auto job = qobject_cast<LsColJob *>(sender());
+    auto job = qobject_cast<PropfindJob *>(sender());
     QScopedValueRollback<bool> isInserting(_inserting, true);
 
     SelectiveSyncTreeViewItem *root = static_cast<SelectiveSyncTreeViewItem *>(_folderTree->topLevelItem(0));
@@ -198,7 +198,7 @@ void SelectiveSyncWidget::slotUpdateDirectories(QStringList list)
 
     // Since / cannot be in the blacklist, expand it to the actual
     // list of top-level folders as soon as possible.
-    if (_oldBlackList == QStringList("/")) {
+    if (_oldBlackList.size() == 1 && _oldBlackList.contains(QLatin1String("/"))) {
         _oldBlackList.clear();
         for (auto path : qAsConst(list)) {
             path.remove(pathToRemove);
@@ -234,13 +234,11 @@ void SelectiveSyncWidget::slotUpdateDirectories(QStringList list)
     for (auto path : qAsConst(list)) {
         auto size = job ? job->sizes().value(path) : 0;
         path.remove(pathToRemove);
-        QStringList paths = path.split('/');
-        if (paths.last().isEmpty())
-            paths.removeLast();
+        const QStringList paths = path.split(QLatin1Char('/'), Qt::SkipEmptyParts);
         if (paths.isEmpty())
             continue;
-        if (!path.endsWith('/')) {
-            path.append('/');
+        if (!path.endsWith(QLatin1Char('/'))) {
+            path.append(QLatin1Char('/'));
         }
         recursiveInsert(root, paths, path, size);
     }
@@ -272,10 +270,10 @@ void SelectiveSyncWidget::slotItemExpanded(QTreeWidgetItem *item)
     QString dir = item->data(0, Qt::UserRole).toString();
     if (dir.isEmpty())
         return;
-    LsColJob *job = new LsColJob(_account, _folderPath + dir, this);
-    job->setProperties(QList<QByteArray>() << "resourcetype"
-                                           << "http://owncloud.org/ns:size");
-    connect(job, &LsColJob::directoryListingSubfolders,
+    // TODO: legacy
+    PropfindJob *job = new PropfindJob(_account, _account->davUrl(), _folderPath + dir, PropfindJob::Depth::One, this);
+    job->setProperties({ QByteArrayLiteral("resourcetype"), QByteArrayLiteral("http://owncloud.org/ns:size") });
+    connect(job, &PropfindJob::directoryListingSubfolders,
         this, &SelectiveSyncWidget::slotUpdateDirectories);
     job->start();
 }
@@ -348,7 +346,7 @@ QStringList SelectiveSyncWidget::createBlackList(QTreeWidgetItem *root) const
 
     switch (root->checkState(0)) {
     case Qt::Unchecked:
-        return QStringList(root->data(0, Qt::UserRole).toString() + "/");
+        return QStringList(root->data(0, Qt::UserRole).toString() + QLatin1Char('/'));
     case Qt::Checked:
         return QStringList();
     case Qt::PartiallyChecked:
@@ -419,7 +417,7 @@ SelectiveSyncDialog::SelectiveSyncDialog(AccountPtr account, Folder *folder, QWi
     init(account);
     QStringList selectiveSyncList = _folder->journalDb()->getSelectiveSyncList(SyncJournalDb::SelectiveSyncBlackList, &ok);
     if (ok) {
-        _selectiveSync->setFolderInfo(_folder->remotePath(), _folder->alias(), selectiveSyncList);
+        _selectiveSync->setFolderInfo(_folder->remotePath(), QStringLiteral("/"), selectiveSyncList);
     } else {
         _okButton->setEnabled(false);
     }
@@ -427,13 +425,12 @@ SelectiveSyncDialog::SelectiveSyncDialog(AccountPtr account, Folder *folder, QWi
     connect(_folder, &QObject::destroyed, this, &QObject::deleteLater);
 }
 
-SelectiveSyncDialog::SelectiveSyncDialog(AccountPtr account, const QString &folder,
-    const QStringList &blacklist, QWidget *parent, Qt::WindowFlags f)
+SelectiveSyncDialog::SelectiveSyncDialog(AccountPtr account, const QString &folder, QWidget *parent, Qt::WindowFlags f)
     : QDialog(parent, f)
     , _folder(nullptr)
 {
     init(account);
-    _selectiveSync->setFolderInfo(folder, folder, blacklist);
+    _selectiveSync->setFolderInfo(folder, folder);
 }
 
 void SelectiveSyncDialog::init(const AccountPtr &account)
