@@ -29,6 +29,9 @@
 #include "accountmanager.h"
 #include "protocolwidget.h"
 
+#include "resources/resources.h"
+
+#include <QDesktopServices>
 #include <QImage>
 #include <QLabel>
 #include <QLayout>
@@ -53,6 +56,8 @@
 void setActivationPolicy(ActivationPolicy policy);
 #endif
 
+Q_LOGGING_CATEGORY(lcSettingsDialog, "gui.settingsdialog", QtInfoMsg);
+
 namespace {
 auto minimumSizeHint(const QWidget *w)
 {
@@ -164,7 +169,7 @@ public:
     void updateIcon()
     {
         if (!_iconName.isEmpty()) {
-            setIcon(Utility::getCoreIcon(_iconName));
+            setIcon(Resources::getCoreIcon(_iconName));
         }
     }
 
@@ -173,7 +178,7 @@ private:
 };
 
 SettingsDialog::SettingsDialog(ownCloudGui *gui, QWidget *parent)
-    : QMainWindow(parent, Qt::WindowFlags() & ~Qt::WindowContextHelpButtonHint)
+    : QMainWindow(parent)
     , _ui(new Ui::SettingsDialog)
     , _gui(gui)
 {
@@ -194,7 +199,7 @@ SettingsDialog::SettingsDialog(ownCloudGui *gui, QWidget *parent)
 
 
     if (Theme::instance()->multiAccount()) {
-        _addAccountAction = createActionWithIcon(QStringLiteral("plus-solid"), tr("Add account"));
+        _addAccountAction = new ToolButtonAction(QStringLiteral("plus-solid"), tr("Add account"), this);
         _addAccountAction->setCheckable(false);
         connect(_addAccountAction, &QAction::triggered, this, []{
             // don't directly connect here, ocApp might not be defined yet
@@ -205,7 +210,7 @@ SettingsDialog::SettingsDialog(ownCloudGui *gui, QWidget *parent)
 
     // Note: all the actions have a '\n' because the account name is in two lines and
     // all buttons must have the same size in order to keep a good layout
-    _activityAction = createActionWithIcon(QStringLiteral("activity"), tr("Activity"));
+    _activityAction = new ToolButtonAction(QStringLiteral("activity"), tr("Activity"), this);
     _actionGroup->addAction(_activityAction);
     _ui->toolBar->addAction(_activityAction);
     _activitySettings = new ActivitySettings;
@@ -216,7 +221,7 @@ SettingsDialog::SettingsDialog(ownCloudGui *gui, QWidget *parent)
         });
     _activitySettings->setNotificationRefreshInterval(cfg.notificationRefreshInterval());
 
-    QAction *generalAction = createActionWithIcon(QStringLiteral("settings"), tr("Settings"));
+    QAction *generalAction = new ToolButtonAction(QStringLiteral("settings"), tr("Settings"), this);
     _actionGroup->addAction(generalAction);
     _ui->toolBar->addAction(generalAction);
     GeneralSettings *generalSettings = new GeneralSettings;
@@ -229,17 +234,27 @@ SettingsDialog::SettingsDialog(ownCloudGui *gui, QWidget *parent)
 
     const auto appNameGui = Theme::instance()->appNameGUI();
 
-    QAction *quitAction = createActionWithIcon(QStringLiteral("quit"), tr("Quit %1").arg(appNameGui));
+    for (const auto &[iconName, name, url] : Theme::instance()->urlButtons()) {
+        auto urlAction = new ToolButtonAction(Theme::instance()->themeUniversalIcon(QStringLiteral("urlIcons/%1").arg(iconName)), name, this);
+        urlAction->setCheckable(false);
+        connect(urlAction, &QAction::triggered, this, [url = url] {
+            if (!QDesktopServices::openUrl(url)) {
+                qWarning(lcSettingsDialog) << "Failed to open" << url;
+            }
+        });
+        _ui->toolBar->addAction(urlAction);
+    }
+
+    QAction *quitAction = new ToolButtonAction(QStringLiteral("quit"), tr("Quit %1").arg(appNameGui), this);
     quitAction->setCheckable(false);
     connect(quitAction, &QAction::triggered, this, [this, appNameGui] {
         auto box = new QMessageBox(QMessageBox::Question, tr("Quit %1").arg(appNameGui),
             tr("Are you sure you want to quit %1?").arg(appNameGui), QMessageBox::Yes | QMessageBox::No, this);
         box->setAttribute(Qt::WA_DeleteOnClose);
-        Utility::setModal(box);
         connect(box, &QMessageBox::accepted, this, [] {
             qApp->quit();
         });
-        box->show();
+        box->open();
     });
     _ui->toolBar->addAction(quitAction);
 
@@ -363,8 +378,7 @@ void SettingsDialog::accountAdded(AccountStatePtr s)
     const QPixmap avatar = s->account()->avatar();
     const QString actionText = brandingSingleAccount ? tr("Account") : s->account()->displayName();
     if (avatar.isNull()) {
-        accountAction = createActionWithIcon(QStringLiteral("account"),
-            actionText);
+        accountAction = new ToolButtonAction(QStringLiteral("account"), actionText, this);
     } else {
         const QIcon icon(AvatarJob::makeCircularAvatar(avatar));
         accountAction = new ToolButtonAction(icon, actionText, this);
@@ -465,13 +479,6 @@ void SettingsDialog::customizeStyle()
     }
 }
 
-
-QAction *SettingsDialog::createActionWithIcon(const QString &icon, const QString &text)
-{
-    QAction *action = new ToolButtonAction(icon, text, this);
-    return action;
-}
-
 void SettingsDialog::slotRefreshActivityAccountStateSender()
 {
     AccountStatePtr accountState(qobject_cast<AccountState *>(sender()));
