@@ -225,7 +225,7 @@ void logCredentialsJobResult(CredentialJob *credentialsJob)
     qCDebug(lcOauth) << "credentials job has finished";
 
     if (!credentialsJob->data().isValid()) {
-        qCCritical(lcOauth) << "Failed to read client id" << credentialsJob->errorString();
+        qCInfo(lcOauth) << "Failed to read client id" << credentialsJob->errorString();
     }
 }
 }
@@ -374,47 +374,55 @@ void OAuth::startAuthentication()
                         return;
                     }
 
-                    auto *job = FetchUserInfoJobFactory::fromOAuth2Credentials(_networkAccessManager, accessToken, this).startJob(_serverUrl);
+                    if (!_davUser.isEmpty()) {
+                        auto *job = FetchUserInfoJobFactory::fromOAuth2Credentials(_networkAccessManager, accessToken).startJob(_serverUrl, this);
 
-                    connect(job, &CoreJob::finished, this, [=]() {
-                        if (!job->success()) {
-                            httpReplyAndClose(socket, QStringLiteral("500 Internal Server Error"),
-                                tr("Login Error"), tr("<h1>Login Error</h1><p>%1</p>").arg(job->errorMessage()));
-                            emit result(Error);
-                        } else {
-                            auto result = job->result().value<FetchUserInfoResult>();
-                            finalize(socket, accessToken, refreshToken, result.userName(), result.displayName(), messageUrl);
-                        }
-                    });
+                        connect(job, &CoreJob::finished, this, [=]() {
+                            if (!job->success()) {
+                                httpReplyAndClose(socket, QStringLiteral("500 Internal Server Error"), tr("Login Error"),
+                                    tr("<h1>Login Error</h1><p>%1</p>").arg(job->errorMessage()));
+                                emit result(Error);
+                            } else {
+                                auto fetchUserInfo = job->result().value<FetchUserInfoResult>();
+
+                                // note: the username still shouldn't be empty
+                                Q_ASSERT(!_davUser.isEmpty());
+
+                                // dav usernames are case-insensitive, we might compare a user input with the string provided by the server
+                                if (fetchUserInfo.userName().compare(_davUser, Qt::CaseInsensitive) != 0) {
+                                    // Connected with the wrong user
+                                    qCWarning(lcOauth) << "We expected the user" << _davUser << "but the server answered with user" << fetchUserInfo.userName();
+                                    const QString message = tr("<h1>Wrong user</h1>"
+                                                               "<p>You logged-in with user <em>%1</em>, but must login with user <em>%2</em>.<br>"
+                                                               "Please return to the %3 client and restart the authentication.</p>")
+                                                                .arg(fetchUserInfo.userName(), _davUser, Theme::instance()->appNameGUI());
+                                    httpReplyAndClose(socket, QStringLiteral("403 Forbidden"), tr("Wrong user"), message);
+                                    emit result(Error);
+                                } else {
+                                    finalize(socket, accessToken, refreshToken, messageUrl);
+                                }
+                            }
+                        });
+                    } else {
+                        finalize(socket, accessToken, refreshToken, messageUrl);
+                    }
                 });
             });
         }
     });
 }
 
-void OAuth::finalize(const QPointer<QTcpSocket> &socket, const QString &accessToken, const QString &refreshToken, const QString &userName, const QString &displayName, const QUrl &messageUrl)
+void OAuth::finalize(const QPointer<QTcpSocket> &socket, const QString &accessToken, const QString &refreshToken, const QUrl &messageUrl)
 {
-    // dav user names are case insensetive, we might compare a user input with the string provided by the server
-    if (!_davUser.isEmpty() && userName.compare(_davUser, Qt::CaseInsensitive) != 0) {
-        // Connected with the wrong user
-        qCWarning(lcOauth) << "We expected the user" << _davUser << "but the server answered with user" << userName;
-        const QString message = tr("<h1>Wrong user</h1>"
-                                   "<p>You logged-in with user <em>%1</em>, but must login with user <em>%2</em>.<br>"
-                                   "Please return to the %3 client and restart the authentication.</p>")
-                                    .arg(userName, _davUser, Theme::instance()->appNameGUI());
-        httpReplyAndClose(socket, QStringLiteral("403 Forbidden"), tr("Wrong user"), message);
-        emit result(Error);
-        return;
-    }
-    const QString loginSuccessfullHtml = tr("<h1>Login Successful</h1><p>You can close this window.</p>");
-    const QString loginSuccessfullTitle = tr("Login Successful");
+    const QString loginSuccessfulHtml = tr("<h1>Login Successful</h1><p>You can close this window.</p>");
+    const QString loginSuccessfulTitle = tr("Login Successful");
     if (messageUrl.isValid()) {
-        httpReplyAndClose(socket, QStringLiteral("303 See Other"), loginSuccessfullTitle, loginSuccessfullHtml,
-            { QStringLiteral("Location: %1").arg(QString::fromUtf8(messageUrl.toEncoded())) });
+        httpReplyAndClose(socket, QStringLiteral("303 See Other"), loginSuccessfulTitle, loginSuccessfulHtml,
+            {QStringLiteral("Location: %1").arg(QString::fromUtf8(messageUrl.toEncoded()))});
     } else {
-        httpReplyAndClose(socket, QStringLiteral("200 OK"), loginSuccessfullTitle, loginSuccessfullHtml);
+        httpReplyAndClose(socket, QStringLiteral("200 OK"), loginSuccessfulTitle, loginSuccessfulHtml);
     }
-    emit result(LoggedIn, userName, accessToken, refreshToken, displayName);
+    emit result(LoggedIn, accessToken, refreshToken);
 }
 
 QNetworkReply *OAuth::postTokenRequest(const QList<QPair<QString, QString>> &queryItems)
@@ -607,7 +615,7 @@ void AccountBasedOAuth::fetchWellKnown()
 {
     qCDebug(lcOauth) << "starting CheckServerJob before fetching" << wellKnownPathC;
 
-    auto *checkServerJob = CheckServerJobFactory(_networkAccessManager, this).startJob(_serverUrl);
+    auto *checkServerJob = CheckServerJobFactory::createFromAccount(_account, true, this).startJob(_serverUrl, this);
 
     connect(checkServerJob, &CoreJob::finished, this, [checkServerJob, this]() {
         if (checkServerJob->success()) {
@@ -664,7 +672,7 @@ void AccountBasedOAuth::refreshAuthentication(const QString &refreshToken)
                         qCWarning(lcOauth) << "Error while refreshing the token:" << error << data.value(QStringLiteral("error_description")).toString();
                     }
                 } else if (reply->error() != QNetworkReply::NoError) {
-                    qCWarning(lcOauth) << "Error while refreshing the token:" << reply->error() << ":" << reply->errorString() << reply->attribute(QNetworkRequest::HttpStatusCodeAttribute);
+                    qCWarning(lcOauth) << "Error while refreshing the token:" << reply->error() << ":" << reply->errorString() << reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
                     Q_EMIT refreshError(reply->error(), reply->errorString());
                     return;
                 } else {
