@@ -77,7 +77,7 @@ bool DiscoveryPhase::isInSelectiveSyncBlackList(const QString &path) const
 }
 
 void DiscoveryPhase::checkSelectiveSyncNewFolder(const QString &path, RemotePermissions remotePerm,
-    std::function<void(bool)> callback)
+    const std::function<void(bool)> &callback)
 {
     if (_syncOptions._confirmExternalStorage && _syncOptions._vfs->mode() == Vfs::Off
         && remotePerm.hasPermission(RemotePermissions::IsMounted)) {
@@ -108,12 +108,12 @@ void DiscoveryPhase::checkSelectiveSyncNewFolder(const QString &path, RemotePerm
     }
 
     // do a PROPFIND to know the size of this folder
-    auto propfindJob = new PropfindJob(_account, _remoteFolder + path, this);
+    auto propfindJob = new PropfindJob(_account, _baseUrl, _remoteFolder + path, PropfindJob::Depth::Zero, this);
     propfindJob->setProperties(QList<QByteArray>() << "resourcetype"
                                                    << "http://owncloud.org/ns:size");
     QObject::connect(propfindJob, &PropfindJob::finishedWithError,
         this, [=] { return callback(false); });
-    QObject::connect(propfindJob, &PropfindJob::result, this, [=](const QMap<QString, QString> &values) {
+    QObject::connect(propfindJob, &PropfindJob::directoryListingIterated, this, [=](const QString, const QMap<QString, QString> &values) {
         auto result = values.value(QStringLiteral("size")).toLongLong();
         if (result >= limit) {
             // we tell the UI there is a new folder
@@ -121,7 +121,7 @@ void DiscoveryPhase::checkSelectiveSyncNewFolder(const QString &path, RemotePerm
             return callback(true);
         } else {
             // it is not too big, put it in the white list (so we will not do more query for the children)
-            // and and do not block.
+            // and do not block.
             auto p = path;
             if (!p.endsWith(QLatin1Char('/')))
                 p += QLatin1Char('/');
@@ -140,7 +140,7 @@ QString DiscoveryPhase::adjustRenamedPath(const QString &original, SyncFileItem:
     return OCC::adjustRenamedPath(d == SyncFileItem::Down ? _renamedItemsRemote : _renamedItemsLocal, original);
 }
 
-QString adjustRenamedPath(const QMap<QString, QString> &renamedItems, const QString &original)
+QString adjustRenamedPath(const QHash<QString, QString> &renamedItems, const QString &original)
 {
     int slashPos = original.size();
     while ((slashPos = original.lastIndexOf(QLatin1Char('/'), slashPos - 1)) > 0) {
@@ -152,12 +152,12 @@ QString adjustRenamedPath(const QMap<QString, QString> &renamedItems, const QStr
     return original;
 }
 
-QPair<bool, QByteArray> DiscoveryPhase::findAndCancelDeletedJob(const QString &originalPath)
+QPair<bool, QString> DiscoveryPhase::findAndCancelDeletedJob(const QString &originalPath)
 {
     bool result = false;
-    QByteArray oldEtag;
-    auto it = _deletedItem.find(originalPath);
-    if (it != _deletedItem.end()) {
+    QString oldEtag;
+    auto it = _deletedItem.constFind(originalPath);
+    if (it != _deletedItem.cend()) {
         const auto &item = *it;
         const SyncInstructions instruction = item->_instruction;
         if (instruction == CSYNC_INSTRUCTION_IGNORE && item->_type == ItemTypeVirtualFile) {
@@ -246,8 +246,17 @@ void DiscoveryPhase::scheduleMoreJobs()
     }
 }
 
+bool DiscoveryPhase::isSpace() const
+{
+    return !(Utility::urlEqual(_account->davUrl(), _baseUrl) || _account->davUrl().isParentOf(_baseUrl));
+}
+
 DiscoverySingleLocalDirectoryJob::DiscoverySingleLocalDirectoryJob(const AccountPtr &account, const QString &localPath, OCC::Vfs *vfs, QObject *parent)
- : QObject(parent), QRunnable(), _localPath(localPath), _account(account), _vfs(vfs)
+    : QObject(parent)
+    , QRunnable()
+    , _localPath(localPath)
+    , _account(account)
+    , _vfs(vfs)
 {
     qRegisterMetaType<QVector<LocalInfo> >("QVector<LocalInfo>");
 }
@@ -260,7 +269,7 @@ void DiscoverySingleLocalDirectoryJob::run() {
 
     auto dh = csync_vio_local_opendir(localPath);
     if (!dh) {
-        qCInfo(lcDiscovery) << "Error while opening directory" << (localPath) << errno;
+        qCCritical(lcDiscovery) << "Error while opening directory" << (localPath) << errno;
         QString errorString = tr("Error while opening directory %1").arg(localPath);
         if (errno == EACCES) {
             errorString = tr("Directory not accessible on client, permission denied");
@@ -286,22 +295,7 @@ void DiscoverySingleLocalDirectoryJob::run() {
         if (dirent->type == ItemTypeSkip)
             continue;
         LocalInfo i;
-        static QTextCodec *codec = QTextCodec::codecForName("UTF-8");
-        OC_ASSERT(codec);
-        QTextCodec::ConverterState state;
-        i.name = codec->toUnicode(dirent->path.constData(), dirent->path.size(), &state);
-        if (state.invalidChars > 0 || state.remainingChars > 0) {
-            emit childIgnored(true);
-            auto item = SyncFileItemPtr::create();
-            //item->_file = _currentFolder._target + i.name;
-            // FIXME ^^ do we really need to use _target or is local fine?
-            item->_file = _localPath + i.name;
-            item->_instruction = CSYNC_INSTRUCTION_IGNORE;
-            item->_status = SyncFileItem::NormalError;
-            item->_errorString = tr("Filename encoding is not valid");
-            emit itemDiscovered(item);
-            continue;
-        }
+        i.name = dirent->path;
         i.modtime = dirent->modtime;
         i.size = dirent->size;
         i.inode = dirent->inode;
@@ -330,10 +324,11 @@ void DiscoverySingleLocalDirectoryJob::run() {
     emit finished(results);
 }
 
-DiscoverySingleDirectoryJob::DiscoverySingleDirectoryJob(const AccountPtr &account, const QString &path, QObject *parent)
+DiscoverySingleDirectoryJob::DiscoverySingleDirectoryJob(const AccountPtr &account, const QUrl &baseUrl, const QString &path, QObject *parent)
     : QObject(parent)
     , _subPath(path)
     , _account(account)
+    , _baseUrl(baseUrl)
     , _ignoredFirst(false)
     , _isRootPath(false)
     , _isExternalStorage(false)
@@ -343,78 +338,74 @@ DiscoverySingleDirectoryJob::DiscoverySingleDirectoryJob(const AccountPtr &accou
 void DiscoverySingleDirectoryJob::start()
 {
     // Start the actual HTTP job
-    LsColJob *lsColJob = new LsColJob(_account, _subPath, this);
+    _proFindJob = new PropfindJob(_account, _baseUrl, _subPath, PropfindJob::Depth::One, this);
 
-    QList<QByteArray> props;
-    props << "resourcetype"
-          << "getlastmodified"
-          << "getcontentlength"
-          << "getetag"
-          << "http://owncloud.org/ns:id"
-          << "http://owncloud.org/ns:downloadURL"
-          << "http://owncloud.org/ns:dDC"
-          << "http://owncloud.org/ns:permissions"
-          << "http://owncloud.org/ns:checksums";
-    if (_isRootPath)
+    QList<QByteArray> props {
+        "resourcetype",
+        "getlastmodified",
+        "getcontentlength",
+        "getetag",
+        "http://owncloud.org/ns:id",
+        "http://owncloud.org/ns:downloadURL",
+        "http://owncloud.org/ns:dDC",
+        "http://owncloud.org/ns:permissions",
+        "http://owncloud.org/ns:checksums",
+        "http://owncloud.org/ns:share-types"
+    };
+    if (_isRootPath) {
         props << "http://owncloud.org/ns:data-fingerprint";
-    if (_account->serverVersionInt() >= Account::makeServerVersion(10, 0, 0)) {
-        // Server older than 10.0 have performances issue if we ask for the share-types on every PROPFIND
-        props << "http://owncloud.org/ns:share-types";
     }
 
-    lsColJob->setProperties(props);
 
-    QObject::connect(lsColJob, &LsColJob::directoryListingIterated,
+    _proFindJob->setProperties(props);
+
+    QObject::connect(_proFindJob, &PropfindJob::directoryListingIterated,
         this, &DiscoverySingleDirectoryJob::directoryListingIteratedSlot);
-    QObject::connect(lsColJob, &LsColJob::finishedWithError, this, &DiscoverySingleDirectoryJob::lsJobFinishedWithErrorSlot);
-    QObject::connect(lsColJob, &LsColJob::finishedWithoutError, this, &DiscoverySingleDirectoryJob::lsJobFinishedWithoutErrorSlot);
-    lsColJob->start();
-
-    _lsColJob = lsColJob;
+    QObject::connect(_proFindJob, &PropfindJob::finishedWithError, this, &DiscoverySingleDirectoryJob::lsJobFinishedWithErrorSlot);
+    QObject::connect(_proFindJob, &PropfindJob::finishedWithoutError, this, &DiscoverySingleDirectoryJob::lsJobFinishedWithoutErrorSlot);
+    _proFindJob->start();
 }
 
 void DiscoverySingleDirectoryJob::abort()
 {
-    if (_lsColJob && _lsColJob->reply()) {
-        _lsColJob->reply()->abort();
+    if (_proFindJob) {
+        _proFindJob->abort();
     }
 }
 
 static void propertyMapToRemoteInfo(const QMap<QString, QString> &map, RemoteInfo &result)
 {
-    for (auto it = map.constBegin(); it != map.constEnd(); ++it) {
-        QString property = it.key();
-        QString value = it.value();
-        if (property == QLatin1String("resourcetype")) {
-            result.isDirectory = value.contains(QLatin1String("collection"));
-        } else if (property == QLatin1String("getlastmodified")) {
-            const auto date = QDateTime::fromString(value, Qt::RFC2822Date);
-            Q_ASSERT(date.isValid());
-            result.modtime = date.toTime_t();
-        } else if (property == QLatin1String("getcontentlength")) {
-            // See #4573, sometimes negative size values are returned
-            bool ok = false;
-            qlonglong ll = value.toLongLong(&ok);
-            if (ok && ll >= 0) {
-                result.size = ll;
-            } else {
-                result.size = 0;
-            }
-        } else if (property == QLatin1String("getetag")) {
-            result.etag = Utility::normalizeEtag(value.toUtf8());
-        } else if (property == QLatin1String("id")) {
-            result.fileId = value.toUtf8();
-        } else if (property == QLatin1String("downloadURL")) {
-            result.directDownloadUrl = value;
-        } else if (property == QLatin1String("dDC")) {
-            result.directDownloadCookies = value;
-        } else if (property == QLatin1String("permissions")) {
-            result.remotePerm = RemotePermissions::fromServerString(value);
-        } else if (property == QLatin1String("checksums")) {
-            result.checksumHeader = findBestChecksum(value.toUtf8());
-        } else if (property == QLatin1String("share-types") && !value.isEmpty()) {
-            // Since QMap is sorted, "share-types" is always after "permissions".
-            if (result.remotePerm.isNull()) {
+    result.directDownloadUrl = map.value(QStringLiteral("downloadURL"));
+    result.directDownloadCookies = map.value(QStringLiteral("dDC"));
+
+    if (auto it = Utility::optionalFind(map, QStringLiteral("resourcetype"))) {
+        result.isDirectory = it->value().contains(QStringLiteral("collection"));
+    }
+    if (auto it = Utility::optionalFind(map, QStringLiteral("getlastmodified"))) {
+        const auto date = QDateTime::fromString(**it, Qt::RFC2822Date);
+        Q_ASSERT(date.isValid());
+        result.modtime = date.toTime_t();
+    }
+    if (auto it = Utility::optionalFind(map, QStringLiteral("getcontentlength"))) {
+        // See #4573, sometimes negative size values are returned
+        result.size = std::max<int64_t>(0, it->value().toLongLong());
+    }
+    if (auto it = Utility::optionalFind(map, QStringLiteral("getetag"))) {
+        result.etag = Utility::normalizeEtag(it->value());
+    }
+    if (auto it = Utility::optionalFind(map, QStringLiteral("id"))) {
+        result.fileId = it->value().toUtf8();
+    }
+    if (auto it = Utility::optionalFind(map, QStringLiteral("checksums"))) {
+        result.checksumHeader = findBestChecksum(it->value().toUtf8());
+    }
+    if (auto it = Utility::optionalFind(map, QStringLiteral("permissions"))) {
+        result.remotePerm = RemotePermissions::fromServerString(it->value());
+    }
+    if (auto it = Utility::optionalFind(map, QStringLiteral("share-types"))) {
+        const QString &value = it->value();
+        if (!value.isEmpty()) {
+            if (!map.contains(QStringLiteral("permissions"))) {
                 qWarning() << "Server returned a share type, but no permissions?";
                 // Empty permissions will cause a sync failure
             } else {
@@ -433,13 +424,13 @@ void DiscoverySingleDirectoryJob::directoryListingIteratedSlot(const QString &fi
     if (!_ignoredFirst) {
         // The first entry is for the folder itself, we should process it differently.
         _ignoredFirst = true;
-        if (map.contains(QStringLiteral("permissions"))) {
-            auto perm = RemotePermissions::fromServerString(map.value(QStringLiteral("permissions")));
+        if (auto it = Utility::optionalFind(map, QStringLiteral("permissions"))) {
+            auto perm = RemotePermissions::fromServerString(it->value());
             emit firstDirectoryPermissions(perm);
             _isExternalStorage = perm.hasPermission(RemotePermissions::IsMounted);
         }
-        if (map.contains(QStringLiteral("data-fingerprint"))) {
-            _dataFingerprint = map.value(QStringLiteral("data-fingerprint")).toUtf8();
+        if (auto it = Utility::optionalFind(map, QStringLiteral("data-fingerprint"))) {
+            _dataFingerprint = it->value().toUtf8();
             if (_dataFingerprint.isEmpty()) {
                 // Placeholder that means that the server supports the feature even if it did not set one.
                 _dataFingerprint = "[empty]";
@@ -466,9 +457,9 @@ void DiscoverySingleDirectoryJob::directoryListingIteratedSlot(const QString &fi
     }
 
     //This works in concerto with the RequestEtagJob and the Folder object to check if the remote folder changed.
-    if (map.contains(QStringLiteral("getetag"))) {
-        if (_firstEtag.isEmpty()) {
-            _firstEtag = parseEtag(map.value(QStringLiteral("getetag")).toUtf8()); // for directory itself
+    if (_firstEtag.isEmpty()) {
+        if (auto it = Utility::optionalFind(map, QStringLiteral("getetag"))) {
+            _firstEtag = Utility::normalizeEtag(it->value()); // for directory itself
         }
     }
 }
@@ -486,7 +477,7 @@ void DiscoverySingleDirectoryJob::lsJobFinishedWithoutErrorSlot()
         deleteLater();
         return;
     }
-    emit etag(_firstEtag, QDateTime::fromString(QString::fromUtf8(_lsColJob->responseTimestamp()), Qt::RFC2822Date));
+    emit etag(_firstEtag, QDateTime::fromString(QString::fromUtf8(_proFindJob->responseTimestamp()), Qt::RFC2822Date));
     emit finished(_results);
     deleteLater();
 }
@@ -501,6 +492,8 @@ void DiscoverySingleDirectoryJob::lsJobFinishedWithErrorSlot(QNetworkReply *r)
         && !contentType.contains(QLatin1String("application/xml; charset=utf-8"))) {
         msg = tr("Server error: PROPFIND reply is not XML formatted!");
 
+    } else if (_proFindJob->timedOut()) {
+        msg = tr("Connection timed out");
     }
     emit finished(HttpError{ httpCode, msg });
     deleteLater();
