@@ -384,70 +384,116 @@ qint64 OwncloudPropagator::smallFileSize()
     return smallFileSize;
 }
 
+/**
+ * This builds all the jobs needed for the propagation.
+ * Each directory is a PropagateDirectory job, which contains the files in it.
+ */
 void OwncloudPropagator::start(SyncFileItemSet &&items)
 {
-    /* This builds all the jobs needed for the propagation.
-     * Each directory is a PropagateDirectory job, which contains the files in it.
-     * In order to do that we loop over the items. (which are sorted by destination)
-     * When we enter a directory, we can create the directory job and push it on the stack. */
+    // The items list is sorted in such a way that an item for a directory come before any items
+    // inside that directory. For example:
+    //  - A       <--- directory
+    //  - A/B     <--- directory
+    //  - A/B/1   <--- file
+    //  - A/2     <--- file
+    // This is important, because we rely on the fact that all actions for items inside a directory
+    // are grouped together, and that an action on the directory itself preceeds the actions for
+    // its child items.
 
     _rootJob.reset(new PropagateRootDirectory(this));
-    QStack<QPair<QString /* directory name */, PropagateDirectory * /* job */>> directories;
-    directories.push(qMakePair(QString(), _rootJob.data()));
-    QVector<PropagatorJob *> directoriesToRemove;
-    QString removedDirectory;
-    QString maybeConflictDirectory;
-    for (const auto &item : qAsConst(items)) {
-        if (!removedDirectory.isEmpty() && item->_file.startsWith(removedDirectory)) {
-            // this is an item in a directory which is going to be removed.
-            PropagateDirectory *delDirJob = qobject_cast<PropagateDirectory *>(directoriesToRemove.first());
 
+    // The algorithm could be done recursively, but the implementation is done iteratively in order
+    // to prevent us running out of stack space. So the next 3 variables are used to maintain the
+    // state.
+
+    // The directories stack below has the current directory on top of the stack. So if we're
+    // processing directory `/A/B/C`, then the stack looks like this:
+    //  - PropagateDirectory job for A/B/C  <--- top-of-stack
+    //  - PropagateDirectory job for A/B
+    //  - PropagateDirectory job for A
+    //  - PropagateDirectory job for /      <--- bottom-of-stack
+    //
+    // IMPORTANT: this parent stack is also used when a directory is removed
+    // (`CSYNC_INSTRUCTION_REMOVE` in the last step below), in order to suppress updating the etag
+    // of the parents of the to-be-removed directory. See the comment in the code for more information.
+    QStack<QPair<QString /* directory name */, PropagateDirectory * /* job */>> directories;
+    directories.push({QString(), _rootJob.data()});
+
+    // Due to the ordering of the items, it can happen that we have the following list:
+    //  - remove directory A
+    //  - remove file A/a
+    //  - etc
+    // So when we find a have a sync instruction to remove a directory, we remember it in the
+    // `currentRemoveDirectoryJob` variable, so we know/see it when processing the next item
+    // in the list.
+    PropagatorJob *currentRemoveDirectoryJob = nullptr;
+
+    // We skip items inside conflict directories. So, when we see an item that's marked as such,
+    // remember its name to see if in the next iteration, we will hit an item for that directory.
+    // See the `else` statment in the second step.
+    QString maybeConflictDirectory;
+
+    for (const auto &item : qAsConst(items)) {
+        // First check if this is an item in a directory which is going to be removed.
+        if (currentRemoveDirectoryJob && FileSystem::isChildPathOf(item->_file, currentRemoveDirectoryJob->path())) {
+            // Check the sync instruction for the item:
             if (item->_instruction == CSYNC_INSTRUCTION_REMOVE) {
                 // already taken care of. (by the removal of the parent directory)
 
-                // increase the number of subjobs that would be there.
-                if (delDirJob) {
+                if (auto delDirJob = qobject_cast<PropagateDirectory *>(currentRemoveDirectoryJob)) {
+                    // increase the number of subjobs that would be there.
                     delDirJob->increaseAffectedCount();
                 }
                 continue;
             } else if (item->isDirectory()
                 && (item->_instruction == CSYNC_INSTRUCTION_NEW
                        || item->_instruction == CSYNC_INSTRUCTION_TYPE_CHANGE)) {
-                // create a new directory within a deleted directory? That can happen if the directory
+                // Create a new directory within a deleted directory? That can happen if the directory
                 // etag was not fetched properly on the previous sync because the sync was aborted
                 // while uploading this directory (which is now removed).  We can ignore it.
-                if (delDirJob) {
+                if (auto delDirJob = qobject_cast<PropagateDirectory *>(currentRemoveDirectoryJob)) {
+                    // increase the number of subjobs that would be there.
                     delDirJob->increaseAffectedCount();
                 }
                 continue;
             } else if (item->_instruction == CSYNC_INSTRUCTION_IGNORE) {
                 continue;
-            } else if (item->_instruction == CSYNC_INSTRUCTION_RENAME) {
+            } else if (item->_instruction != CSYNC_INSTRUCTION_RENAME) {
                 // all is good, the rename will be executed before the directory deletion
-            } else {
+                Q_ASSERT(false); // we shouldn't land here, but assert for debug purposes
                 qCWarning(lcPropagator) << "WARNING:  Job within a removed directory?  This should not happen!"
                                         << item->_file << item->_instruction;
             }
         }
 
-        // If a CONFLICT item contains files these can't be processed because
+        // Second step: if a CONFLICT item contains files, these files can't be processed because
         // the conflict handling is likely to rename the directory. This can happen
         // when there's a new local directory at the same time as a remote file.
         if (!maybeConflictDirectory.isEmpty()) {
-            if (item->destination().startsWith(maybeConflictDirectory)) {
+            if (FileSystem::isChildPathOf(item->destination(), maybeConflictDirectory)) {
+                // We're processing an item in a CONFLICT directory.
                 qCInfo(lcPropagator) << "Skipping job inside CONFLICT directory"
                                      << item->_file << item->_instruction;
                 item->_instruction = CSYNC_INSTRUCTION_NONE;
                 continue;
             } else {
+                // This is not an item inside the conflict directory, which means we're done
+                // processing the conflict directory in the previous iteration, so clear the string.
                 maybeConflictDirectory.clear();
             }
         }
 
-        while (!item->destination().startsWith(directories.top().first)) {
+        // Thirth step: pop the directory stack when we're finished there.
+        // For example, we were processing directory `A/B`, which means that the
+        // `PropagateDirectory` job for `B` is on top of the stack. If we now have
+        // an item `A/c`, then we're back to processing directory `A`, so we have
+        // to pop the `B` job off of the stack.
+        while (!FileSystem::isChildPathOf(item->destination(), directories.top().first)) {
             directories.pop();
         }
 
+        // The last step is to add a subtask for the item. There are 4 cases covered here:
+        // a type change vs. a removal, and a file vs. a directory.
         if (item->isDirectory()) {
             PropagateDirectory *dir = new PropagateDirectory(this, item);
 
@@ -459,7 +505,7 @@ void OwncloudPropagator::start(SyncFileItemSet &&items)
                 // of the file we're about to delete to decide whether uploading
                 // to the new dir is ok...
                 for (const auto &item2 : qAsConst(items)) {
-                    if (item2->destination().startsWith(item->destination() + QLatin1Char('/'))) {
+                    if (item2 != item && FileSystem::isChildPathOf(item2->destination(), item->destination())) {
                         item2->_instruction = CSYNC_INSTRUCTION_NONE;
                         _anotherSyncNeeded = true;
                     }
@@ -469,27 +515,30 @@ void OwncloudPropagator::start(SyncFileItemSet &&items)
             if (item->_instruction == CSYNC_INSTRUCTION_REMOVE) {
                 // We do the removal of directories at the end, because there might be moves from
                 // these directories that will happen later.
-                directoriesToRemove.prepend(dir);
-                removedDirectory = item->_file + QLatin1Char('/');
+                currentRemoveDirectoryJob = dir;
+                _rootJob->addDeleteJob(currentRemoveDirectoryJob);
 
                 // We should not update the etag of parent directories of the removed directory
                 // since it would be done before the actual remove (issue #1845)
                 // NOTE: Currently this means that we don't update those etag at all in this sync,
                 //       but it should not be a problem, they will be updated in the next sync.
-                for (int i = 0; i < directories.size(); ++i) {
-                    if (directories[i].second->item()->_instruction == CSYNC_INSTRUCTION_UPDATE_METADATA)
-                        directories[i].second->item()->_instruction = CSYNC_INSTRUCTION_NONE;
+                for (auto &dir : directories) {
+                    if (dir.second->item()->_instruction == CSYNC_INSTRUCTION_UPDATE_METADATA) {
+                        dir.second->item()->_instruction = CSYNC_INSTRUCTION_NONE;
+                        _anotherSyncNeeded = true;
+                    }
                 }
             } else {
-                PropagateDirectory *currentDirJob = directories.top().second;
-                currentDirJob->appendJob(dir);
+                directories.top().second->appendJob(dir);
             }
-            directories.push(qMakePair(item->destination() + QLatin1Char('/'), dir));
+
+            directories.push(qMakePair(item->destination(), dir));
         } else {
+            // The item is not a directory, but a file.
             if (item->_instruction == CSYNC_INSTRUCTION_TYPE_CHANGE) {
                 // will delete directories, so defer execution
-                directoriesToRemove.prepend(createJob(item));
-                removedDirectory = item->_file + QLatin1Char('/');
+                currentRemoveDirectoryJob = createJob(item);
+                _rootJob->addDeleteJob(currentRemoveDirectoryJob);
             } else {
                 directories.top().second->appendTask(item);
             }
@@ -497,15 +546,11 @@ void OwncloudPropagator::start(SyncFileItemSet &&items)
             if (item->_instruction == CSYNC_INSTRUCTION_CONFLICT) {
                 // This might be a file or a directory on the local side. If it's a
                 // directory we want to skip processing items inside it.
-                maybeConflictDirectory = item->_file + QLatin1Char('/');
+                maybeConflictDirectory = item->_file;
             }
         }
     }
 
-    for (auto *it : qAsConst(directoriesToRemove)) {
-        _rootJob->_dirDeletionJobs.appendJob(it);
-    }
-
     connect(_rootJob.data(), &PropagatorJob::finished, this, &OwncloudPropagator::emitFinished);
 
     _jobScheduled = false;
@@ -782,7 +827,6 @@ Result<Vfs::ConvertToPlaceholderResult, QString> OwncloudPropagator::updatePlace
             // don't use destinatio() with suffix placeholder
             const auto pin = syncOptions()._vfs->pinState(item._file);
             if (pin && pin.get() == PinState::AlwaysLocal) {
-                qDebug() << fileName << item.destination() << item._file;
                 return false;
             }
         }
@@ -795,10 +839,14 @@ Result<Vfs::ConvertToPlaceholderResult, QString> OwncloudPropagator::updateMetad
 {
     const QString fsPath = fullLocalPath(item.destination());
     const auto result = updatePlaceholder(item, fsPath, {});
-    if (!result || result.get() != Vfs::ConvertToPlaceholderResult::Ok) {
+    if (!result) {
         return result;
     }
-    const auto record = item.toSyncJournalFileRecordWithInode(fsPath);
+    auto record = item.toSyncJournalFileRecordWithInode(fsPath);
+    if (result.get() == Vfs::ConvertToPlaceholderResult::Locked) {
+        record._hasDirtyPlaceholder = true;
+        Q_EMIT seenLockedFile(fullLocalPath(item._file), FileSystem::LockMode::Exclusive);
+    }
     const auto dBresult = _journal->setFileRecord(record);
     if (!dBresult) {
         return dBresult.error();
@@ -808,9 +856,10 @@ Result<Vfs::ConvertToPlaceholderResult, QString> OwncloudPropagator::updateMetad
 
 // ================================================================================
 
-PropagatorJob::PropagatorJob(OwncloudPropagator *propagator)
+PropagatorJob::PropagatorJob(OwncloudPropagator *propagator, const QString &path)
     : QObject(propagator)
     , _state(NotYetStarted)
+    , _path(path)
 {
 }
 
@@ -914,18 +963,24 @@ void PropagatorCompositeJob::slotSubJobFinished(SyncFileItem::Status status)
 
     // Delete the job and remove it from our list of jobs.
     subJob->deleteLater();
-    int i = _runningJobs.indexOf(subJob);
-    OC_ENFORCE(i >= 0); // should only happen if this function is called more than once
-    _runningJobs.remove(i);
+    OC_ENFORCE(_runningJobs.removeAll(subJob) == 1); // we removed exactly one item
 
     // Any sub job error will cause the whole composite to fail. This is important
     // for knowing whether to update the etag in PropagateDirectory, for example.
-    if (status == SyncFileItem::FatalError
-        || status == SyncFileItem::NormalError
-        || status == SyncFileItem::SoftError
-        || status == SyncFileItem::DetailError
-        || status == SyncFileItem::BlacklistedError) {
-        _hasError = status;
+    switch (status) {
+    case SyncFileItem::FatalError:
+        [[fallthrough]];
+    case SyncFileItem::NormalError:
+        [[fallthrough]];
+    case SyncFileItem::SoftError:
+        [[fallthrough]];
+    case SyncFileItem::DetailError:
+        [[fallthrough]];
+    case SyncFileItem::BlacklistedError:
+        _errorPaths.insert(subJob->path(), status);
+        break;
+    default:
+        break;
     }
 
     if (_jobsToDo.isEmpty() && _tasksToDo.empty() && _runningJobs.isEmpty()) {
@@ -943,7 +998,7 @@ void PropagatorCompositeJob::finalize()
         return;
 
     _state = Finished;
-    emit finished(_hasError == SyncFileItem::NoStatus ? SyncFileItem::Success : _hasError);
+    emit finished(_errorPaths.empty() ? SyncFileItem::Success : _errorPaths.last());
 }
 
 qint64 PropagatorCompositeJob::committedDiskSpace() const
@@ -955,12 +1010,17 @@ qint64 PropagatorCompositeJob::committedDiskSpace() const
     return needed;
 }
 
+PropagatorCompositeJob::PropagatorCompositeJob(OwncloudPropagator *propagator, const QString &path)
+    : PropagatorJob(propagator, path)
+{
+}
+
 // ================================================================================
 
 PropagateDirectory::PropagateDirectory(OwncloudPropagator *propagator, const SyncFileItemPtr &item)
     : PropagateItemJob(propagator, item)
     , _firstJob(propagator->createJob(item))
-    , _subJobs(propagator)
+    , _subJobs(propagator, path())
 {
     if (_firstJob) {
         connect(_firstJob.data(), &PropagatorJob::finished, this, &PropagateDirectory::slotFirstJobFinished);
@@ -1008,9 +1068,16 @@ void PropagateDirectory::slotFirstJobFinished(SyncFileItem::Status status)
 {
     _firstJob.take()->deleteLater();
 
-    if (status != SyncFileItem::Success
-        && status != SyncFileItem::Restoration
-        && status != SyncFileItem::Conflict) {
+    switch (status) {
+    // non critical states
+    case SyncFileItem::Success:
+        [[fallthrough]];
+    case SyncFileItem::Restoration:
+        [[fallthrough]];
+    case SyncFileItem::Conflict:
+        break;
+    // handle all other cases as errors
+    default:
         if (_state != Finished) {
             // Synchronously abort
             abort(AbortType::Synchronous);
@@ -1083,8 +1150,12 @@ void PropagateDirectory::slotSubJobsFinished(const SyncFileItem::Status status)
 }
 
 PropagateRootDirectory::PropagateRootDirectory(OwncloudPropagator *propagator)
-    : PropagateDirectory(propagator, SyncFileItemPtr(new SyncFileItem))
-    , _dirDeletionJobs(propagator)
+    : PropagateDirectory(propagator, SyncFileItemPtr([] {
+        auto f = new SyncFileItem;
+        f->_file = QLatin1Char('/');
+        return f;
+    }()))
+    , _dirDeletionJobs(propagator, path())
 {
     connect(&_dirDeletionJobs, &PropagatorJob::finished, this, &PropagateRootDirectory::slotDirDeletionJobsFinished);
 }
@@ -1097,10 +1168,11 @@ PropagatorJob::JobParallelism PropagateRootDirectory::parallelism()
 
 void PropagateRootDirectory::abort(PropagatorJob::AbortType abortType)
 {
-    if (_firstJob)
+    if (_firstJob) {
         // Force first job to abort synchronously
         // even if caller allows async abort (asyncAbort)
         _firstJob->abort(AbortType::Synchronous);
+    }
 
     if (abortType == AbortType::Asynchronous) {
         struct AbortsFinished {
@@ -1131,40 +1203,67 @@ qint64 PropagateRootDirectory::committedDiskSpace() const
 
 bool PropagateRootDirectory::scheduleSelfOrChild()
 {
-    if (_state == Finished)
+    if (_state == Finished) {
         return false;
+    }
 
-    if (PropagateDirectory::scheduleSelfOrChild())
+    if (PropagateDirectory::scheduleSelfOrChild()) {
         return true;
+    }
 
     // Important: Finish _subJobs before scheduling any deletes.
-    if (_subJobs._state != Finished)
+    if (_subJobs._state != Finished) {
         return false;
+    }
 
     return _dirDeletionJobs.scheduleSelfOrChild();
 }
 
 void PropagateRootDirectory::slotSubJobsFinished(SyncFileItem::Status status)
 {
-    if (status != SyncFileItem::Success
-        && status != SyncFileItem::Restoration
-        && status != SyncFileItem::Conflict) {
+    switch (status) {
+    // non critical states
+    case SyncFileItem::Success:
+        [[fallthrough]];
+    case SyncFileItem::Restoration:
+        [[fallthrough]];
+    case SyncFileItem::Conflict:
+        break;
+    // handle all other cases as errors
+    default:
+        _status = status;
         if (_state != Finished) {
-            // Synchronously abort
-            abort(AbortType::Synchronous);
-            _state = Finished;
-            emit finished(status);
+            auto subJob = qobject_cast<PropagatorCompositeJob *>(sender());
+            if (OC_ENSURE(subJob)) {
+                const QMap<QString, SyncFileItem::Status> errorPaths = subJob->errorPaths();
+                auto deleteJobs = _dirDeletionJobs.jobsToDo();
+                deleteJobs.erase(std::remove_if(deleteJobs.begin(), deleteJobs.end(),
+                                     [&](auto *p) {
+                                         for (const auto &[path, error] : Utility::asKeyValueRange(errorPaths)) {
+                                             const QFileInfo info(path);
+                                             if (FileSystem::isChildPathOf(p->path(), info.isDir() ? info.filePath() : info.path())) {
+                                                 return true;
+                                             }
+                                         }
+                                         return false;
+                                     }),
+                    deleteJobs.end());
+                _dirDeletionJobs.setJobsToDo(deleteJobs);
+            }
         }
-        return;
     }
-
     propagator()->scheduleNextJob();
 }
 
 void PropagateRootDirectory::slotDirDeletionJobsFinished(SyncFileItem::Status status)
 {
     _state = Finished;
-    emit finished(status);
+    emit finished(_status != SyncFileItem::NoStatus ? _status : status);
+}
+
+void PropagateRootDirectory::addDeleteJob(PropagatorJob *job)
+{
+    _dirDeletionJobs.appendJob(job);
 }
 
 // ================================================================================
