@@ -12,20 +12,23 @@
  * for more details.
  */
 
-#include "config.h"
-#include "owncloudpropagator_p.h"
 #include "propagatedownload.h"
-#include "networkjobs.h"
 #include "account.h"
+#include "config.h"
+#include "filesystem.h"
+#include "networkjobs.h"
+#include "owncloudpropagator_p.h"
+#include "propagatorjobs.h"
+
+#include "common/asserts.h"
+#include "common/checksums.h"
 #include "common/syncjournaldb.h"
 #include "common/syncjournalfilerecord.h"
 #include "common/utility.h"
-#include "filesystem.h"
-#include "propagatorjobs.h"
-#include "common/checksums.h"
-#include "common/asserts.h"
 #include "common/vfs.h"
 
+#include "libsync/theme.h"
+
 #include <QDir>
 #include <QFileInfo>
 #include <QLoggingCategory>
@@ -45,6 +48,85 @@ namespace OCC {
 Q_LOGGING_CATEGORY(lcGetJob, "sync.networkjob.get", QtInfoMsg)
 Q_LOGGING_CATEGORY(lcPropagateDownload, "sync.propagator.download", QtInfoMsg)
 
+
+namespace CernRecallFeature {
+    void handleRecallFile(const QString &filePath, const QString &folderPath, SyncJournalDb &journal)
+    {
+        OC_ENFORCE(Theme::instance()->enableCernBranding());
+
+        auto makeRecallFileName = [](const QString &fn) {
+            QString recallFileName(fn);
+            // Add _recall-XXXX  before the extension.
+            int dotLocation = recallFileName.lastIndexOf(QLatin1Char('.'));
+            // If no extension, add it at the end  (take care of cases like foo/.hidden or foo.bar/file)
+            if (dotLocation <= recallFileName.lastIndexOf(QLatin1Char('/')) + 1) {
+                dotLocation = recallFileName.size();
+            }
+
+            QString timeString = QDateTime::currentDateTimeUtc().toString(QStringLiteral("yyyyMMdd-hhmmss"));
+            recallFileName.insert(dotLocation, QStringLiteral("_.sys.admin#recall#-") + timeString);
+
+            return recallFileName;
+        };
+
+        qCDebug(lcPropagateDownload) << "handleRecallFile: " << filePath;
+
+        FileSystem::setFileHidden(filePath, true);
+
+        QFile file(filePath);
+        if (!file.open(QIODevice::ReadOnly)) {
+            qCWarning(lcPropagateDownload) << "Could not open recall file" << file.errorString();
+            return;
+        }
+        QFileInfo existingFile(filePath);
+        QDir baseDir = existingFile.dir();
+
+        while (!file.atEnd()) {
+            QByteArray line = file.readLine();
+            line.chop(1); // remove trailing \n
+
+            QString recalledFile = QDir::cleanPath(baseDir.filePath(QString::fromUtf8(line)));
+            if (!recalledFile.startsWith(folderPath) || !recalledFile.startsWith(baseDir.path())) {
+                qCWarning(lcPropagateDownload) << "Ignoring recall of " << recalledFile;
+                continue;
+            }
+
+            // Path of the recalled file in the local folder
+            QString localRecalledFile = recalledFile.mid(folderPath.size());
+
+            SyncJournalFileRecord record;
+            if (!journal.getFileRecord(localRecalledFile, &record) || !record.isValid()) {
+                qCWarning(lcPropagateDownload) << "No db entry for recall of" << localRecalledFile;
+                continue;
+            }
+
+            qCInfo(lcPropagateDownload) << "Recalling" << localRecalledFile << "Checksum:" << record._checksumHeader;
+
+            QString targetPath = makeRecallFileName(recalledFile);
+
+            qCDebug(lcPropagateDownload) << "Copy recall file: " << recalledFile << " -> " << targetPath;
+            // Remove the target first, QFile::copy will not overwrite it.
+            FileSystem::remove(targetPath);
+            QFile::copy(recalledFile, targetPath);
+        }
+    }
+
+
+}
+
+namespace {
+    void preserveGroupOwnership(const QString &fileName, const QFileInfo &fi)
+    {
+#ifdef Q_OS_UNIX
+        if (chown(fileName.toLocal8Bit().constData(), -1, fi.groupId()) != 0) {
+            qCWarning(lcPropagateDownload) << "Unable to chown" << fileName << "to previous group owner" << strerror(errno);
+        }
+#else
+        Q_UNUSED(fileName);
+        Q_UNUSED(fi);
+#endif
+    }
+}
 // Always coming in with forward slashes.
 // In csync_excluded_no_ctx we ignore all files with longer than 254 chars
 // This function also adds a dot at the beginning of the filename to hide the file on OS X and Linux
@@ -560,6 +642,8 @@ void PropagateDownloadFile::startFullDownload()
             &_tmpFile, headers, _expectedEtagForResume, _resumeStart, this);
     }
     _job->setBandwidthManager(propagator()->_bandwidthManager);
+    _job->setExpectedContentLength(_item->_size - _resumeStart);
+
     connect(_job.data(), &GETFileJob::finishedSignal, this, &PropagateDownloadFile::slotGetFinished);
     connect(qobject_cast<GETFileJob *>(_job.data()), &GETFileJob::downloadProgress,
         this, &PropagateDownloadFile::slotDownloadProgress);
@@ -782,80 +866,6 @@ void PropagateDownloadFile::deleteExistingFolder()
     }
 }
 
-namespace { // Anonymous namespace for the recall feature
-    static QString makeRecallFileName(const QString &fn)
-    {
-        QString recallFileName(fn);
-        // Add _recall-XXXX  before the extension.
-        int dotLocation = recallFileName.lastIndexOf(QLatin1Char('.'));
-        // If no extension, add it at the end  (take care of cases like foo/.hidden or foo.bar/file)
-        if (dotLocation <= recallFileName.lastIndexOf(QLatin1Char('/')) + 1) {
-            dotLocation = recallFileName.size();
-        }
-
-        QString timeString = QDateTime::currentDateTimeUtc().toString(QStringLiteral("yyyyMMdd-hhmmss"));
-        recallFileName.insert(dotLocation, QStringLiteral("_.sys.admin#recall#-") + timeString);
-
-        return recallFileName;
-    }
-
-    void handleRecallFile(const QString &filePath, const QString &folderPath, SyncJournalDb &journal)
-    {
-        qCDebug(lcPropagateDownload) << "handleRecallFile: " << filePath;
-
-        FileSystem::setFileHidden(filePath, true);
-
-        QFile file(filePath);
-        if (!file.open(QIODevice::ReadOnly)) {
-            qCWarning(lcPropagateDownload) << "Could not open recall file" << file.errorString();
-            return;
-        }
-        QFileInfo existingFile(filePath);
-        QDir baseDir = existingFile.dir();
-
-        while (!file.atEnd()) {
-            QByteArray line = file.readLine();
-            line.chop(1); // remove trailing \n
-
-            QString recalledFile = QDir::cleanPath(baseDir.filePath(QString::fromUtf8(line)));
-            if (!recalledFile.startsWith(folderPath) || !recalledFile.startsWith(baseDir.path())) {
-                qCWarning(lcPropagateDownload) << "Ignoring recall of " << recalledFile;
-                continue;
-            }
-
-            // Path of the recalled file in the local folder
-            QString localRecalledFile = recalledFile.mid(folderPath.size());
-
-            SyncJournalFileRecord record;
-            if (!journal.getFileRecord(localRecalledFile, &record) || !record.isValid()) {
-                qCWarning(lcPropagateDownload) << "No db entry for recall of" << localRecalledFile;
-                continue;
-            }
-
-            qCInfo(lcPropagateDownload) << "Recalling" << localRecalledFile << "Checksum:" << record._checksumHeader;
-
-            QString targetPath = makeRecallFileName(recalledFile);
-
-            qCDebug(lcPropagateDownload) << "Copy recall file: " << recalledFile << " -> " << targetPath;
-            // Remove the target first, QFile::copy will not overwrite it.
-            FileSystem::remove(targetPath);
-            QFile::copy(recalledFile, targetPath);
-        }
-    }
-
-    static void preserveGroupOwnership(const QString &fileName, const QFileInfo &fi)
-    {
-#ifdef Q_OS_UNIX
-        if (chown(fileName.toLocal8Bit().constData(), -1, fi.groupId()) != 0) {
-            qCWarning(lcPropagateDownload) << "Unable to chown" << fileName << "to previous group owner" << strerror(errno);
-        }
-#else
-        Q_UNUSED(fileName);
-        Q_UNUSED(fi);
-#endif
-    }
-} // end namespace
-
 void PropagateDownloadFile::transmissionChecksumValidated(CheckSums::Algorithm checksumType, const QByteArray &checksum)
 {
     const CheckSums::Algorithm theContentChecksumType = propagator()->account()->capabilities().preferredUploadChecksumType();
@@ -1023,11 +1033,12 @@ void PropagateDownloadFile::updateMetadata(bool isConflict)
     done(isConflict ? SyncFileItem::Conflict : SyncFileItem::Success);
 
     // handle the special recall file
-    if (!_item->_remotePerm.hasPermission(RemotePermissions::IsShared)
-        && (_item->_file == QLatin1String(".sys.admin#recall#")
-               || _item->_file.endsWith(QLatin1String("/.sys.admin#recall#")))) {
-        const QString fn = propagator()->fullLocalPath(_item->destination());
-        handleRecallFile(fn, propagator()->localPath(), *propagator()->_journal);
+    if (Q_UNLIKELY(Theme::instance()->enableCernBranding())) {
+        if (!_item->_remotePerm.hasPermission(RemotePermissions::IsShared)
+            && (_item->_file == QLatin1String(".sys.admin#recall#") || _item->_file.endsWith(QLatin1String("/.sys.admin#recall#")))) {
+            const QString fn = propagator()->fullLocalPath(_item->destination());
+            CernRecallFeature::handleRecallFile(fn, propagator()->localPath(), *propagator()->_journal);
+        }
     }
 
     const auto duration = std::chrono::milliseconds(_stopwatch.elapsed());
