@@ -23,10 +23,11 @@
 
 #include <set>
 
-#include <common/utility.h>
-#include <csync.h>
+#include "common/syncjournaldb.h"
+#include "common/utility.h"
+#include "csync.h"
 
-#include <owncloudlib.h>
+#include "owncloudlib.h"
 
 namespace OCC {
 
@@ -42,14 +43,14 @@ class OWNCLOUDSYNC_EXPORT SyncFileItem
 {
     Q_GADGET
 public:
-    enum Direction {
+    enum Direction : uint8_t {
         None = 0,
         Up,
         Down
     };
     Q_ENUM(Direction)
 
-    enum Status { // stored in 4 bits
+    enum Status : uint8_t { // stored in 4 bits
         NoStatus,
 
         FatalError, ///< Error that causes the sync to stop
@@ -91,12 +92,19 @@ public:
         /**
          * The file is excluded by the ignore list
          */
-        Excluded
+        Excluded,
+
+        /**
+         *  Similar to SoftError but will not cause any error handling
+         */
+        Message,
+
+        /** For use in an array or vector for the number of items in this enum.
+         */
+        StatusCount
     };
     Q_ENUM(Status)
 
-    static QString statusEnumDisplayName(Status s);
-
     SyncJournalFileRecord toSyncJournalFileRecordWithInode(const QString &localFileName) const;
 
     /** Creates a basic SyncFileItem from a DB record
@@ -123,6 +131,7 @@ public:
         , _inode(0)
         , _previousSize(0)
         , _previousModtime(0)
+        , _relevantDirectoyInstruction(false)
     {
     }
 
@@ -232,19 +241,19 @@ public:
      */
     QString _originalFile;
 
-    ItemType _type BITFIELD(3);
-    Direction _direction BITFIELD(3);
-    bool _serverHasIgnoredFiles BITFIELD(1);
+    ItemType _type;
+    Direction _direction;
+    bool _serverHasIgnoredFiles;
 
     /// Whether there's an entry in the blacklist table.
     /// Note: that entry may have retries left, so this can be true
     /// without the status being FileIgnored.
-    bool _hasBlacklistEntry BITFIELD(1);
+    bool _hasBlacklistEntry;
 
     // Variables useful to report to the user
-    Status _status BITFIELD(4);
-    bool _isRestoration BITFIELD(1); // The original operation was forbidden, and this is a restoration
-    bool _isSelectiveSync BITFIELD(1); // The file is removed or ignored because it is in the selective sync list
+    Status _status;
+    bool _isRestoration; // The original operation was forbidden, and this is a restoration
+    bool _isSelectiveSync; // The file is removed or ignored because it is in the selective sync list
     quint16 _httpErrorCode;
     RemotePermissions _remotePerm;
     QString _errorString; // Contains a string only in case of error
@@ -257,7 +266,7 @@ public:
     // Variables used by the propagator
     SyncInstructions _instruction;
     time_t _modtime;
-    QByteArray _etag;
+    QString _etag;
     qint64 _size;
     quint64 _inode;
     QByteArray _fileId;
@@ -276,8 +285,26 @@ public:
 
     QString _directDownloadUrl;
     QString _directDownloadCookies;
+
+    bool _relevantDirectoyInstruction = false;
+    bool _finished = false;
+
+    auto toUploadInfo() const
+    {
+        SyncJournalDb::UploadInfo out;
+        out._valid = true;
+        out._modtime = _modtime;
+        out._contentChecksum = _checksumHeader;
+        out._size = _size;
+        return out;
+    }
 };
 
+
+template <>
+OWNCLOUDSYNC_EXPORT QString Utility::enumToDisplayName(SyncFileItem::Status s);
+
+
 inline bool operator<(const SyncFileItemPtr &item1, const SyncFileItemPtr &item2)
 {
     return *item1 < *item2;
@@ -286,8 +313,15 @@ inline bool operator<(const SyncFileItemPtr &item1, const SyncFileItemPtr &item2
 using SyncFileItemSet = std::set<SyncFileItemPtr>;
 }
 
+Q_DECLARE_METATYPE(OCC::SyncFileItemSet)
 Q_DECLARE_METATYPE(OCC::SyncFileItem)
 Q_DECLARE_METATYPE(OCC::SyncFileItemPtr)
 
+OWNCLOUDSYNC_EXPORT QDebug operator<<(QDebug debug, const OCC::SyncFileItem *item);
+inline QDebug operator<<(QDebug debug, const OCC::SyncFileItemPtr &item)
+{
+    return debug << item.data();
+}
+
 
 #endif // SYNCFILEITEM_H
