@@ -5,12 +5,23 @@
  *
  */
 
-#include <QtTest>
-#include "testutils/syncenginetestutils.h"
 #include <syncengine.h>
 
+#include "testutils/syncenginetestutils.h"
+#include "testutils/testutils.h"
+
+#include <QtTest>
+
 using namespace OCC;
 
+namespace {
+auto uploadConflictFilesCapabilities(bool b)
+{
+    auto cap = TestUtils::testCapabilities();
+    cap.insert({ { "uploadConflictFiles", b } });
+    return cap;
+}
+
 bool itemSuccessful(const ItemCompletedSpy &spy, const QString &path, const SyncInstructions instr)
 {
     auto item = spy.findItem(path);
@@ -34,25 +45,30 @@ QStringList findConflicts(const FileInfo &dir)
 {
     QStringList conflicts;
     for (const auto &item : dir.children) {
-        if (item.name.contains("(conflicted copy")) {
+        if (item.name.contains(QLatin1String("(conflicted copy"))) {
             conflicts.append(item.path());
         }
     }
     return conflicts;
 }
 
-bool expectAndWipeConflict(FileModifier &local, FileInfo state, const QString path)
+bool expectAndWipeConflict(FakeFolder &fFolder, const QString &path)
 {
     PathComponents pathComponents(path);
-    auto base = state.find(pathComponents.parentDirComponents());
-    if (!base)
+    auto localState = fFolder.currentLocalState();
+    auto base = localState.find(pathComponents.parentDirComponents());
+    if (!base) {
         return false;
+    }
+
     for (const auto &item : qAsConst(base->children)) {
-        if (item.name.startsWith(pathComponents.fileName()) && item.name.contains("(conflicted copy")) {
-            local.remove(item.path());
+        if (item.name.startsWith(pathComponents.fileName()) && item.name.contains(QLatin1String("(conflicted copy"))) {
+            fFolder.localModifier().remove(item.path());
+            fFolder.applyLocalModificationsWithoutSync();
             return true;
         }
     }
+
     return false;
 }
 
@@ -63,38 +79,69 @@ SyncJournalFileRecord dbRecord(FakeFolder &folder, const QString &path)
     return record;
 }
 
+}
+
 class TestSyncConflict : public QObject
 {
     Q_OBJECT
 
 private slots:
+    void initTestCase_data()
+    {
+        QTest::addColumn<Vfs::Mode>("vfsMode");
+        QTest::addColumn<bool>("filesAreDehydrated");
+
+        QTest::newRow("Vfs::Off") << Vfs::Off << false;
+
+        if (VfsPluginManager::instance().isVfsPluginAvailable(Vfs::WindowsCfApi)) {
+            QTest::newRow("Vfs::WindowsCfApi dehydrated") << Vfs::WindowsCfApi << true;
+
+            // TODO: the hydrated version will fail due to an issue in the winvfs plugin, so leave it disabled for now.
+            // QTest::newRow("Vfs::WindowsCfApi hydrated") << Vfs::WindowsCfApi << false;
+        } else if (Utility::isWindows()) {
+            QWARN("Skipping Vfs::WindowsCfApi");
+        }
+    }
+
     void testNoUpload()
     {
-        FakeFolder fakeFolder{ FileInfo::A12_B12_C12_S12() };
+        QFETCH_GLOBAL(Vfs::Mode, vfsMode);
+        QFETCH_GLOBAL(bool, filesAreDehydrated);
+
+        FakeFolder fakeFolder(FileInfo::A12_B12_C12_S12(), vfsMode, filesAreDehydrated);
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
 
-        fakeFolder.localModifier().setContents("A/a1", 'L');
-        fakeFolder.remoteModifier().setContents("A/a1", 'R');
-        fakeFolder.localModifier().appendByte("A/a2");
-        fakeFolder.remoteModifier().appendByte("A/a2");
-        fakeFolder.remoteModifier().appendByte("A/a2");
-        QVERIFY(fakeFolder.syncOnce());
+        fakeFolder.localModifier().setContents(QStringLiteral("A/a1"), FileModifier::DefaultFileSize, 'L');
+        fakeFolder.remoteModifier().setContents(QStringLiteral("A/a1"), FileModifier::DefaultFileSize, 'R');
+        fakeFolder.localModifier().appendByte(QStringLiteral("A/a2"));
+        fakeFolder.remoteModifier().appendByte(QStringLiteral("A/a2"));
+        fakeFolder.remoteModifier().appendByte(QStringLiteral("A/a2"));
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
 
-        // Verify that the conflict names don't have the user name
-        const auto &conflicts = findConflicts(fakeFolder.currentLocalState().children["A"]);
-        for (const auto &name : conflicts) {
-            QVERIFY(!name.contains(fakeFolder.syncEngine().account()->davDisplayName()));
+        const auto &conflicts = findConflicts(fakeFolder.currentLocalState().children[QStringLiteral("A")]);
+        if (filesAreDehydrated) {
+            // There should be no conflicts: before a modification is done to a local file,
+            // it will be downloaded from the remote first.
+            QCOMPARE(conflicts.size(), 0);
+        } else {
+            // Verify that the conflict names don't have the user name
+            for (const auto &name : conflicts) {
+                QVERIFY(!name.contains(fakeFolder.syncEngine().account()->davDisplayName()));
+            }
+
+            QVERIFY(expectAndWipeConflict(fakeFolder, "A/a1"));
+            QVERIFY(expectAndWipeConflict(fakeFolder, "A/a2"));
+            QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
         }
-
-        QVERIFY(expectAndWipeConflict(fakeFolder.localModifier(), fakeFolder.currentLocalState(), "A/a1"));
-        QVERIFY(expectAndWipeConflict(fakeFolder.localModifier(), fakeFolder.currentLocalState(), "A/a2"));
-        QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
     }
 
     void testUploadAfterDownload()
     {
-        FakeFolder fakeFolder{ FileInfo::A12_B12_C12_S12() };
-        fakeFolder.syncEngine().account()->setCapabilities({ { "uploadConflictFiles", true } });
+        QFETCH_GLOBAL(Vfs::Mode, vfsMode);
+        QFETCH_GLOBAL(bool, filesAreDehydrated);
+
+        FakeFolder fakeFolder(FileInfo::A12_B12_C12_S12(), vfsMode, filesAreDehydrated);
+        fakeFolder.account()->setCapabilities(uploadConflictFilesCapabilities(true));
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
 
         QMap<QByteArray, QString> conflictMap;
@@ -115,37 +162,46 @@ private slots:
             return nullptr;
         });
 
-        fakeFolder.localModifier().setContents("A/a1", 'L');
-        fakeFolder.remoteModifier().setContents("A/a1", 'R');
-        fakeFolder.localModifier().appendByte("A/a2");
-        fakeFolder.remoteModifier().appendByte("A/a2");
-        fakeFolder.remoteModifier().appendByte("A/a2");
-        QVERIFY(fakeFolder.syncOnce());
-        auto local = fakeFolder.currentLocalState();
-        auto remote = fakeFolder.currentRemoteState();
-        QCOMPARE(local, remote);
+        fakeFolder.localModifier().setContents(QStringLiteral("A/a1"), FileModifier::DefaultFileSize, 'L');
+        fakeFolder.remoteModifier().setContents(QStringLiteral("A/a1"), FileModifier::DefaultFileSize, 'R');
+        fakeFolder.localModifier().appendByte(QStringLiteral("A/a2"));
+        fakeFolder.remoteModifier().appendByte(QStringLiteral("A/a2"));
+        fakeFolder.remoteModifier().appendByte(QStringLiteral("A/a2"));
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
+        if (filesAreDehydrated) {
+            // There should be no conflicts: before a modification is done to a local file,
+            // it will be downloaded from the remote first.
+            QCOMPARE(conflictMap.size(), 0);
+        } else {
+            auto local = fakeFolder.currentLocalState();
+            auto remote = fakeFolder.currentRemoteState();
+            QCOMPARE(local, remote);
 
-        auto a1FileId = fakeFolder.remoteModifier().find("A/a1")->fileId;
-        auto a2FileId = fakeFolder.remoteModifier().find("A/a2")->fileId;
-        QVERIFY(conflictMap.contains(a1FileId));
-        QVERIFY(conflictMap.contains(a2FileId));
-        QCOMPARE(conflictMap.size(), 2);
-        QCOMPARE(Utility::conflictFileBaseNameFromPattern(conflictMap[a1FileId].toUtf8()), QByteArray("A/a1"));
+            auto a1FileId = fakeFolder.remoteModifier().find("A/a1")->fileId;
+            auto a2FileId = fakeFolder.remoteModifier().find("A/a2")->fileId;
+            QVERIFY(conflictMap.contains(a1FileId));
+            QVERIFY(conflictMap.contains(a2FileId));
+            QCOMPARE(conflictMap.size(), 2);
+            QCOMPARE(Utility::conflictFileBaseNameFromPattern(conflictMap[a1FileId].toUtf8()), QByteArray("A/a1"));
 
-        // Check that the conflict file contains the username
-        QVERIFY(conflictMap[a1FileId].contains(QString("(conflicted copy %1 ").arg(fakeFolder.syncEngine().account()->davDisplayName())));
+            // Check that the conflict file contains the username
+            QVERIFY(conflictMap[a1FileId].contains(QString("(conflicted copy %1 ").arg(fakeFolder.syncEngine().account()->davDisplayName())));
 
-        QCOMPARE(remote.find(conflictMap[a1FileId])->contentChar, 'L');
-        QCOMPARE(remote.find("A/a1")->contentChar, 'R');
+            QCOMPARE(remote.find(conflictMap[a1FileId])->contentChar, 'L');
+            QCOMPARE(remote.find("A/a1")->contentChar, 'R');
 
-        QCOMPARE(remote.find(conflictMap[a2FileId])->contentSize, 5);
-        QCOMPARE(remote.find("A/a2")->contentSize, 6);
+            QCOMPARE(remote.find(conflictMap[a2FileId])->contentSize, 5);
+            QCOMPARE(remote.find("A/a2")->contentSize, 6);
+        }
     }
 
     void testSeparateUpload()
     {
-        FakeFolder fakeFolder{ FileInfo::A12_B12_C12_S12() };
-        fakeFolder.syncEngine().account()->setCapabilities({ { "uploadConflictFiles", true } });
+        QFETCH_GLOBAL(Vfs::Mode, vfsMode);
+        QFETCH_GLOBAL(bool, filesAreDehydrated);
+
+        FakeFolder fakeFolder(FileInfo::A12_B12_C12_S12(), vfsMode, filesAreDehydrated);
+        fakeFolder.account()->setCapabilities(uploadConflictFilesCapabilities(true));
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
 
         QMap<QByteArray, QString> conflictMap;
@@ -170,14 +226,14 @@ private slots:
         // file didn't finish in the same sync run that the conflict was created.
         // To do that we need to create a mock conflict record.
         auto a1FileId = fakeFolder.remoteModifier().find("A/a1")->fileId;
-        QString conflictName = QLatin1String("A/a1 (conflicted copy me 1234)");
-        fakeFolder.localModifier().insert(conflictName, 64, 'L');
+        QString conflictName = QStringLiteral("A/a1 (conflicted copy me 1234)");
+        fakeFolder.localModifier().insert(conflictName, FileModifier::DefaultFileSize, 'L');
         ConflictRecord conflictRecord;
         conflictRecord.path = conflictName.toUtf8();
         conflictRecord.baseFileId = a1FileId;
         conflictRecord.initialBasePath = "A/a1";
         fakeFolder.syncJournal().setConflictRecord(conflictRecord);
-        QVERIFY(fakeFolder.syncOnce());
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
         QCOMPARE(conflictMap.size(), 1);
         QCOMPARE(conflictMap[a1FileId], conflictName);
@@ -186,16 +242,16 @@ private slots:
 
         // Now the user can locally alter the conflict file and it will be uploaded
         // as usual.
-        fakeFolder.localModifier().setContents(conflictName, 'P');
-        QVERIFY(fakeFolder.syncOnce());
+        fakeFolder.localModifier().setContents(conflictName, FileModifier::DefaultFileSize + 1, 'P'); // make sure the file sizes are different
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
         QCOMPARE(conflictMap.size(), 1);
         QCOMPARE(conflictMap[a1FileId], conflictName);
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
         conflictMap.clear();
 
         // Similarly, remote modifications of conflict files get propagated downwards
-        fakeFolder.remoteModifier().setContents(conflictName, 'Q');
-        QVERIFY(fakeFolder.syncOnce());
+        fakeFolder.remoteModifier().setContents(conflictName, FileModifier::DefaultFileSize + 1, 'Q'); // make sure the file sizes are different
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
         QVERIFY(conflictMap.isEmpty());
 
@@ -204,25 +260,28 @@ private slots:
         fakeFolder.remoteModifier().appendByte(conflictName);
         fakeFolder.remoteModifier().appendByte(conflictName);
         fakeFolder.localModifier().appendByte(conflictName);
-        QVERIFY(fakeFolder.syncOnce());
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
         QCOMPARE(conflictMap.size(), 1);
         QVERIFY(conflictMap.contains(a1ConflictFileId));
-        QCOMPARE(fakeFolder.currentRemoteState().find(conflictName)->contentSize, 66);
-        QCOMPARE(fakeFolder.currentRemoteState().find(conflictMap[a1ConflictFileId])->contentSize, 65);
+        QCOMPARE(fakeFolder.currentRemoteState().find(conflictName)->contentSize, FileModifier::DefaultFileSize + 3);
+        QCOMPARE(fakeFolder.currentRemoteState().find(conflictMap[a1ConflictFileId])->contentSize, FileModifier::DefaultFileSize + 2);
         conflictMap.clear();
     }
 
     // What happens if we download a conflict file? Is the metadata set up correctly?
     void testDownloadingConflictFile()
     {
-        FakeFolder fakeFolder{ FileInfo::A12_B12_C12_S12() };
-        fakeFolder.syncEngine().account()->setCapabilities({ { "uploadConflictFiles", true } });
+        QFETCH_GLOBAL(Vfs::Mode, vfsMode);
+        QFETCH_GLOBAL(bool, filesAreDehydrated);
+
+        FakeFolder fakeFolder(FileInfo::A12_B12_C12_S12(), vfsMode, filesAreDehydrated);
+        fakeFolder.account()->setCapabilities(uploadConflictFilesCapabilities(true));
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
 
         // With no headers from the server
-        fakeFolder.remoteModifier().insert("A/a1 (conflicted copy 1234)");
-        QVERIFY(fakeFolder.syncOnce());
+        fakeFolder.remoteModifier().insert(QStringLiteral("A/a1 (conflicted copy 1234)"));
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
         auto conflictRecord = fakeFolder.syncJournal().conflictRecord("A/a1 (conflicted copy 1234)");
         QVERIFY(conflictRecord.isValid());
@@ -244,22 +303,32 @@ private slots:
             }
             return nullptr;
         });
-        fakeFolder.remoteModifier().insert("A/really-a-conflict"); // doesn't look like a conflict, but headers say it is
-        QVERIFY(fakeFolder.syncOnce());
+        fakeFolder.remoteModifier().insert(QStringLiteral("A/really-a-conflict")); // doesn't look like a conflict, but headers say it is
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
+
         conflictRecord = fakeFolder.syncJournal().conflictRecord("A/really-a-conflict");
-        QVERIFY(conflictRecord.isValid());
-        QCOMPARE(conflictRecord.baseFileId, a2FileId);
-        QCOMPARE(conflictRecord.baseModtime, 1234);
-        QCOMPARE(conflictRecord.baseEtag, QByteArray("etag"));
-        QCOMPARE(conflictRecord.initialBasePath, QByteArray("A/original"));
+
+        if (filesAreDehydrated) {
+            // A placeholder for the conflicting file is created, but no actual GET request is made, so there should be no conflict record
+            QVERIFY(!conflictRecord.isValid());
+        } else {
+            QVERIFY(conflictRecord.isValid());
+            QCOMPARE(conflictRecord.baseFileId, a2FileId);
+            QCOMPARE(conflictRecord.baseModtime, 1234);
+            QCOMPARE(conflictRecord.baseEtag, QByteArray("etag"));
+            QCOMPARE(conflictRecord.initialBasePath, QByteArray("A/original"));
+        }
     }
 
     // Check that conflict records are removed when the file is gone
     void testConflictRecordRemoval1()
     {
-        FakeFolder fakeFolder{ FileInfo::A12_B12_C12_S12() };
-        fakeFolder.syncEngine().account()->setCapabilities({ { "uploadConflictFiles", true } });
+        QFETCH_GLOBAL(Vfs::Mode, vfsMode);
+        QFETCH_GLOBAL(bool, filesAreDehydrated);
+
+        FakeFolder fakeFolder(FileInfo::A12_B12_C12_S12(), vfsMode, filesAreDehydrated);
+        fakeFolder.account()->setCapabilities(uploadConflictFilesCapabilities(true));
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
 
         // Make conflict records
@@ -270,14 +339,14 @@ private slots:
         fakeFolder.syncJournal().setConflictRecord(conflictRecord);
 
         // A nothing-to-sync keeps them alive
-        QVERIFY(fakeFolder.syncOnce());
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
         QVERIFY(fakeFolder.syncJournal().conflictRecord("A/a1").isValid());
         QVERIFY(fakeFolder.syncJournal().conflictRecord("A/a2").isValid());
 
         // When the file is removed, the record is removed too
-        fakeFolder.localModifier().remove("A/a2");
-        QVERIFY(fakeFolder.syncOnce());
+        fakeFolder.localModifier().remove(QStringLiteral("A/a2"));
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
         QVERIFY(fakeFolder.syncJournal().conflictRecord("A/a1").isValid());
         QVERIFY(!fakeFolder.syncJournal().conflictRecord("A/a2").isValid());
@@ -286,39 +355,48 @@ private slots:
     // Same test, but with uploadConflictFiles == false
     void testConflictRecordRemoval2()
     {
-        FakeFolder fakeFolder{ FileInfo::A12_B12_C12_S12() };
-        fakeFolder.syncEngine().account()->setCapabilities({ { "uploadConflictFiles", false } });
+        QFETCH_GLOBAL(Vfs::Mode, vfsMode);
+        QFETCH_GLOBAL(bool, filesAreDehydrated);
+
+        FakeFolder fakeFolder(FileInfo::A12_B12_C12_S12(), vfsMode, filesAreDehydrated);
+        fakeFolder.account()->setCapabilities(uploadConflictFilesCapabilities(false));
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
 
         // Create two conflicts
-        fakeFolder.localModifier().appendByte("A/a1");
-        fakeFolder.localModifier().appendByte("A/a1");
-        fakeFolder.remoteModifier().appendByte("A/a1");
-        fakeFolder.localModifier().appendByte("A/a2");
-        fakeFolder.localModifier().appendByte("A/a2");
-        fakeFolder.remoteModifier().appendByte("A/a2");
-        QVERIFY(fakeFolder.syncOnce());
+        fakeFolder.localModifier().appendByte(QStringLiteral("A/a1"));
+        fakeFolder.localModifier().appendByte(QStringLiteral("A/a1"));
+        fakeFolder.remoteModifier().appendByte(QStringLiteral("A/a1"));
+        fakeFolder.localModifier().appendByte(QStringLiteral("A/a2"));
+        fakeFolder.localModifier().appendByte(QStringLiteral("A/a2"));
+        fakeFolder.remoteModifier().appendByte(QStringLiteral("A/a2"));
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
 
         const auto &conflicts = findConflicts(fakeFolder.currentLocalState().children["A"]);
-        QByteArray a1conflict;
-        QByteArray a2conflict;
-        for (const auto & conflict : conflicts) {
-            if (conflict.contains("a1"))
-                a1conflict = conflict.toUtf8();
-            if (conflict.contains("a2"))
-                a2conflict = conflict.toUtf8();
+        if (filesAreDehydrated) {
+            // There should be no conflicts: before a modification is done to a local file,
+            // it will be downloaded from the remote first.
+            QCOMPARE(conflicts.size(), 0);
+        } else {
+            QByteArray a1conflict;
+            QByteArray a2conflict;
+            for (const auto &conflict : conflicts) {
+                if (conflict.contains("a1"))
+                    a1conflict = conflict.toUtf8();
+                if (conflict.contains("a2"))
+                    a2conflict = conflict.toUtf8();
+            }
+
+            // A nothing-to-sync keeps them alive
+            QVERIFY(fakeFolder.applyLocalModificationsAndSync());
+            QVERIFY(fakeFolder.syncJournal().conflictRecord(a1conflict).isValid());
+            QVERIFY(fakeFolder.syncJournal().conflictRecord(a2conflict).isValid());
+
+            // When the file is removed, the record is removed too
+            fakeFolder.localModifier().remove(a2conflict);
+            QVERIFY(fakeFolder.applyLocalModificationsAndSync());
+            QVERIFY(fakeFolder.syncJournal().conflictRecord(a1conflict).isValid());
+            QVERIFY(!fakeFolder.syncJournal().conflictRecord(a2conflict).isValid());
         }
-
-        // A nothing-to-sync keeps them alive
-        QVERIFY(fakeFolder.syncOnce());
-        QVERIFY(fakeFolder.syncJournal().conflictRecord(a1conflict).isValid());
-        QVERIFY(fakeFolder.syncJournal().conflictRecord(a2conflict).isValid());
-
-        // When the file is removed, the record is removed too
-        fakeFolder.localModifier().remove(a2conflict);
-        QVERIFY(fakeFolder.syncOnce());
-        QVERIFY(fakeFolder.syncJournal().conflictRecord(a1conflict).isValid());
-        QVERIFY(!fakeFolder.syncJournal().conflictRecord(a2conflict).isValid());
     }
 
     void testConflictFileBaseName_data()
@@ -403,8 +481,11 @@ private slots:
 
     void testLocalDirRemoteFileConflict()
     {
-        FakeFolder fakeFolder{ FileInfo::A12_B12_C12_S12() };
-        fakeFolder.syncEngine().account()->setCapabilities({ { "uploadConflictFiles", true } });
+        QFETCH_GLOBAL(Vfs::Mode, vfsMode);
+        QFETCH_GLOBAL(bool, filesAreDehydrated);
+
+        FakeFolder fakeFolder(FileInfo::A12_B12_C12_S12(), vfsMode, filesAreDehydrated);
+        fakeFolder.account()->setCapabilities(uploadConflictFilesCapabilities(true));
         ItemCompletedSpy completeSpy(fakeFolder);
 
         auto cleanup = [&]() {
@@ -413,26 +494,26 @@ private slots:
         cleanup();
 
         // 1) a NEW/NEW conflict
-        fakeFolder.localModifier().mkdir("Z");
-        fakeFolder.localModifier().mkdir("Z/subdir");
-        fakeFolder.localModifier().insert("Z/foo");
-        fakeFolder.remoteModifier().insert("Z", 63);
+        fakeFolder.localModifier().mkdir(QStringLiteral("Z"));
+        fakeFolder.localModifier().mkdir(QStringLiteral("Z/subdir"));
+        fakeFolder.localModifier().insert(QStringLiteral("Z/foo"));
+        fakeFolder.remoteModifier().insert(QStringLiteral("Z"), 63_b);
 
         // 2) local file becomes a dir; remote file changes
-        fakeFolder.localModifier().remove("A/a1");
-        fakeFolder.localModifier().mkdir("A/a1");
-        fakeFolder.localModifier().insert("A/a1/bar");
-        fakeFolder.remoteModifier().appendByte("A/a1");
+        fakeFolder.localModifier().remove(QStringLiteral("A/a1"));
+        fakeFolder.localModifier().mkdir(QStringLiteral("A/a1"));
+        fakeFolder.localModifier().insert(QStringLiteral("A/a1/bar"));
+        fakeFolder.remoteModifier().appendByte(QStringLiteral("A/a1"));
 
         // 3) local dir gets a new file; remote dir becomes a file
-        fakeFolder.localModifier().insert("B/zzz");
-        fakeFolder.remoteModifier().remove("B");
-        fakeFolder.remoteModifier().insert("B", 31);
+        fakeFolder.localModifier().insert(QStringLiteral("B/zzz"));
+        fakeFolder.remoteModifier().remove(QStringLiteral("B"));
+        fakeFolder.remoteModifier().insert(QStringLiteral("B"), 31_b);
 
-        QVERIFY(fakeFolder.syncOnce());
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
 
         auto conflicts = findConflicts(fakeFolder.currentLocalState());
-        conflicts += findConflicts(fakeFolder.currentLocalState().children["A"]);
+        conflicts += findConflicts(fakeFolder.currentLocalState().children[QStringLiteral("A")]);
         QCOMPARE(conflicts.size(), 3);
         std::sort(conflicts.begin(), conflicts.end());
 
@@ -468,7 +549,7 @@ private slots:
         // another sync.
         QVERIFY(fakeFolder.syncEngine().isAnotherSyncNeeded() == ImmediateFollowUp);
         cleanup();
-        QVERIFY(fakeFolder.syncOnce());
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
 
         QVERIFY(itemSuccessful(completeSpy, conflicts[0], CSYNC_INSTRUCTION_NEW));
         QVERIFY(itemSuccessful(completeSpy, conflicts[0] + "/bar", CSYNC_INSTRUCTION_NEW));
@@ -481,30 +562,40 @@ private slots:
 
     void testLocalFileRemoteDirConflict()
     {
-        FakeFolder fakeFolder{ FileInfo::A12_B12_C12_S12() };
-        fakeFolder.syncEngine().account()->setCapabilities({ { "uploadConflictFiles", true } });
+        QFETCH_GLOBAL(Vfs::Mode, vfsMode);
+        QFETCH_GLOBAL(bool, filesAreDehydrated);
+
+        FakeFolder fakeFolder(FileInfo::A12_B12_C12_S12(), vfsMode, filesAreDehydrated);
+        fakeFolder.account()->setCapabilities(uploadConflictFilesCapabilities(true));
         ItemCompletedSpy completeSpy(fakeFolder);
 
         // 1) a NEW/NEW conflict
-        fakeFolder.remoteModifier().mkdir("Z");
-        fakeFolder.remoteModifier().mkdir("Z/subdir");
-        fakeFolder.remoteModifier().insert("Z/foo");
-        fakeFolder.localModifier().insert("Z");
+        fakeFolder.remoteModifier().mkdir(QStringLiteral("Z"));
+        fakeFolder.remoteModifier().mkdir(QStringLiteral("Z/subdir"));
+        fakeFolder.remoteModifier().insert(QStringLiteral("Z/foo"));
+        fakeFolder.localModifier().insert(QStringLiteral("Z"));
 
         // 2) local dir becomes file: remote dir adds file
-        fakeFolder.localModifier().remove("A");
-        fakeFolder.localModifier().insert("A", 63);
-        fakeFolder.remoteModifier().insert("A/bar");
+        fakeFolder.localModifier().remove(QStringLiteral("A"));
+        fakeFolder.localModifier().insert(QStringLiteral("A"), 63_b);
+        fakeFolder.remoteModifier().insert(QStringLiteral("A/bar"));
 
         // 3) local file changes; remote file becomes dir
-        fakeFolder.localModifier().appendByte("B/b1");
-        fakeFolder.remoteModifier().remove("B/b1");
-        fakeFolder.remoteModifier().mkdir("B/b1");
-        fakeFolder.remoteModifier().insert("B/b1/zzz");
+        fakeFolder.localModifier().appendByte(QStringLiteral("B/b1"));
+        fakeFolder.remoteModifier().remove(QStringLiteral("B/b1"));
+        fakeFolder.remoteModifier().mkdir(QStringLiteral("B/b1"));
+        fakeFolder.remoteModifier().insert(QStringLiteral("B/b1/zzz"));
 
-        QVERIFY(fakeFolder.syncOnce());
+        if (filesAreDehydrated) {
+            QSKIP("Known bug: https://github.com/owncloud/client/issues/10223");
+            // The QSKIP below is due to operations order (verified on a local machine),
+            // but the sync below already always fails in the CI. So we will be skipping
+            // this entire test until the issue is fixed.
+        }
+
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
         auto conflicts = findConflicts(fakeFolder.currentLocalState());
-        conflicts += findConflicts(fakeFolder.currentLocalState().children["B"]);
+        conflicts += findConflicts(fakeFolder.currentLocalState().children[QStringLiteral("B")]);
         QCOMPARE(conflicts.size(), 3);
         std::sort(conflicts.begin(), conflicts.end());
 
@@ -527,27 +618,33 @@ private slots:
         QVERIFY(conflicts[1].contains("B/b1"));
         QCOMPARE(conflicts[1].toUtf8(), conflictRecords[1]);
 
+        if (filesAreDehydrated) {
+            QSKIP("Known bug: https://github.com/owncloud/client/issues/10223");
+        }
         // Also verifies that conflicts were uploaded
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
     }
 
     void testTypeConflictWithMove()
     {
-        FakeFolder fakeFolder{ FileInfo::A12_B12_C12_S12() };
+        QFETCH_GLOBAL(Vfs::Mode, vfsMode);
+        QFETCH_GLOBAL(bool, filesAreDehydrated);
+
+        FakeFolder fakeFolder(FileInfo::A12_B12_C12_S12(), vfsMode, filesAreDehydrated);
         ItemCompletedSpy completeSpy(fakeFolder);
 
         // the remote becomes a file, but a file inside the dir has moved away!
-        fakeFolder.remoteModifier().remove("A");
-        fakeFolder.remoteModifier().insert("A");
-        fakeFolder.localModifier().rename("A/a1", "a1");
+        fakeFolder.remoteModifier().remove(QStringLiteral("A"));
+        fakeFolder.remoteModifier().insert(QStringLiteral("A"));
+        fakeFolder.localModifier().rename(QStringLiteral("A/a1"), QStringLiteral("a1"));
 
         // same, but with a new file inside the dir locally
-        fakeFolder.remoteModifier().remove("B");
-        fakeFolder.remoteModifier().insert("B");
-        fakeFolder.localModifier().rename("B/b1", "b1");
-        fakeFolder.localModifier().insert("B/new");
+        fakeFolder.remoteModifier().remove(QStringLiteral("B"));
+        fakeFolder.remoteModifier().insert(QStringLiteral("B"));
+        fakeFolder.localModifier().rename(QStringLiteral("B/b1"), QStringLiteral("b1"));
+        fakeFolder.localModifier().insert(QStringLiteral("B/new"));
 
-        QVERIFY(fakeFolder.syncOnce());
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
 
         QVERIFY(itemSuccessful(completeSpy, "A", CSYNC_INSTRUCTION_TYPE_CHANGE));
         QVERIFY(itemConflict(completeSpy, "B"));
@@ -566,24 +663,31 @@ private slots:
 
     void testTypeChange()
     {
-        FakeFolder fakeFolder{ FileInfo::A12_B12_C12_S12() };
+        QFETCH_GLOBAL(Vfs::Mode, vfsMode);
+        QFETCH_GLOBAL(bool, filesAreDehydrated);
+
+        if (filesAreDehydrated) {
+            QSKIP("Known bug: https://github.com/owncloud/client/issues/10223");
+        }
+
+        FakeFolder fakeFolder(FileInfo::A12_B12_C12_S12(), vfsMode, filesAreDehydrated);
         ItemCompletedSpy completeSpy(fakeFolder);
 
         // dir becomes file
-        fakeFolder.remoteModifier().remove("A");
-        fakeFolder.remoteModifier().insert("A");
-        fakeFolder.localModifier().remove("B");
-        fakeFolder.localModifier().insert("B");
+        fakeFolder.remoteModifier().remove(QStringLiteral("A"));
+        fakeFolder.remoteModifier().insert(QStringLiteral("A"));
+        fakeFolder.localModifier().remove(QStringLiteral("B"));
+        fakeFolder.localModifier().insert(QStringLiteral("B"));
 
         // file becomes dir
-        fakeFolder.remoteModifier().remove("C/c1");
-        fakeFolder.remoteModifier().mkdir("C/c1");
-        fakeFolder.remoteModifier().insert("C/c1/foo");
-        fakeFolder.localModifier().remove("C/c2");
-        fakeFolder.localModifier().mkdir("C/c2");
-        fakeFolder.localModifier().insert("C/c2/bar");
+        fakeFolder.remoteModifier().remove(QStringLiteral("C/c1"));
+        fakeFolder.remoteModifier().mkdir(QStringLiteral("C/c1"));
+        fakeFolder.remoteModifier().insert(QStringLiteral("C/c1/foo"));
+        fakeFolder.localModifier().remove(QStringLiteral("C/c2"));
+        fakeFolder.localModifier().mkdir(QStringLiteral("C/c2"));
+        fakeFolder.localModifier().insert(QStringLiteral("C/c2/bar"));
 
-        QVERIFY(fakeFolder.syncOnce());
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
 
         QVERIFY(itemSuccessful(completeSpy, "A", CSYNC_INSTRUCTION_TYPE_CHANGE));
         QVERIFY(itemSuccessful(completeSpy, "B", CSYNC_INSTRUCTION_TYPE_CHANGE));
@@ -595,27 +699,30 @@ private slots:
         const auto &conflicts = findConflicts(fakeFolder.currentLocalState());
         QVERIFY(conflicts.size() == 1);
         QVERIFY(conflicts[0].contains("A (conflicted copy"));
-        for (const auto &conflict : conflicts)
+        for (const auto &conflict : conflicts) {
             QDir(fakeFolder.localPath() + conflict).removeRecursively();
+        }
 
         QVERIFY(fakeFolder.syncEngine().isAnotherSyncNeeded() == ImmediateFollowUp);
-        QVERIFY(fakeFolder.syncOnce());
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
     }
 
     // Test what happens if we remove entries both on the server, and locally
     void testRemoveRemove()
     {
-        FakeFolder fakeFolder{ FileInfo::A12_B12_C12_S12() };
-        fakeFolder.remoteModifier().remove("A");
-        fakeFolder.localModifier().remove("A");
-        fakeFolder.remoteModifier().remove("B/b1");
-        fakeFolder.localModifier().remove("B/b1");
+        QFETCH_GLOBAL(Vfs::Mode, vfsMode);
+        QFETCH_GLOBAL(bool, filesAreDehydrated);
 
-        QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
-        auto expectedState = fakeFolder.currentLocalState();
+        FakeFolder fakeFolder(FileInfo::A12_B12_C12_S12(), vfsMode, filesAreDehydrated);
+        fakeFolder.remoteModifier().remove(QStringLiteral("A"));
+        fakeFolder.localModifier().remove(QStringLiteral("A"));
+        fakeFolder.remoteModifier().remove(QStringLiteral("B/b1"));
+        fakeFolder.localModifier().remove(QStringLiteral("B/b1"));
 
-        QVERIFY(fakeFolder.syncOnce());
+        auto expectedState = fakeFolder.currentRemoteState();
+
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
 
         QCOMPARE(fakeFolder.currentLocalState(), expectedState);
         QCOMPARE(fakeFolder.currentRemoteState(), expectedState);
