@@ -6,18 +6,23 @@
  */
 
 #include "syncenginetestutils.h"
+#include "testutils.h"
 
 #include "httplogger.h"
 #include "accessmanager.h"
 #include "libsync/configfile.h"
 
+#include <thread>
+
+using namespace std::chrono_literals;
+
 PathComponents::PathComponents(const char *path)
     : PathComponents { QString::fromUtf8(path) }
 {
 }
 
 PathComponents::PathComponents(const QString &path)
-    : QStringList { path.split(QLatin1Char('/'), QString::SkipEmptyParts) }
+    : QStringList { path.split(QLatin1Char('/'), Qt::SkipEmptyParts) }
 {
 }
 
@@ -38,76 +43,92 @@ PathComponents PathComponents::subComponents() const &
 
 void DiskFileModifier::remove(const QString &relativePath)
 {
-    QFileInfo fi { _rootDir.filePath(relativePath) };
-    if (fi.isFile())
-        QVERIFY(_rootDir.remove(relativePath));
-    else
-        QVERIFY(QDir { fi.filePath() }.removeRecursively());
+    _processArguments.append({ "remove", relativePath });
 }
 
-void DiskFileModifier::insert(const QString &relativePath, qint64 size, char contentChar)
+void DiskFileModifier::insert(const QString &relativePath, quint64 size, char contentChar)
 {
-    QFile file { _rootDir.filePath(relativePath) };
-    QVERIFY(!file.exists());
-    file.open(QFile::WriteOnly);
-    QByteArray buf(1024, contentChar);
-    for (int x = 0; x < size / buf.size(); ++x) {
-        file.write(buf);
-    }
-    file.write(buf.data(), size % buf.size());
-    file.close();
-    // Set the mtime 30 seconds in the past, for some tests that need to make sure that the mtime differs.
-    OCC::FileSystem::setModTime(file.fileName(), OCC::Utility::qDateTimeToTime_t(QDateTime::currentDateTimeUtc().addSecs(-30)));
-    QCOMPARE(file.size(), size);
+    // In the test helper, the insert command is implemented by filling a QByteArray, and then
+    // writing that to disk. The size of the QByteArray is an int, so make sure that we don't end
+    // up with a negative int when converting from quint64 -> int.
+    Q_ASSERT(size <= std::numeric_limits<int>::max());
+
+    _processArguments.append({ "insert", relativePath, QString::number(size), QChar::fromLatin1(contentChar) });
 }
 
-void DiskFileModifier::setContents(const QString &relativePath, char contentChar)
+void DiskFileModifier::setContents(const QString &relativePath, quint64 newSize, char contentChar)
 {
-    QFile file { _rootDir.filePath(relativePath) };
-    QVERIFY(file.exists());
-    qint64 size = file.size();
-    file.open(QFile::WriteOnly);
-    file.write(QByteArray {}.fill(contentChar, size));
+    _processArguments.append({ "contents", relativePath, QString::number(newSize), QChar::fromLatin1(contentChar) });
 }
 
 void DiskFileModifier::appendByte(const QString &relativePath, char contentChar)
 {
-    QFile file { _rootDir.filePath(relativePath) };
-    QVERIFY(file.exists());
-    file.open(QFile::ReadWrite);
-    QByteArray contents;
-    if (contentChar)
-        contents += contentChar;
-    else
-        contents = file.read(1);
-    file.seek(file.size());
-    file.write(contents);
-}
-
-void DiskFileModifier::modifyByte(const QString &relativePath, quint64 offset, char contentChar)
-{
-    QFile file { _rootDir.filePath(relativePath) };
-    QVERIFY(file.exists());
-    file.open(QFile::ReadWrite);
-    file.seek(offset);
-    file.write(&contentChar, 1);
-    file.close();
+    _processArguments.append({ "appendbyte", relativePath, QChar::fromLatin1(contentChar) });
 }
 
 void DiskFileModifier::mkdir(const QString &relativePath)
 {
-    _rootDir.mkpath(relativePath);
+    _processArguments.append({ "mkdir", relativePath });
 }
 
 void DiskFileModifier::rename(const QString &from, const QString &to)
 {
-    QVERIFY(_rootDir.exists(from));
-    QVERIFY(_rootDir.rename(from, to));
+    _processArguments.append({ "rename", from, to });
 }
 
 void DiskFileModifier::setModTime(const QString &relativePath, const QDateTime &modTime)
 {
-    OCC::FileSystem::setModTime(_rootDir.filePath(relativePath), OCC::Utility::qDateTimeToTime_t(modTime));
+    _processArguments.append({ "mtime", relativePath, QString::number(modTime.toSecsSinceEpoch()) });
+}
+
+class HelperProcess : public QProcess
+{
+public:
+    HelperProcess(const QDir &rootDir, QStringList processArguments) // copy is on purpose
+    {
+        if (processArguments.isEmpty()) { // fast-path:
+            finished = true;
+            succeeded = true;
+            return;
+        }
+
+        setProcessChannelMode(QProcess::MergedChannels);
+        processArguments.prepend(rootDir.absolutePath());
+        QObject::connect(this, &QProcess::readyRead, [this]() {
+            while (canReadLine()) {
+                qDebug() << "helper output:" << readLine();
+            }
+        });
+        QObject::connect(this, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished),
+            [this](int exitCode, QProcess::ExitStatus exitStatus) {
+                qDebug() << "helper finished:" << exitCode << exitStatus;
+                this->finished = true;
+                this->succeeded = exitStatus == QProcess::NormalExit && exitCode == 0;
+            });
+
+        qDebug() << "Starting helper:" << TEST_HELPER_EXE << processArguments;
+        start(TEST_HELPER_EXE, processArguments);
+    }
+
+public:
+    bool finished = false;
+    bool succeeded = true;
+};
+
+bool DiskFileModifier::applyModifications()
+{
+    if (_processArguments.isEmpty()) {
+        return true;
+    }
+
+    HelperProcess helper(_rootDir, _processArguments);
+    helper.waitForStarted();
+    _processArguments.clear(); // the helper has a copy, we clean it now for the next run
+    do {
+        std::this_thread::sleep_for(50ms);
+        QThread::currentThread()->eventDispatcher()->processEvents(QEventLoop::AllEvents);
+    } while (!helper.finished);
+    return helper.succeeded;
 }
 
 FileInfo FileInfo::A12_B12_C12_S12()
@@ -145,25 +166,28 @@ void FileInfo::remove(const QString &relativePath)
     const PathComponents pathComponents { relativePath };
     FileInfo *parent = findInvalidatingEtags(pathComponents.parentDirComponents());
     Q_ASSERT(parent);
-    parent->children.erase(std::find_if(parent->children.begin(), parent->children.end(),
-        [&pathComponents](const FileInfo &fi) { return fi.name == pathComponents.fileName(); }));
+    parent->children.remove(pathComponents.fileName());
 }
 
-void FileInfo::insert(const QString &relativePath, qint64 size, char contentChar)
+void FileInfo::insert(const QString &relativePath, quint64 size, char contentChar)
 {
     create(relativePath, size, contentChar);
 }
 
-void FileInfo::setContents(const QString &relativePath, char contentChar)
+void FileInfo::setContents(const QString &relativePath, quint64 newSize, char contentChar)
 {
     FileInfo *file = findInvalidatingEtags(relativePath);
     Q_ASSERT(file);
     file->contentChar = contentChar;
+    file->contentSize = newSize;
+    if (!file->isDehydratedPlaceholder) {
+        file->fileSize = newSize;
+    }
 }
 
 void FileInfo::appendByte(const QString &relativePath, char contentChar)
 {
-    Q_UNUSED(contentChar);
+    Q_UNUSED(contentChar)
     FileInfo *file = findInvalidatingEtags(relativePath);
     Q_ASSERT(file);
     if (!file->isDehydratedPlaceholder) {
@@ -172,15 +196,6 @@ void FileInfo::appendByte(const QString &relativePath, char contentChar)
     file->fileSize += 1;
 }
 
-void FileInfo::modifyByte(const QString &relativePath, quint64 offset, char contentChar)
-{
-    Q_UNUSED(offset);
-    Q_UNUSED(contentChar);
-    FileInfo *file = findInvalidatingEtags(relativePath);
-    Q_ASSERT(file);
-    Q_ASSERT(!"unimplemented");
-}
-
 void FileInfo::mkdir(const QString &relativePath)
 {
     createDir(relativePath);
@@ -241,7 +256,7 @@ FileInfo *FileInfo::createDir(const QString &relativePath)
     return &child;
 }
 
-FileInfo *FileInfo::create(const QString &relativePath, qint64 size, char contentChar)
+FileInfo *FileInfo::create(const QString &relativePath, quint64 size, char contentChar)
 {
     const PathComponents pathComponents { relativePath };
     FileInfo *parent = findInvalidatingEtags(pathComponents.parentDirComponents());
@@ -258,35 +273,42 @@ bool FileInfo::equals(const FileInfo &other, CompareWhat compareWhat) const
     // Only check the content and contentSize if both files are hydrated:
     if (!isDehydratedPlaceholder && !other.isDehydratedPlaceholder) {
         if (contentSize != other.contentSize || contentChar != other.contentChar) {
+            qDebug() << "1" << name << "!=" << other.name;
             return false;
         }
     }
 
     // We need to check this before we use isDir in the next if-statement:
     if (isDir != other.isDir) {
+        qDebug() << "2" << name << "!=" << other.name;
         return false;
     }
 
     if (compareWhat == CompareLastModified) {
         // Don't check directory mtime: it might change when (unsynced) files get created.
         if (!isDir && _lastModifiedInSecondsUTC != other._lastModifiedInSecondsUTC) {
+            qDebug() << "3" << name << "!=" << other.name;
             return false;
         }
     }
 
     if (name != other.name || fileSize != other.fileSize) {
+        qDebug() << "4" << name << "!=" << other.name;
         return false;
     }
 
     if (children.size() != other.children.size()) {
+        qDebug() << "5" << name << "!=" << other.name;
         return false;
     }
 
     for (auto it = children.constBegin(), eit = children.constEnd(); it != eit; ++it) {
         auto oit = other.children.constFind(it.key());
         if (oit == other.children.constEnd()) {
+            qDebug() << "6" << name << "!=" << other.name;
             return false;
         } else if (!it.value().equals(oit.value(), compareWhat)) {
+            qDebug() << "7" << name << "!=" << other.name;
             return false;
         }
     }
@@ -547,6 +569,7 @@ void FakeMoveReply::respond()
 
 FakeGetReply::FakeGetReply(FileInfo &remoteRootFileInfo, QNetworkAccessManager::Operation op, const QNetworkRequest &request, QObject *parent)
     : FakeReply { parent }
+    , _range(parseRange(request))
 {
     setRequest(request);
     setUrl(request.url());
@@ -557,45 +580,89 @@ FakeGetReply::FakeGetReply(FileInfo &remoteRootFileInfo, QNetworkAccessManager::
     Q_ASSERT(!fileName.isEmpty());
     fileInfo = remoteRootFileInfo.find(fileName);
     if (!fileInfo) {
-        qDebug() << "meh;";
+        qDebug() << "Could not find file" << fileName << "on the remote";
+        state = State::FileNotFound;
     }
-    Q_ASSERT_X(fileInfo, Q_FUNC_INFO, "Could not find file on the remote");
     QMetaObject::invokeMethod(this, &FakeGetReply::respond, Qt::QueuedConnection);
 }
 
+std::pair<qint64, qint64> FakeGetReply::parseRange(const QNetworkRequest &request)
+{
+    if (request.hasRawHeader("Range")) {
+        const QString range = QString::fromUtf8(request.rawHeader("Range"));
+        const QRegularExpression bytesPattern(QStringLiteral(R"(bytes=(?P<start>\d+)-(?P<end>\d*))"));
+        const QRegularExpressionMatch match = bytesPattern.match(range);
+        if (match.hasMatch()) {
+            const int start = match.captured(QStringLiteral("start")).toInt();
+            const QString end = match.captured(QStringLiteral("end"));
+            if (end.isEmpty()) {
+                // until the end
+                return { start, -1 };
+            } else {
+                // to the end of the range
+                return { start, end.toLongLong() };
+            }
+        }
+    }
+    return { 0, 0 };
+}
+
 void FakeGetReply::respond()
 {
-    if (aborted) {
+    switch (state) {
+    case State::Aborted:
         setError(OperationCanceledError, QStringLiteral("Operation Canceled"));
         emit metaDataChanged();
-        emit finished();
-        return;
+        break;
+    case State::FileNotFound:
+        setAttribute(QNetworkRequest::HttpStatusCodeAttribute, 404);
+        setError(ContentNotFoundError, QStringLiteral("File Not Found"));
+        emit metaDataChanged();
+        break;
+    case State::Ok:
+        payload = fileInfo->contentChar;
+        if (_range.second != 0) {
+            if (_range.second == -1) {
+                size = fileInfo->contentSize - _range.first;
+            } else {
+                size = _range.second - _range.first;
+            }
+            setRawHeader("Content-Range", QByteArrayLiteral("bytes ") + QByteArray::number(_range.first) + '-');
+        } else {
+            size = fileInfo->contentSize;
+        }
+        setHeader(QNetworkRequest::ContentLengthHeader, size);
+        setAttribute(QNetworkRequest::HttpStatusCodeAttribute, 200);
+        setRawHeader("OC-ETag", fileInfo->etag);
+        setRawHeader("ETag", fileInfo->etag);
+        setRawHeader("OC-FileId", fileInfo->fileId);
+        setRawHeader("X-OC-Mtime", QByteArray::number(fileInfo->lastModifiedInSecondsUTC()));
+
+        emit metaDataChanged();
+        if (bytesAvailable()) {
+            emit readyRead();
+        }
     }
-    payload = fileInfo->contentChar;
-    size = fileInfo->contentSize;
-    setHeader(QNetworkRequest::ContentLengthHeader, size);
-    setAttribute(QNetworkRequest::HttpStatusCodeAttribute, 200);
-    setRawHeader("OC-ETag", fileInfo->etag);
-    setRawHeader("ETag", fileInfo->etag);
-    setRawHeader("OC-FileId", fileInfo->fileId);
-    setRawHeader("X-OC-Mtime", QByteArray::number(fileInfo->lastModifiedInSecondsUTC()));
-    emit metaDataChanged();
-    if (bytesAvailable())
-        emit readyRead();
     emit finished();
 }
 
 void FakeGetReply::abort()
 {
     setError(OperationCanceledError, QStringLiteral("Operation Canceled"));
-    aborted = true;
+    state = State::Aborted;
 }
 
 qint64 FakeGetReply::bytesAvailable() const
 {
-    if (aborted)
+    switch (state) {
+    case State::Ok:
+        return size + QIODevice::bytesAvailable();
+    case State::Aborted:
+        Q_FALLTHROUGH();
+    case State::FileNotFound:
         return 0;
-    return size + QIODevice::bytesAvailable();
+    }
+    Q_UNREACHABLE(); // Unreachable, but GCC on CentOS 7 does not understand that.
 }
 
 qint64 FakeGetReply::readData(char *data, qint64 maxlen)
@@ -606,74 +673,6 @@ qint64 FakeGetReply::readData(char *data, qint64 maxlen)
     return len;
 }
 
-FakeGetWithDataReply::FakeGetWithDataReply(FileInfo &remoteRootFileInfo, const QByteArray &data, QNetworkAccessManager::Operation op, const QNetworkRequest &request, QObject *parent)
-    : FakeReply { parent }
-{
-    setRequest(request);
-    setUrl(request.url());
-    setOperation(op);
-    open(QIODevice::ReadOnly);
-
-    Q_ASSERT(!data.isEmpty());
-    payload = data;
-    QString fileName = getFilePathFromUrl(request.url());
-    Q_ASSERT(!fileName.isEmpty());
-    fileInfo = remoteRootFileInfo.find(fileName);
-    QMetaObject::invokeMethod(this, "respond", Qt::QueuedConnection);
-
-    if (request.hasRawHeader("Range")) {
-        const QString range = QString::fromUtf8(request.rawHeader("Range"));
-        const QRegularExpression bytesPattern(QStringLiteral("bytes=(?<start>\\d+)-(?<end>\\d+)"));
-        const QRegularExpressionMatch match = bytesPattern.match(range);
-        if (match.hasMatch()) {
-            const int start = match.captured(QStringLiteral("start")).toInt();
-            const int end = match.captured(QStringLiteral("end")).toInt();
-            payload = payload.mid(start, end - start + 1);
-        }
-    }
-}
-
-void FakeGetWithDataReply::respond()
-{
-    if (aborted) {
-        setError(OperationCanceledError, QStringLiteral("Operation Canceled"));
-        emit metaDataChanged();
-        emit finished();
-        return;
-    }
-    setHeader(QNetworkRequest::ContentLengthHeader, payload.size());
-    setAttribute(QNetworkRequest::HttpStatusCodeAttribute, 200);
-    setRawHeader("OC-ETag", fileInfo->etag);
-    setRawHeader("ETag", fileInfo->etag);
-    setRawHeader("OC-FileId", fileInfo->fileId);
-    setRawHeader("X-OC-Mtime", QByteArray::number(fileInfo->lastModifiedInSecondsUTC()));
-    emit metaDataChanged();
-    if (bytesAvailable())
-        emit readyRead();
-    emit finished();
-}
-
-void FakeGetWithDataReply::abort()
-{
-    setError(OperationCanceledError, QStringLiteral("Operation Canceled"));
-    aborted = true;
-}
-
-qint64 FakeGetWithDataReply::bytesAvailable() const
-{
-    if (aborted)
-        return 0;
-    return payload.size() - offset + QIODevice::bytesAvailable();
-}
-
-qint64 FakeGetWithDataReply::readData(char *data, qint64 maxlen)
-{
-    qint64 len = std::min(payload.size() - offset, quint64(maxlen));
-    std::memcpy(data, payload.constData() + offset, len);
-    offset += len;
-    return len;
-}
-
 FakeChunkMoveReply::FakeChunkMoveReply(FileInfo &uploadsFileInfo, FileInfo &remoteRootFileInfo, QNetworkAccessManager::Operation op, const QNetworkRequest &request, QObject *parent)
     : FakeReply { parent }
 {
@@ -783,7 +782,7 @@ FakePayloadReply::FakePayloadReply(QNetworkAccessManager::Operation op, const QN
     setUrl(request.url());
     setOperation(op);
     open(QIODevice::ReadOnly);
-    QTimer::singleShot(10, this, &FakePayloadReply::respond);
+    QTimer::singleShot(10ms, this, &FakePayloadReply::respond);
 }
 
 void FakePayloadReply::respond()
@@ -827,7 +826,7 @@ void FakeErrorReply::respond()
     emit metaDataChanged();
     emit readyRead();
     // finishing can come strictly after readyRead was called
-    QTimer::singleShot(5, this, &FakeErrorReply::slotSetFinished);
+    QTimer::singleShot(5ms, this, &FakeErrorReply::slotSetFinished);
 }
 
 void FakeErrorReply::slotSetFinished()
@@ -868,13 +867,13 @@ void FakeHangingReply::abort()
     emit finished();
 }
 
-FakeQNAM::FakeQNAM(FileInfo initialRoot)
+FakeAM::FakeAM(FileInfo initialRoot)
     : _remoteRootFileInfo { std::move(initialRoot) }
 {
     setCookieJar(new OCC::CookieJar);
 }
 
-QNetworkReply *FakeQNAM::createRequest(QNetworkAccessManager::Operation op, const QNetworkRequest &request, QIODevice *outgoingData)
+QNetworkReply *FakeAM::createRequest(QNetworkAccessManager::Operation op, const QNetworkRequest &request, QIODevice *outgoingData)
 {
     QNetworkReply *reply = nullptr;
     auto newRequest = request;
@@ -916,47 +915,65 @@ QNetworkReply *FakeQNAM::createRequest(QNetworkAccessManager::Operation op, cons
             Q_UNREACHABLE();
         }
     }
+    // timeout would be handled by Qt
+    if (request.transferTimeout() != 0) {
+        QTimer::singleShot(request.transferTimeout(), reply, [reply] {
+            reply->abort();
+        });
+    }
     OCC::HttpLogger::logRequest(reply, op, outgoingData);
     return reply;
 }
 
-FakeFolder::FakeFolder(const FileInfo &fileTemplate, OCC::Vfs::Mode vfsMode)
+FakeFolder::FakeFolder(const FileInfo &fileTemplate, OCC::Vfs::Mode vfsMode, bool filesAreDehydrated)
     : _localModifier(_tempDir.path())
-    , _vfsMode(vfsMode)
 {
     // Needs to be done once
     OCC::SyncEngine::minimumFileAgeForUpload = std::chrono::milliseconds(0);
 
     QDir rootDir { _tempDir.path() };
     qDebug() << "FakeFolder operating on" << rootDir;
-    toDisk(rootDir, fileTemplate);
+    toDisk(rootDir, filesAreDehydrated ? FileInfo() : fileTemplate);
 
-    _fakeQnam = new FakeQNAM(fileTemplate);
-    _account = OCC::Account::create();
-    _account->setUrl(QUrl(QStringLiteral("http://admin:admin@localhost/owncloud")));
-    _account->setCredentials(new FakeCredentials { _fakeQnam });
-    _account->setDavDisplayName(QStringLiteral("fakename"));
-    _account->setServerVersion(QStringLiteral("10.0.0"));
+    _fakeAm = new FakeAM(fileTemplate);
+    _account = OCC::TestUtils::createDummyAccount();
+    _account->setCredentials(new FakeCredentials { _fakeAm });
 
     _journalDb.reset(new OCC::SyncJournalDb(localPath() + QStringLiteral(".sync_test.db")));
-    _syncEngine.reset(new OCC::SyncEngine(_account, localPath(), QString(), _journalDb.get()));
+    // TODO: davUrl
+
+    _syncEngine.reset(new OCC::SyncEngine(_account, _account->davUrl(), localPath(), QString(), _journalDb.get()));
+    _syncEngine->setSyncOptions(OCC::SyncOptions { QSharedPointer<OCC::Vfs>(OCC::VfsPluginManager::instance().createVfsFromPlugin(vfsMode).release()) });
+
     // Ignore temporary files from the download. (This is in the default exclude list, but we don't load it)
     _syncEngine->excludedFiles().addManualExclude(QStringLiteral("]*.~*"));
 
     // handle aboutToRemoveAllFiles with a timeout in case our test does not handle it
-    QObject::connect(_syncEngine.get(), &OCC::SyncEngine::aboutToRemoveAllFiles, _syncEngine.get(), [this](OCC::SyncFileItem::Direction, std::function<void(bool)> callback) {
-        QTimer::singleShot(1 * 1000, _syncEngine.get(), [callback] {
+    QObject::connect(_syncEngine.get(), &OCC::SyncEngine::aboutToRemoveAllFiles, _syncEngine.get(), [this](OCC::SyncFileItem::Direction, const std::function<void(bool)> &callback) {
+        QTimer::singleShot(1s, _syncEngine.get(), [callback] {
             callback(false);
         });
     });
 
-    // Ensure we have a valid VfsOff instance "running"
-    switchToVfs(_syncEngine->syncOptions()._vfs);
+    auto vfs = _syncEngine->syncOptions()._vfs;
+    if (vfsMode != vfs->mode()) {
+        vfs.reset(OCC::VfsPluginManager::instance().createVfsFromPlugin(vfsMode).release());
+        Q_ASSERT(vfs);
+    }
+
+    // Ensure we have a valid Vfs instance "running"
+    switchToVfs(vfs);
+
+    if (vfsMode != OCC::Vfs::Off) {
+        const auto pinState = filesAreDehydrated ? OCC::PinState::OnlineOnly : OCC::PinState::AlwaysLocal;
+        syncJournal().internalPinStates().setForPath("", pinState);
+        OC_ENFORCE(vfs->setPinState("", pinState));
+    }
 
     // A new folder will update the local file state database on first sync.
     // To have a state matching what users will encounter, we have to a sync
     // using an identical local/remote file tree first.
-    OC_ENFORCE(syncOnce());
+    OC_ENFORCE(syncOnce())
 }
 
 void FakeFolder::switchToVfs(QSharedPointer<OCC::Vfs> vfs)
@@ -969,18 +986,20 @@ void FakeFolder::switchToVfs(QSharedPointer<OCC::Vfs> vfs)
     opts._vfs = vfs;
     _syncEngine->setSyncOptions(opts);
 
-    OCC::VfsSetupParams vfsParams;
+    OCC::VfsSetupParams vfsParams(_account, _account->davUrl(), false);
     vfsParams.filesystemPath = localPath();
     vfsParams.remotePath = QLatin1Char('/');
-    vfsParams.account = _account;
     vfsParams.journal = _journalDb.get();
     vfsParams.providerName = QStringLiteral("OC-TEST");
     vfsParams.providerDisplayName = QStringLiteral("OC-TEST");
-    vfsParams.providerVersion = QStringLiteral("0.1");
+    vfsParams.providerVersion = QVersionNumber(0, 1, 0);
+    vfsParams.multipleAccountsRegistered = false;
     QObject::connect(_syncEngine.get(), &QObject::destroyed, vfs.data(), [vfs]() {
         vfs->stop();
         vfs->unregisterFolder();
     });
+    QObject::connect(&_syncEngine->syncFileStatusTracker(), &OCC::SyncFileStatusTracker::fileStatusChanged,
+        vfs.data(), &OCC::Vfs::fileStatusChanged);
 
     QObject::connect(vfs.get(), &OCC::Vfs::error, vfs.get(), [](const QString &error) {
         QFAIL(qUtf8Printable(error));
@@ -1014,7 +1033,7 @@ QString FakeFolder::localPath() const
 void FakeFolder::scheduleSync()
 {
     // Have to be done async, else, an error before exec() does not terminate the event loop.
-    QMetaObject::invokeMethod(_syncEngine.get(), "startSync", Qt::QueuedConnection);
+    QMetaObject::invokeMethod(_syncEngine.get(), &OCC::SyncEngine::startSync, Qt::QueuedConnection);
 }
 
 void FakeFolder::execUntilBeforePropagation()
@@ -1042,7 +1061,12 @@ void FakeFolder::execUntilItemCompleted(const QString &relativePath)
 
 bool FakeFolder::isDehydratedPlaceholder(const QString &filePath)
 {
-    return _syncEngine->syncOptions()._vfs->isDehydratedPlaceholder(filePath);
+    return vfs()->isDehydratedPlaceholder(filePath);
+}
+
+QSharedPointer<OCC::Vfs> FakeFolder::vfs() const
+{
+    return _syncEngine->syncOptions()._vfs;
 }
 
 void FakeFolder::toDisk(QDir &dir, const FileInfo &templateFi)
@@ -1067,6 +1091,11 @@ void FakeFolder::fromDisk(QDir &dir, FileInfo &templateFi)
 {
     const auto infoList = dir.entryInfoList(QDir::AllEntries | QDir::NoDotAndDotDot);
     for (const auto &diskChild : infoList) {
+        if (diskChild.isHidden() || diskChild.fileName().startsWith(QStringLiteral(".sync_"))) {
+            // Skip system files, sqlite db files, sync log, etc.
+            continue;
+        }
+
         if (diskChild.isDir()) {
             QDir subDir = dir;
             subDir.cd(diskChild.fileName());
@@ -1081,15 +1110,17 @@ void FakeFolder::fromDisk(QDir &dir, FileInfo &templateFi)
             fi.setLastModified(diskChild.lastModified());
             if (fi.isDehydratedPlaceholder) {
                 fi.contentChar = '\0';
+                fi.contentSize = 0;
             } else {
                 QFile f { diskChild.filePath() };
-                f.open(QFile::ReadOnly);
+                OC_ENFORCE(f.open(QFile::ReadOnly));
                 auto content = f.read(1);
                 if (content.size() == 0) {
                     qWarning() << "Empty file at:" << diskChild.filePath();
-                    continue;
+                    fi.contentChar = FileInfo::DefaultContentChar;
+                } else {
+                    fi.contentChar = content.at(0);
                 }
-                fi.contentChar = content.at(0);
                 fi.contentSize = fi.fileSize;
             }
 
@@ -1098,7 +1129,7 @@ void FakeFolder::fromDisk(QDir &dir, FileInfo &templateFi)
     }
 }
 
-FileInfo &findOrCreateDirs(FileInfo &base, PathComponents components)
+FileInfo &findOrCreateDirs(FileInfo &base, const PathComponents &components)
 {
     if (components.isEmpty())
         return base;
